<?php

/**
 * SHD SM-VIEW Cable Administration Unit API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_admin_unit extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'admin_unit';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_admin_unit::instance(isys_application::instance()->database);
    }

    /**
     * Create admin unit.
     *
     * $p_param structure:
     * array (
     *     'description'    => string,
     *     'classification' => string,
     *     'parent_id'      => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'description',
            'classification',
        );
        $optional_params = array(
            'parent_id',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update admin unit.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'             => int,
     *     'description'    => string,
     *     'classification' => string,
     *     'parent_id'      => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'description',
            'classification',
        );
        $ref_params = array(
            'parent_id',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters, $ref_params);
    }

    /**
     * Delete admin unit.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
