<?php

/**
 * SHD SM-VIEW Cable API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_cable extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'cable';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable::instance(isys_application::instance()->database);
    }

    /**
     * Create cable.
     *
     * $p_param structure:
     * array (
     *     'obj_id'             => int,
     *     'state_id'           => int,
     *     'user_state'         => string,
     *     'group_id'           => int,
     *     'length'             => int,
     *     'capacity'           => int,
     *     'net_type'           => string,
     *     'admin_unit_id'      => int,
     *     'type_id'            => int,
     *     'location_in_id'     => int,
     *     'location_out_id'    => int,
     *     'is_section_cable'   => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'obj_id',
        );
        $optional_params = array(
            'state_id',
            'user_state',
            'group_id',
            'length',
            'type_id',
            'capacity',
            'net_type',
            'admin_unit_id',
            'location_in_id',
            'location_out_id',
            'is_section_cable',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update cable.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'cable_id'           => int,
     *     'state_id'           => int,
     *     'user_state'         => string,
     *     'group_id'           => int,
     *     'length'             => int,
     *     'capacity'           => int,
     *     'net_type'           => string,
     *     'admin_unit_id'      => int,
     *     'type_id'            => int,
     *     'location_in_id'     => int,
     *     'location_out_id'    => int,
     *     'is_section_cable'   => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'user_state',
            'length',
            'capacity',
            'net_type',
        );
        $ref_params = array(
            'state_id',
            'group_id',
            'type_id',
            'admin_unit_id',
            'location_in_id',
            'location_out_id',
            'is_section_cable',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters, $ref_params);
    }

    /**
     * Find usages of an element.
     *
     * @param $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int
     * )
     *
     * @return array
     * @throws isys_exception_database
     */
    public function find_usages($p_params)
    {
        $this->check_permission_of_category(isys_auth::VIEW, $p_params['id']);
        $cable_catg = new isys_cmdb_dao_category_g_smview_cable(isys_application::instance()->database);
        return $cable_catg->find_usages($p_params['id']);
    }

    /**
     * Delete cable.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        // check if a wire is in use by a section or a route
        $cable = $this->dao->get_object('id', $p_params['id']);
        $wire_dao = isys_cmdb_dao_smview_cable_wire::instance(isys_application::instance()->database);
        $wires = $wire_dao->filter_objects(array('obj_id' => $cable['obj_id']));

        $language = isys_application::instance()->container->get('language');
        $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire(isys_application::instance()->database);
        foreach ($wires as $wire) {
            $in_use = $wire_catg->get_in_use($wire['id']);
            if (! empty($in_use) && $in_use == $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED')) {
                return $this->return_error($this->module_name, 'delete', 'can not delete cable because its wires are in use by a section or a route');
            }
        }
        return $this->_delete('id', $p_params);
    }
}
