<?php

/**
 * SHD SM-VIEW Cable Connection Type API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_connection_type extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'connection_type';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_connection_type::instance(isys_application::instance()->database);
    }

    /**
     * Create connection type.
     *
     * $p_param structure:
     * array (
     *     'name'        => string,
     *     'count'       => int,
     *     'description' => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'name',
        );
        $optional_params = array(
            'count',
            'description',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update connection type.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'connection_type_id' => int,
     *     'name'               => string,
     *     'count'              => int,
     *     'description'        => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'name',
            'count',
            'description',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete connection type.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'connection_type_id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
