<?php

/**
 * SHD SM-VIEW Cable Generator API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_generator extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'generator';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_generator::instance(isys_application::instance()->database);
    }

    /**
     * Create generator.
     *
     * $p_param structure:
     * array (
     *     'title'       => string,
     *     'mask'        => string,
     *     'description' => string,
     *     'counter_min' => string,
     *     'counter_max' => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'title',
            'mask',
            'counter_min',
            'counter_max',
        );
        $optional_params = array(
            'description',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update generator.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'          => int,
     *     'title'       => string,
     *     'mask'        => string,
     *     'description' => string,
     *     'counter_min' => string,
     *     'counter_max' => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'title',
            'mask',
            'description',
            'counter_min',
            'counter_max',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }
    
    /*
     * Generate names by generator.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     * @return array
     */
    public function generate($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $required_params = array(
            'id',
            'start_pattern',
        );
        try {
            $this->check_required_parameters($required_params, $p_params['data']);
        } catch (\isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'create', 'could not generate names - ' . $ex->getMessage());
        }
        
        $data = $p_params['data'];
        $id = $data['id'];
        $start = $data['start_pattern'];
        $end = !empty($data['end_pattern']) ? $data['end_pattern'] : null;
        $max_count = !empty($data['max_count']) ? $data['max_count'] : null;
        
        $name_generator = new \idoit\Module\Cable\LabelGenerator($id);
        $names = $name_generator->generate_names_by_generator($start, $end, $max_count);
        return $names;
    }
    
    /*
     * Count generated names by generator.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     * @return array
     */
    public function count($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::VIEW, strtolower($this->category_name));
        }

        $names = $this->generate($p_params);
        if (isset($names['error'])) {
            // leave error message
            return $names;
        }
        $count = count($names);
        $last = array_pop($names);
        return [
            'count'       => $count,
            'end_element' => $last,
        ];
    }

    /**
     * Delete generator.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
