<?php

/**
 * SHD SM-VIEW Cable Generator Algorithm API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_generator_algorithm extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'generator_algorithm';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_generator_algorithm::instance(isys_application::instance()->database);
    }

    /**
     * Create generator algorithm.
     *
     * $p_param structure:
     * array (
     *     'generator_id'  => int,
     *     'index'         => int,
     *     'mask_position' => int,
     *     'type'          => string,
     *     'description'   => string,
     *     'length'        => int,
     *     'increment'     => int,
     *     'elements'      => string,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'generator_id',
            'index',
            'mask_position',
        );
        $optional_params = array(
            'type',
            'description',
            'length',
            'increment',
            'elements',
        );
        $data = $p_params['data'];
        $this->_validate_index($data);
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update generator algorithm.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'            => int,
     *     'index'         => int,
     *     'mask_position' => int,
     *     'type'          => string,
     *     'description'   => string,
     *     'length'        => int,
     *     'increment'     => int,
     *     'elements'      => string,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'index',
            'mask_position',
            'type',
            'description',
            'length',
            'increment',
            'elements',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        $this->_validate_index($data);
        return $this->_update($data, $id, $parameters);
    }
    
    /**
     * Validate index.
     *
     * @param  array $p_params
     * @return null|array
     */
    private function _validate_index($p_params)
    {
        if (! isset($p_params['index'])) {
            return;
        }
        
        $index = $p_params['index'];
        if (empty($index)) {
            throw new \isys_exception_api('index is invalid');
        }
    }

    /**
     * Delete generator algorithm.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
