<?php

use idoit\Module\Cable\ObjectTitleValidator;

/**
 * SHD SM-VIEW Cable Object Title Check API.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_object_title_check implements isys_api_model_interface
{

    /**
     * Read settings.
     *
     * @param  array $p_params
     * @return array
     */
    public function read($p_params)
    {
        $check_enabled = isys_settings::get(isys_module_smview_cable::OBJECT_TITLE_CHECK_ENABLED, null);
        $check_object_types = isys_settings::get(isys_module_smview_cable::OBJECT_TITLE_CHECK_OBJECT_TYPES, null);
        $check_properties = isys_settings::get(isys_module_smview_cable::OBJECT_TITLE_CHECK_PROPERTIES, null);

        return [
            'enabled'      => $check_enabled == 1,
            'object_types' => explode(',', $check_object_types),
            'properties'   => explode(',', $check_properties),
        ];
    }

    /**
     * Validate property of an object.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     */
    public function validate($p_params)
    {
        if (empty($p_params['properties'])) {
            throw new isys_exception_api("Parameter 'properties' is missing");
        }

        $p_properties = $p_params['properties'];
        if (empty($p_properties['title'])) {
            throw new isys_exception_api("Parameter 'title' is missing within properties");
        }

        $p_obj_id = $p_params['obj_id'] ?? null;
        $p_obj_type_id = $p_params['obj_type_id'] ?? null;
        return ObjectTitleValidator::validate($p_obj_id, $p_obj_type_id, $p_properties);
    }
}
