<?php

/**
 * SHD SM-VIEW Cable Railway Network API
 *
 * @package    idoit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2024 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class cable_api_model_railway_network extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'railway_network';

    /**
     * Read method.
     *
     * @param array $p_params
     * @return array
     */
    public function read($p_params)
    {
        return $this->return_error($this->module_name, __METHOD__, 'method not implemented');
    }

    /**
     * Get capacity color settings of railway network.
     *
     * @return array
     */
    public function get_capacity_color_settings(): array
    {
        return isys_module_smview_cable::get_capacity_color_settings();
    }

    /**
     * Set capacity color settings of railway network.
     *
     * @param array $p_params
     * @return array
     * @throws isys_exception_validation
     */
    public function set_capacity_color_settings(array $p_params): array
    {
        $settings = $p_params['settings'] ?? [];
        isys_module_smview_cable::set_capacity_color_settings($settings);
        return $this->return_success($this->module_name, __METHOD__, null);
    }

    /**
     * Get all railway line colors.
     *
     * @return array
     */
    public function get_line_colors(): array
    {
        return isys_module_smview_cable::get_line_colors();
    }

    /**
     * Set color of a railway line.
     *
     * @param array $p_params
     * @return array
     * @throws isys_exception_validation
     */
    public function set_line_color(array $p_params): array
    {
        $this->check_required_parameters(['line', 'color'], $p_params);
        isys_module_smview_cable::set_line_color($p_params['line'], $p_params['color']);
        return $this->return_success($this->module_name, __METHOD__, null);
    }

    /**
     * Create method.
     *
     * @param  array $p_params
     * @return array
     */
    public function create($p_params)
    {
        return $this->return_error($this->module_name, __METHOD__, 'method not implemented');
    }

    /**
     * Update method.
     *
     * @param  array $p_params
     * @return array
     */
    public function update($p_params)
    {
        return $this->return_error($this->module_name, __METHOD__, 'method not implemented');
    }

    /**
     * Delete method.
     *
     * @param array $p_params
     * @return array
     */
    public function delete($p_params)
    {
        return $this->return_error($this->module_name, __METHOD__, 'method not implemented');
    }
}
