<?php

/**
 * SHD SM-VIEW Cable Route Usage API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_route_usage extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'route_usage';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE_ROUTE';

    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_route_usage::instance(isys_application::instance()->database);
    }

    /**
     * Create route usage.
     *
     * $p_param structure:
     * array (
     *     'name'        => string,
     *     'description' => string,
     *     'can_be_rented' => boolean,
     *     'technology_id' => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'name',
        );
        $optional_params = array(
            'description',
            'can_be_rented',
            'technology_id',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters, $optional_params);
    }

    /**
     * Update route usage.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'          => int,
     *     'name'        => string,
     *     'description' => string,
     *     'can_be_rented' => boolean,
     *     'technology_id' => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'name',
            'description',
            'can_be_rented',
            'technology_id',
        );

        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters);
    }

    /**
     * Delete route usage.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int,
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
