<?php

/**
 * SHD SM-VIEW Cable Type API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_type extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'type';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_type::instance(isys_application::instance()->database);
    }

    /**
     * Create type.
     *
     * $p_param structure:
     * array (
     *     'description'   => string,
     *     'wire_count'    => int,
     *     'technology_id' => int,
     * )
     *
     * @return array
     */
    public function create($p_params)
    {
        $required_parameters = array (
            'description',
            'wire_count',
            'technology_id',
        );
        $data = $p_params['data'];
        return $this->_create($data, $required_parameters);
    }

    /**
     * Update type.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'id'            => int,
     *     'description'   => string,
     *     'wire_count'    => int,
     *     'technology_id' => int,
     * )
     * @return array
     */
    public function update($p_params)
    {
        $parameters = array (
            'description',
            'wire_count',
        );
        $ref_params = array (
            'technology_id',
        );
        
        $id = $p_params['id'];
        $data = $p_params['data'];
        return $this->_update($data, $id, $parameters, $ref_params);
    }

    /**
     * Delete type.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array (
     *     'id' => int
     * )
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->_delete('id', $p_params);
    }
}
