<?php

/**
 * SHD SM-VIEW Cable Wire Pin API
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class cable_api_model_wire_pin extends cable_api_model_abstract
{

    /**
     * Name of this module.
     *
     * @var string
     */
    protected $module_name = 'wire_pin';

    /**
     * Name of this category.
     *
     * @var string
     */
    protected $category_name = 'C__CATG__SMVIEW_CABLE';
    
    /**
     * Constructor.
     */
    function __construct()
    {
        parent::__construct();
        $this->dao = isys_cmdb_dao_smview_cable_wire_pin::instance(isys_application::instance()->database);
    }

    /**
     * Create wire pin.
     *
     * $p_param structure:
     * array ()
     *
     * @return array
     */
    public function create($p_params)
    {
        return $this->return_error($this->module_name, 'create', 'wire pins will be created automatically');
    }

    /**
     * Update wire pin.
     *
     * @param  array $p_params
     *
     * $p_param structure:
     * array (
     *     'wire_pin_id'      => int,
     *     'description'      => string,
     *     'state'            => string, // obsolete
     *     'status_id'        => int,
     *     'connected_pin_id' => int,
     * )
     * @return array
     * @throws Exception
     */
    public function update($p_params)
    {
        $id = $p_params['id'];
        $data = $p_params['data'];
        $parameters = array (
            'description',
            'status_id',
        );
        $ref_params = array(
            'connected_pin_id'
        );

        // handle old state parameter for compatibility reasons
        if (isset($data['state']) && ! isset($data['status_id'])) {
            $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
            $data['status_id'] = array_search($data['state'], $states);
        }
        
        if (empty($data['connected_pin_id'])) {
            return $this->_update($data, $id, $parameters, $ref_params);
        }
        
        // check if a wire is in use by a section or a route
        if (empty($data['ignore_in_use'])) {
            $wire_pin = $this->dao->get_object('id', $id);
            $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire(isys_application::instance()->database);
            $language = isys_application::instance()->container->get('language');
            $in_use = $wire_catg->get_in_use($wire_pin['wire_id']);
            if (! empty($in_use) && $in_use == $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED')) {
                // do not update if connected pin has changed
                return $this->return_error($this->module_name, 'update', 'can not update wire pin because it is in use by a section or a route');
            }
        }
        return $this->_update($data, $id, $parameters, $ref_params);
    }

    /**
     * Move a bunch of wires to new pins.
     *
     * @param $p_params
     * @return array
     */
    public function batch_reconnect_pins($p_params)
    {
        if ($this->useAuth && ! empty($this->category_name)) {
            isys_auth_cmdb::instance()->category(isys_auth::EDIT, strtolower($this->category_name));
        }

        $data = $p_params['data'];
        try {
            $this->check_required_parameters(['mapping'], $data);
        } catch (\isys_exception_validation $ex) {
            return $this->return_error($this->module_name, 'batch_reconnect_pins', 'mapping does not exist');
        }

        $mapping = $data['mapping'];
        $dao = new \isys_cmdb_dao_smview_cable_wire_pin(isys_application::instance()->database);
        try {
            $dao->reconnect_wires($mapping);
        } catch (Exception $e) {
            return $this->return_error($this->module_name, 'batch_reconnect_pins', 'can not map wires: ' . $e->getMessage());
        }

        return $this->return_success($this->module_name, 'batch_reconnect_pins', null);
    }

    /**
     * Delete wire pin.
     *
     * @param array $p_params
     *
     * $p_param structure:
     * array ()
     *
     * @return array
     */
    public function delete($p_params)
    {
        return $this->return_error($this->module_name, 'delete', 'wire pins will be deleted automatically');
    }
}
