import * as React from 'react'
import { BayItemCell } from '../BayItemCell'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_CABLE__FIBRE_WIRE: 'Fibre',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY_PLANNED_ITEM: 'Planned Item',
}
const mockFunction = jest.fn()
const wireBundles = [
  {
    cable_name: 'Cable01',
    cable_url: 'url',
    wire_start: 1,
    wire_end: 14,
    wire_state: '',
    device_colors: {},
    sibling_object_title: 'Object01',
    sibling_pin_scope: 'B1',
    sibling_object_url: 'url'
  }
]
const bayItem = {
  id: '1',
  title: 'Bu01',
  position: 1,
  color: 1,
  short_name: 'Bu01',
  he: 1,
  comment: 'Test',
  wire_bundles: wireBundles
}
const colorLayouts = [
  {
    id: 1,
    title: 'Spleißbox',
    layout: 3,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  },
  {
    id: 2,
    title: 'Pigtail-Box',
    layout: 2,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  },
  {
    id: 3,
    title: 'Einzelspleiß',
    layout: 1,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  }
]

it('renders unselected bay item correctly', () => {
  const item = bayItem
  item.color = 1
  const component = ReactTestRenderer.create(
    <BayItemCell language={lang} is_selected={false} bay_item={item} onSelectionChanged={mockFunction} colorLayouts={colorLayouts} />
  )
  expect(component).toMatchSnapshot()
})

it('renders selected bay item one correctly', () => {
  const item = bayItem
  item.color = 1
  const component = ReactTestRenderer.create(
    <BayItemCell language={lang} is_selected={true} bay_item={item} onSelectionChanged={mockFunction} colorLayouts={colorLayouts} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const div = instance.findByProps({ id: '1' })
  div.props.onClick()
  expect(mockFunction).toHaveBeenCalledWith('1')
})

it('renders selected bay item two correctly', () => {
  const item = bayItem
  item.color = 2
  const component = ReactTestRenderer.create(
    <BayItemCell language={lang} is_selected={true} bay_item={item} onSelectionChanged={mockFunction} colorLayouts={colorLayouts} />
  )
  expect(component).toMatchSnapshot()
})

it('renders selected bay item three correctly', () => {
  const item = bayItem
  item.color = 3
  const component = ReactTestRenderer.create(
    <BayItemCell language={lang} is_selected={true} bay_item={item} onSelectionChanged={mockFunction} colorLayouts={colorLayouts} />
  )
  expect(component).toMatchSnapshot()
})

it('renders multiple bundle bay item correctly', () => {
  const wireBundles = [
    {
      cable_name: 'Cable01',
      cable_url: 'url',
      wire_start: 1,
      wire_end: 14,
      wire_state: '',
      device_colors: {},
      sibling_object_title: 'Object01',
      sibling_pin_scope: 'B1',
      sibling_object_url: 'url'
    },
    {
      cable_name: 'Cable02',
      cable_url: 'url',
      wire_start: 5,
      wire_end: 6,
      wire_state: '',
      device_colors: {},
      sibling_object_title: 'Object02',
      sibling_pin_scope: 'B4',
      sibling_object_url: 'url'
    }
  ]
  const item = {
    id: '1',
    title: 'Bu01',
    position: 1,
    color: 1,
    short_name: 'Bu01',
    he: 1,
    comment: 'Test',
    wire_bundles: wireBundles
  }
  const component = ReactTestRenderer.create(
      <BayItemCell language={lang} is_selected={true} bay_item={item} onSelectionChanged={mockFunction} colorLayouts={colorLayouts} />
  )
  expect(component).toMatchSnapshot()
})
