import * as React from 'react'
import { BayItemProperties } from '../BayItemProperties'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_NOT_EMPTY: 'Value should not be empty',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_NOT_BETWEEN: 'Value is not between',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_ALREADY_ASSIGNED: 'HE is already assigned',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_HE_IN_USE: 'Some HEs are already assigned within this range',
  LC__SHD__TELECOMMUNICATION_CABLE: 'Cable',
  LC__CMDB__CATG__SMVIEW_CABLE__FIBRE_WIRE: 'Fibre Wire',
  LC__CMDB__CATG__SMVIEW_CABLE__CONNECTED_WITH: 'Connected with',
  LC__CMDB__CATG__SMVIEW_CABLE__SUGGESTED_COLOR_NOT_MATCH: 'Selected device color does not match with suggestion (anymore)',
  LC__CMDB__CATG__SMVIEW_CABLE__NO_COLOR_SUGGESTED: 'Found no matching suggestion',
  LC__CMDB__CATG__SMVIEW_CABLE__SUGGESTED_COLORS: 'Suggested Colors',
  LC__CMDB__CATG__SMVIEW_CABLE__SHORT_NAME: 'Short Name',
  LC__CMDB__CATG__SMVIEW_CABLE__WIRE_BUNDLES: 'Wire Bundles',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR: 'ETSI Device Color',
  LC__CMDB__CATG__SMVIEW_CABLE__HE: 'HE',
  LC__CMDB__CATG__SMVIEW_CABLE__POSITION: 'Position',
  LC__CMDB__CATG__SMVIEW_CABLE__PROJECT_IDENTIFIER: 'Project Identifier',
  LC__CMDB__CATG__SMVIEW_CABLE__COMMENT: 'Comment'
}
const mockFunction = jest.fn()
const wireBundle = [
  {
    cable_name: 'Cable01',
    cable_url: 'url',
    wire_start: 1,
    wire_end: 12,
    wire_state: '',
    device_colors: {},
    sibling_object_title: 'Object01',
    sibling_pin_scope: 'B1',
    sibling_object_url: 'url'
  }
]
const wireBundles = [
  {
    cable_name: 'Cable01',
    cable_url: 'url',
    wire_start: 1,
    wire_end: 12,
    wire_state: '',
    device_colors: {
      1: '12er Spleißbox mit Pigtails',
      2: 'Spleißbox Koppler 12'
    },
    sibling_object_title: 'Object01',
    sibling_pin_scope: 'B1',
    sibling_object_url: 'url'
  },
  {
    cable_name: 'Cable02',
    cable_url: 'url',
    wire_start: 13,
    wire_end: 24,
    wire_state: 'E2000',
    device_colors: {
      1: '12er Spleißbox mit Pigtails',
      2: 'Spleißbox Koppler 12'
    },
    sibling_object_title: 'Object02',
    sibling_pin_scope: 'B1',
    sibling_object_url: 'url'
  }
]
const bayItem = {
  id: '1',
  title: 'Bu01',
  position: 2,
  color: 1,
  short_name: 'Bu01',
  he: 1,
  comment: 'Test',
  project_identifier: '',
  wire_bundles: wireBundle
}
const colorLayouts = [
  {
    id: 1,
    title: 'Spleißbox',
    layout: 3,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  },
  {
    id: 2,
    title: 'Pigtail-Box',
    layout: 2,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  },
  {
    id: 3,
    title: 'Einzelspleiß',
    layout: 1,
    color_left_top: '#112233',
    color_left_bottom: '#AA00FF',
    color_right_top: '#6699AA',
    color_right_bottom: '#AACCFF'
  }
]

it('renders readonly bay item properties correctly', () => {
  const component = ReactTestRenderer.create(
    <BayItemProperties language={lang} isEditable={false} bayItem={bayItem} onChangedBayItem={mockFunction} colorLayouts={colorLayouts} bayHe={4} assignedHes={[2]} />
  )
  expect(component).toMatchSnapshot()
})

it('renders simple bay item properties correctly', () => {
  const component = ReactTestRenderer.create(
    <BayItemProperties language={lang} isEditable={true} bayItem={bayItem} onChangedBayItem={mockFunction} colorLayouts={colorLayouts} bayHe={4} assignedHes={[2]} />
  )
  expect(component).toMatchSnapshot()
})

it('renders bay item properties correctly', () => {
  const item = bayItem
  item.he = 2
  const component = ReactTestRenderer.create(
    <BayItemProperties language={lang} isEditable={true} bayItem={item} onChangedBayItem={mockFunction} colorLayouts={colorLayouts} bayHe={4} assignedHes={[2]} />
  )
  expect(component).toMatchSnapshot()

  // change position
  const instance = component.root
  let input = instance.findByProps({ name: 'position' })
  input.props.onChange({
    currentTarget: { name: 'position', value: '3' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 1, comment: 'Test', he: 2, id: '1', position: 3, project_identifier: '', short_name: 'Bu01', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // do not change position
  input.props.onChange({
    currentTarget: { name: 'position', value: '8' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 1, comment: 'Test', he: 2, id: '1', position: 3, project_identifier: '', short_name: 'Bu01', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // change he
  input = instance.findByProps({ name: 'he' })
  input.props.onChange({
    currentTarget: { name: 'he', value: '1' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 1, comment: 'Test', he: 1, id: '1', position: 3, project_identifier: '', short_name: 'Bu01', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // do not change he
  input = instance.findByProps({ name: 'he' })
  input.props.onChange({
    currentTarget: { name: 'he', value: '3' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 1, comment: 'Test', he: 1, id: '1', position: 3, project_identifier: '', short_name: 'Bu01', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // change short name
  input = instance.findByProps({ name: 'short_name' })
  input.props.onChange({
    currentTarget: { name: 'short_name', value: 'Bu11' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 1, comment: 'Test', he: 1, id: '1', position: 3, project_identifier: '', short_name: 'Bu11', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // change color
  input = instance.findByType('select')
  input.props.onChange({
    currentTarget: { value: '2' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 2, comment: 'Test', he: 1, id: '1', position: 3, project_identifier: '', short_name: 'Bu11', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })

  // change comment
  input = instance.findByProps({ name: 'comment' })
  input.props.onChange({
    currentTarget: { name: 'comment', value: 'New Comment' }
  })
  expect(mockFunction).toHaveBeenCalledWith({ color: 2, comment: 'New Comment', he: 1, id: '1', position: 3, project_identifier: '', short_name: 'Bu11', title: 'Bu01', wire_bundles: [{ cable_name: 'Cable01', cable_url: 'url', device_colors: {}, sibling_object_title: 'Object01', sibling_object_url: 'url', sibling_pin_scope: 'B1', wire_end: 12, wire_start: 1, wire_state: '' }] })
})

it('renders bay item properties with project identifier correctly', () => {
  const item = bayItem
  item.project_identifier = 'LISI'
  const component = ReactTestRenderer.create(
    <BayItemProperties language={lang} isEditable={true} bayItem={item} onChangedBayItem={mockFunction} colorLayouts={colorLayouts} bayHe={4} assignedHes={[2]} />
  )
  expect(component).toMatchSnapshot()
})

it('renders bay item properties with multiple wire bundles correctly', () => {
  const item = bayItem
  item.wire_bundles = wireBundles
  const component = ReactTestRenderer.create(
    <BayItemProperties language={lang} isEditable={true} bayItem={item} onChangedBayItem={mockFunction} colorLayouts={colorLayouts} bayHe={4} assignedHes={[1]} />
  )
  expect(component).toMatchSnapshot()
})
