import * as React from 'react'

import { ColorTable } from './ColorTable'
import { WireStatus } from './WireStatus'
import { Layout } from './Layout'

export interface ColorProps {
    layout: string,
    title_left_top: string,
    color_left_top: string,
    title_left_bottom: string,
    color_left_bottom: string,
    title_right_top: string,
    color_right_top: string,
    title_right_bottom: string,
    color_right_bottom: string,
    wire_states: WireState[],
}

export interface WireState {
  id: number,
  wire_pin_status_id: number,
  operator: string,
  not: boolean,
}

export interface WirePinStatus {
  id: number,
  const: string,
}

export interface ColorComposeProps {
    language: {[key: string]: string},
    is_editable: boolean,
    data: ColorProps,
    wire_pin_status: WirePinStatus[],
}

export class ColorCompose extends React.Component<ColorComposeProps, ColorProps> {
  constructor (props: ColorComposeProps) {
    super(props)
    this.state = {
      ...this.props.data,
      wire_states: this.props.data.wire_states ? this.props.data.wire_states : []
    }

    this.handleDataChange = this.handleDataChange.bind(this)
    this.addWireState = this.addWireState.bind(this)
    this.removeWireState = this.removeWireState.bind(this)
    this.handleOperatorSelection = this.handleOperatorSelection.bind(this)
    this.updateWireState = this.updateWireState.bind(this)
    this._updateWireStates = this._updateWireStates.bind(this)
  }

  handleDataChange (property_name: string, value: string) {
    this.setState((prevState) => ({
      ...prevState,
      [property_name]: value
    }), () => {
      this._saveToDom(this.state)
    })
  }

  addWireState() {
    let states = this.state.wire_states
    let nextId = 0
    let operator = 'AND'
    states.forEach(s => {
      operator = s.operator
      if (s.id > nextId) {
        nextId = parseInt(s.id.toString())
      }
    })
    nextId++

    states.push({
      id: nextId,
      wire_pin_status_id: 0,
      operator: operator,
      not: false
    })
    this._updateWireStates(states)
  }

  removeWireState (id: number) {
    const states = this.state.wire_states.filter(w => w.id != id)
    this._updateWireStates(states)
  }

  updateWireState (e: React.FormEvent) {
    const id = parseInt(e.currentTarget.parentElement.parentElement.id)
    let states = this.state.wire_states.filter(w => w.id == id)
    let state = states[0]
    if (e.currentTarget instanceof HTMLSelectElement) {
      state.wire_pin_status_id = parseInt(e.currentTarget.value)
    } else if (e.currentTarget instanceof HTMLInputElement) {
      state.not = e.currentTarget.checked
    }

    states = this.state.wire_states.filter(w => w.id != id)
    states.push(state)
    this._updateWireStates(states)
  }

  handleOperatorSelection (operator: string) {
    const states: WireState[] = this.state.wire_states.map(w => {
      w.operator = operator
      return w
    })
    this._updateWireStates(states)
  }

  _updateWireStates (states: WireState[]) {
    this.setState((prevState) => ({
      ...prevState,
      wire_states: states
    }), () => {
      this._saveToDom(this.state)
    })
  }

  _saveToDom (data: ColorProps) {
    // save json to DOM
    const json = JSON.stringify(data)
    document.getElementById('C__CATG__SMVIEW_CABLE_ETSI_DEVICE_COLOR').setAttribute('value', json)
  }

  render () {
    const wireStatus = <WireStatus language={this.props.language}
      is_editable={this.props.is_editable}
      wire_states={this.state.wire_states}
      wire_pin_status={this.props.wire_pin_status}
      onAddWireState={this.addWireState}
      onRemoveWireState={this.removeWireState}
      onUpdateWireState={this.updateWireState}
      onOperatorSelection={this.handleOperatorSelection} />

    return <div>
      {wireStatus}

      <Layout is_editable={this.props.is_editable}
        layout={this.state.layout}
        onLayoutChange={this.handleDataChange} />

      <ColorTable language={this.props.language}
        is_editable={this.props.is_editable}
        data={this.state}
        onDataChange={this.handleDataChange} />
    </div>
  }
}
