import * as React from 'react'
import { ColorProps } from './ColorCompose'

interface ColorTableProps {
    language: {[key: string]: string},
    is_editable: boolean,
    data: ColorProps;
    onDataChange: Function,
}

export class ColorTable extends React.Component<ColorTableProps, {}> {
  constructor (props: ColorTableProps) {
    super(props)
    this.handleValueChange = this.handleValueChange.bind(this)
  }

  handleValueChange (property_name: string, value: string) {
    this.props.onDataChange(property_name, value)
  }

  render () {
    const layout = parseInt(this.props.data.layout)
    const left_top_cell = <Cell language={this.props.language}
      is_editable={this.props.is_editable}
      suffix='_left_top'
      title={this.props.data.title_left_top}
      onChangeTitle={this.handleValueChange}
      color={this.props.data.color_left_top}
      onChangeColor={this.handleValueChange} />
    let right_top_cell = null
    if (layout >= 2) {
      right_top_cell = <Cell language={this.props.language}
        is_editable={this.props.is_editable}
        suffix='_right_top'
        title={this.props.data.title_right_top}
        onChangeTitle={this.handleValueChange}
        color={this.props.data.color_right_top}
        onChangeColor={this.handleValueChange} />
    }
    let right_bottom_cell = null
    if (layout >= 3) {
      right_bottom_cell = <Cell language={this.props.language}
        is_editable={this.props.is_editable}
        suffix='_right_bottom'
        title={this.props.data.title_right_bottom}
        onChangeTitle={this.handleValueChange}
        color={this.props.data.color_right_bottom}
        onChangeColor={this.handleValueChange} />
    }
    let left_bottom_cell = null
    if (layout >= 4) {
      left_bottom_cell = <Cell language={this.props.language}
        is_editable={this.props.is_editable}
        suffix='_left_bottom'
        title={this.props.data.title_left_bottom}
        onChangeTitle={this.handleValueChange}
        color={this.props.data.color_left_bottom}
        onChangeColor={this.handleValueChange} />
    }

    let tbody = <tbody>
      <tr>
        <td rowSpan={2} colSpan={2}>{left_top_cell}</td>
      </tr>
    </tbody>
    if (layout == 2) {
      tbody = <tbody>
        <tr>
          <td rowSpan={2}>{left_top_cell}</td>
          <td rowSpan={2}>{right_top_cell}</td>
        </tr>
      </tbody>
    }
    if (layout == 3) {
      tbody = <tbody>
        <tr>
          <td rowSpan={2}>{left_top_cell}</td>
          <td>{right_top_cell}</td>
        </tr>
        <tr>
          <td>{right_bottom_cell}</td>
        </tr>
      </tbody>
    }
    if (layout == 4) {
      tbody = <tbody>
        <tr>
          <td>{left_top_cell}</td>
          <td>{right_top_cell}</td>
        </tr>
        <tr>
          <td>{left_bottom_cell}</td>
          <td>{right_bottom_cell}</td>
        </tr>
      </tbody>
    }

    const is_hex = /^#([0-9A-F]{3}){1,2}$/i.test(this.props.data.color_left_top)
    const color = is_hex ? this.props.data.color_left_top : 'none'
    const style = { backgroundColor: color }
    return <table className='etsi-color' style={style}>
      {tbody}
    </table>
  }
}

interface CellProps {
    language: {[key: string]: string},
    is_editable: boolean,
    suffix: string,
    title: string,
    onChangeTitle: Function,
    color: string,
    onChangeColor: Function,
}

class Cell extends React.Component<CellProps, {}> {
  constructor (props: CellProps) {
    super(props)
    this.handleChange = this.handleChange.bind(this)
  }

  handleChange (e: React.FormEvent<HTMLInputElement>) {
    this.props.onChangeTitle(e.currentTarget.id, e.currentTarget.value)
  }

  render () {
    const is_hex = /^#([0-9A-F]{3}){1,2}$/i.test(this.props.color)
    const color = is_hex ? this.props.color : 'none'
    const style = { backgroundColor: color }
    return <div className='etsi-color-cell' style={style}>
      <input id={'title' + this.props.suffix}
        type='text'
        onChange={this.handleChange}
        value={this.props.title ? this.props.title : ''}
        disabled={!this.props.is_editable}
        placeholder={this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_DESCRIPTION} />
      <input id={'color' + this.props.suffix}
        type='color'
        onChange={this.handleChange}
        value={this.props.color ? this.props.color : ''}
        disabled={!this.props.is_editable}
        placeholder={this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_PLACEHOLDER} />
    </div>
  }
}
