import * as React from 'react'

interface LayoutProps {
  is_editable: boolean,
  layout: string,
  onLayoutChange: Function,
}

export class Layout extends React.Component<LayoutProps, {}> {
  constructor (props: LayoutProps) {
    super(props)
    this.handleLayoutChange = this.handleLayoutChange.bind(this)
  }

  handleLayoutChange (e: React.FormEvent<HTMLInputElement>) {
    const layout = e.currentTarget.value
    this.props.onLayoutChange('layout', layout)
  }

  render () {
    if (!this.props.is_editable) {
      return null
    }

    return <div className='layout'>
      <div>
        <input type='radio'
          name='layout[]'
          value='1'
          defaultChecked={this.props.layout == '1'}
          onChange={this.handleLayoutChange} />
        Layout 1
        <table className='preview'>
          <tbody>
            <tr>
              <td rowSpan={2} colSpan={2}>
                <div className='preview-left-top' />
              </td>
            </tr>
          </tbody>
        </table>
      </div>
      <div>
        <input type='radio'
          name='layout[]'
          value='2'
          defaultChecked={this.props.layout == '2'}
          onChange={this.handleLayoutChange} />
        Layout 2
        <table className='preview'>
          <tbody>
            <tr>
              <td rowSpan={2}>
                <div className='preview-left-top' />
              </td>
              <td rowSpan={2}>
                <div className='preview-right-top' />
              </td>
            </tr>
          </tbody>
        </table>
      </div>
      <div>
        <input type='radio'
          name='layout[]'
          value='3'
          defaultChecked={this.props.layout == '3'}
          onChange={this.handleLayoutChange} />
        Layout 3
        <table className='preview'>
          <tbody>
            <tr>
              <td rowSpan={2}>
                <div className='preview-left-top' />
              </td>
              <td>
                <div className='preview-right-top' />
              </td>
            </tr>
            <tr>
              <td>
                <div className='preview-right-bottom' />
              </td>
            </tr>
          </tbody>
        </table>
      </div>
      <div>
        <input type='radio'
          name='layout[]'
          value='4'
          defaultChecked={this.props.layout == '4'}
          onChange={this.handleLayoutChange} />
        Layout 4
        <table className='preview'>
          <tbody>
            <tr>
              <td>
                <div className='preview-left-top' />
              </td>
              <td>
                <div className='preview-right-top' />
              </td>
            </tr>
            <tr>
              <td>
                <div className='preview-left-bottom' />
              </td>
              <td>
                <div className='preview-right-bottom' />
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    </div>
  }
}
