import * as React from 'react'
import { WirePinStatus, WireState } from './ColorCompose'

interface WireStatusProps {
  language: {[key: string]: string},
  is_editable: boolean,
  wire_states: WireState[],
  wire_pin_status: WirePinStatus[],
  onAddWireState: Function,
  onRemoveWireState: Function,
  onUpdateWireState: Function,
  onOperatorSelection: Function,
}

export class WireStatus extends React.Component<WireStatusProps, {}> {
  constructor (props: WireStatusProps) {
    super(props)
    this.addWireState = this.addWireState.bind(this)
    this.removeWireState = this.removeWireState.bind(this)
    this.handleNotChecked = this.handleNotChecked.bind(this)
    this.handleStatusSelection = this.handleStatusSelection.bind(this)
    this.handleOperatorSelection = this.handleOperatorSelection.bind(this)
  }

  addWireState () {
    this.props.onAddWireState()
  }

  removeWireState (e: React.FormEvent<HTMLButtonElement>) {
    const id = parseInt(e.currentTarget.parentElement.parentElement.id)
    this.props.onRemoveWireState(id)
  }

  handleNotChecked (e: React.FormEvent<HTMLInputElement>) {
    this.props.onUpdateWireState(e)
  }

  handleStatusSelection (e: React.FormEvent<HTMLSelectElement>) {
    this.props.onUpdateWireState(e)
  }

  handleOperatorSelection (e: React.FormEvent<HTMLSelectElement>) {
    const operator = e.currentTarget.value
    this.props.onOperatorSelection(operator)
  }

  render () {
    let wirePinStates: JSX.Element[] = []
    wirePinStates.push(<option key='0' value='0'>-</option>)
    this.props.wire_pin_status.forEach(w => {
      wirePinStates.push(<option key={'pin_state_' + w.id} value={w.id}>{w.const}</option>)
    })

    const operators = [
      <option key='op_and' value='AND'>{this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_AND}</option>,
      <option key='op_or' value='OR'>{this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_OR}</option>
    ]

    let current_states = this.props.wire_states;
    current_states.sort((a: WireState, b: WireState) => a.id > b.id ? 1 : -1)
    if (current_states.length == 0 && this.props.is_editable) {
      // add empty option
      current_states.push({
        id: 1,
        wire_pin_status_id: 0,
        operator: 'AND',
        not: false
      })
    }

    let states: JSX.Element[] = []
    current_states.forEach(w => {
      let removeButton = null
      let addButton = null
      if (this.props.is_editable) {
        removeButton = <button type='button'
                               className="btn"
                               onClick={this.removeWireState}
                               title={this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_REMOVE_RULE}>-</button>
        addButton = current_states.length == states.length + 1 ? <button type='button'
                                                                         className="btn"
                                                                         onClick={this.addWireState}
                                                                         title={this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_ADD_RULE}>+</button> : null
      }

      states.push(
        <tr key={'status_' + w.id}
          id={w.id.toString()}>
          <td>
              ! <input type='checkbox'
              className="input"
              name={'not_' + w.id}
              defaultChecked={w.not}
              onChange={this.handleNotChecked}
              disabled={!this.props.is_editable} />
          </td>
          <td>
            <select name={'pin_status_' + w.id}
              className="select input input-small"
              onChange={this.handleStatusSelection}
              value={w.wire_pin_status_id}
              disabled={!this.props.is_editable}>{wirePinStates}</select>
          </td>
          <td>
            <select name={'operator_' + w.id}
              className="select input input-small"
              onChange={this.handleOperatorSelection}
              value={w.operator}
              disabled={!this.props.is_editable}>{operators}</select>
          </td>
          <td>
            {removeButton}
            {addButton}
          </td>
        </tr>
      )
    })

    return <table className='wire-status'>
      <thead>
        <tr>
          <th>{this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_NOT}</th>
          <th>{this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_WIRE_STATUS}</th>
          <th>{this.props.language.LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_OPERAND}</th>
          <th />
        </tr>
      </thead>
      <tbody>
        {states}
      </tbody>
    </table>
  }
}
