import * as React from 'react'
import { ColorTable } from '../ColorTable'
import { WireState } from '../ColorCompose'
const ReactTestRenderer = require('react-test-renderer')

const lang = {}
const mockFunction = jest.fn()
const ws: WireState[] = []
const basicData = {
  layout: '1',
  title_left_top: 'Title01',
  color_left_top: '#112233',
  title_left_bottom: 'Title04',
  color_left_bottom: '#AA00FF',
  title_right_top: 'Title02',
  color_right_top: '#6699AA',
  title_right_bottom: 'Title03',
  color_right_bottom: '#AACCFF',
  wire_states: ws
}

it('renders readonly color table correctly', () => {
  const data = basicData
  data.layout = '3'
  const component = ReactTestRenderer.create(
    <ColorTable language={lang} is_editable={false} data={data} onDataChange={mockFunction} />
  )
  expect(component).toMatchSnapshot()
})

it('renders color table one correctly', () => {
  const data = basicData
  data.layout = '1'
  const component = ReactTestRenderer.create(
    <ColorTable language={lang} is_editable={true} data={data} onDataChange={mockFunction} />
  )
  expect(component).toMatchSnapshot()
})

it('renders color table two correctly', () => {
  const data = basicData
  data.layout = '2'
  const component = ReactTestRenderer.create(
    <ColorTable language={lang} is_editable={true} data={data} onDataChange={mockFunction} />
  )
  expect(component).toMatchSnapshot()
})

it('renders color table three correctly', () => {
  const data = basicData
  data.layout = '3'
  const component = ReactTestRenderer.create(
    <ColorTable language={lang} is_editable={true} data={data} onDataChange={mockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  const select = instance.findByProps({ value: 'Title01' })
  select.props.onChange({
    currentTarget: { id: 'title_left_top', value: 'TitleA' }
  })
  expect(mockFunction).toHaveBeenCalledWith('title_left_top', 'TitleA')
})

it('renders color table four correctly', () => {
  const data = basicData
  data.layout = '4'
  const component = ReactTestRenderer.create(
    <ColorTable language={lang} is_editable={true} data={data} onDataChange={mockFunction} />
  )
  expect(component).toMatchSnapshot()
})
