import * as React from 'react'
import { WireStatus } from '../WireStatus'
const ReactTestRenderer = require('react-test-renderer')

const lang = {
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_NOT: 'not',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_WIRE_STATUS: 'wire status',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_OPERAND: 'operand',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_ADD_RULE: 'add',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_REMOVE_RULE: 'remove',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_AND: 'AND',
  LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR_OR: 'OR'
}
const wire_states = [
  {
    id: 1,
    wire_pin_status_id: 2,
    operator: 'AND',
    not: false
  }
]
const wire_pin_states = [
  {
    id: 1,
    const: 'S'
  },
  {
    id: 2,
    const: 'E2000'
  }
]

it('renders readonly wire status correctly', () => {
  const component = ReactTestRenderer.create(
    <WireStatus language={lang}
      is_editable={false}
      wire_states={wire_states}
      wire_pin_status={wire_pin_states}
      onAddWireState={null}
      onRemoveWireState={null}
      onUpdateWireState={null}
      onOperatorSelection={null} />
  )
  expect(component).toMatchSnapshot()
})

it('renders wire status correctly', () => {
  const addWireStateMockFunction = jest.fn()
  const removeWireStateMockFunction = jest.fn()
  const updateWireStateMockFunction = jest.fn()
  const operatorSelectionMockFunction = jest.fn()
  const component = ReactTestRenderer.create(
    <WireStatus language={lang}
      is_editable={true}
      wire_states={wire_states}
      wire_pin_status={wire_pin_states}
      onAddWireState={addWireStateMockFunction}
      onRemoveWireState={removeWireStateMockFunction}
      onUpdateWireState={updateWireStateMockFunction}
      onOperatorSelection={operatorSelectionMockFunction} />
  )
  expect(component).toMatchSnapshot()

  const instance = component.root
  let select = instance.findByProps({ name: 'operator_1' })
  select.props.onChange({
    currentTarget: { value: 'OR' }
  })
  expect(operatorSelectionMockFunction).toHaveBeenCalledWith('OR')

  select = instance.findByProps({ name: 'pin_status_1' })
  select.props.onChange({ test: 1 })
  expect(updateWireStateMockFunction).toHaveBeenCalledWith({ test: 1 })

  const input = instance.findByProps({ name: 'not_1' })
  input.props.onChange({ test: 2 })
  expect(updateWireStateMockFunction).toHaveBeenCalledWith({ test: 2 })

  let button = instance.findByProps({ title: 'remove' })
  button.props.onClick({
    currentTarget: { parentElement: { parentElement: { id: '1' } } }
  })
  expect(removeWireStateMockFunction).toHaveBeenCalledWith(1)

  button = instance.findByProps({ title: 'add' })
  button.props.onClick()
  expect(addWireStateMockFunction).toHaveBeenCalled()
})

it('renders empty wire status correctly', () => {
  const component = ReactTestRenderer.create(
    <WireStatus language={lang}
      is_editable={true}
      wire_states={[]}
      wire_pin_status={wire_pin_states}
      onAddWireState={null}
      onRemoveWireState={null}
      onUpdateWireState={null}
      onOperatorSelection={null} />
  )
  expect(component).toMatchSnapshot()
})
