import * as React from 'react'

export interface CommonObject {
  id: number,
  name: string,
}

export interface PositionalObject extends CommonObject {
  position: number,
  description: string,
  obj_title: string,
  item_count: number,
}

export function dropObject (objectId: number, posString: string, availableObjects: PositionalObject[], assignedObjects: PositionalObject[]) {
  let object: PositionalObject = null
  const availableItems = availableObjects.filter(r => r.id == objectId)
  if (availableItems.length > 0) {
    object = availableItems[0]
  }

  let oldPos: number = null
  let assignedItems = assignedObjects.filter(r => r.id == objectId)
  if (assignedItems.length > 0) {
    // item is already assigned
    object = assignedItems[0]
    oldPos = object.position
  }

  if (!object) {
    return assignedObjects
  }

  assignedItems = assignedObjects.filter(r => r.id != objectId)
  let pos = parseInt(posString.substring(3))
  if (oldPos && oldPos < pos) {
    // decrement position if object moved to higher position
    pos--
  }
  object.position = pos

  // update all other positions
  const newAssignedItems: PositionalObject[] = []
  assignedItems.forEach(r => {
    if (!oldPos && r.position >= pos) {
      // on new insertions
      r.position++
    }

    if (oldPos && ((r.position >= oldPos && r.position <= pos) || (r.position <= oldPos && r.position >= pos))) {
      // on movement
      if (oldPos < pos) {
        // move to right
        r.position--
      }
      if (oldPos >= pos) {
        // move to left
        r.position++
      }
    }
    newAssignedItems.push(r)
  })
  newAssignedItems.push(object)
  return newAssignedItems
}

export function addObjects (objects: PositionalObject[], assignedObjects: PositionalObject[]) {
  let nextPos = getNextPosition(assignedObjects)
  const newAssignedObjects = assignedObjects
  objects.forEach(r => {
    const filteredObjects = assignedObjects.filter(r2 => r2.id == r.id)
    if (filteredObjects.length > 0) {
      return
    }
    r.position = nextPos
    newAssignedObjects.push(r)
    nextPos++
  })
  return newAssignedObjects
}

export function getNextPosition (objects: PositionalObject[]) {
  let lastPos = 0
  objects.forEach(r => {
    if (r.position > lastPos) {
      lastPos = r.position
    }
  })
  return ++lastPos
}

export function removeObject (objId: number, objects: PositionalObject[]) {
  const racks = objects.filter(r => r.id == objId)
  const objectToRemove = racks.length > 0 ? racks[0] : null

  if (!objectToRemove) {
    return objects
  }

  const pos = objectToRemove.position
  const assignedObjects: PositionalObject[] = []
  objects.forEach(r => {
    if (r.id == objectToRemove.id) {
      return
    }
    if (r.position > pos) {
      r.position--
    }
    assignedObjects.push(r)
  })
  return assignedObjects
}
