import * as React from 'react'
import { addObjects, dropObject, getNextPosition, removeObject } from '../Common'

const obj1 = {
  id: 1,
  name: 'obj01',
  position: 1,
  description: 'Obj01',
  obj_title: 'Object 01',
  item_count: 2
}
const obj2 = {
  id: 2,
  name: 'obj02',
  position: 2,
  description: 'Obj02',
  obj_title: 'Object 02',
  item_count: 1
}
const obj3 = {
  id: 3,
  name: 'obj03',
  position: 3,
  description: 'Obj03',
  obj_title: 'Object 03',
  item_count: 8
}

it('get next position', () => {
  let res = getNextPosition([obj1])
  expect(res).toBe(2)

  res = getNextPosition([obj1, obj2])
  expect(res).toBe(3)
})

it('drop new object first', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const o3 = JSON.parse(JSON.stringify(obj3))
  const res = dropObject(3, 'pos1', [o1, o2, o3], [o1, o2])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  const exp2 = JSON.parse(JSON.stringify(obj2))
  const exp3 = JSON.parse(JSON.stringify(obj3))
  exp1.position = 2
  exp2.position = 3
  exp3.position = 1
  expect(res).toStrictEqual([exp1, exp2, exp3])
})

it('drop new object in the middle', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const o3 = JSON.parse(JSON.stringify(obj3))
  const res = dropObject(3, 'pos2', [o1, o2, o3], [o1, o2])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  const exp2 = JSON.parse(JSON.stringify(obj2))
  const exp3 = JSON.parse(JSON.stringify(obj3))
  exp1.position = 1
  exp2.position = 3
  exp3.position = 2
  expect(res).toStrictEqual([exp1, exp2, exp3])
})

it('drop new object last', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const o3 = JSON.parse(JSON.stringify(obj3))
  const res = dropObject(3, 'pos3', [o1, o2, o3], [o1, o2])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  const exp2 = JSON.parse(JSON.stringify(obj2))
  const exp3 = JSON.parse(JSON.stringify(obj3))
  expect(res).toStrictEqual([exp1, exp2, exp3])
})

it('drop moved object to left', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const o3 = JSON.parse(JSON.stringify(obj3))
  const res = dropObject(3, 'pos2', [o1, o2, o3], [o1, o2, o3])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  const exp2 = JSON.parse(JSON.stringify(obj2))
  const exp3 = JSON.parse(JSON.stringify(obj3))
  exp1.position = 1
  exp2.position = 3
  exp3.position = 2
  expect(res).toStrictEqual([exp1, exp2, exp3])
})

it('drop moved object to right', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const o3 = JSON.parse(JSON.stringify(obj3))
  const res = dropObject(1, 'pos3', [o1, o2, o3], [o1, o2, o3])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  const exp2 = JSON.parse(JSON.stringify(obj2))
  const exp3 = JSON.parse(JSON.stringify(obj3))
  exp1.position = 2
  exp2.position = 1
  exp3.position = 3
  expect(res).toStrictEqual([exp2, exp3, exp1])
})

it('drop unknown object', () => {
  // create deep copies
  const o1 = JSON.parse(JSON.stringify(obj1))
  const o2 = JSON.parse(JSON.stringify(obj2))
  const res = dropObject(88, 'pos2', [o1, o2], [o1])

  const exp1 = JSON.parse(JSON.stringify(obj1))
  expect(res).toStrictEqual([exp1])
})

it('add objects', () => {
  const obj3 = {
    id: 3,
    name: 'obj03',
    position: 3,
    description: 'Obj03',
    obj_title: 'Object 03',
    item_count: 8
  }

  const res = addObjects([obj3], [obj1, obj2])
  expect(res).toStrictEqual([obj1, obj2, obj3])
})

it('remove object', () => {
  let res = removeObject(2, [obj1, obj2])
  expect(JSON.stringify(res)).toBe(JSON.stringify([obj1]))

  res = removeObject(88, [obj1, obj2])
  expect(JSON.stringify(res)).toBe(JSON.stringify([obj1, obj2]))
})
