import * as React from 'react'
import {IndexedElement} from "../SectionRouteEditor";

interface IndexedRowProps {
    is_editable: boolean,
    is_route: boolean,
    element: IndexedElement,
    completeLength: number,
    isLastRow: boolean,
    is_associated_element: boolean,
    onConfiguredCableSelectionChange: Function,
    onDeleteClick: Function,
}

export class IndexedRow extends React.Component<IndexedRowProps, {}> {
    constructor (props: IndexedRowProps) {
        super(props)
        this.handleConfiguredCableSelectionChange = this.handleConfiguredCableSelectionChange.bind(this)
        this.handleDeleteClick = this.handleDeleteClick.bind(this)
    }

    handleConfiguredCableSelectionChange(e: React.ChangeEvent<HTMLSelectElement>) {
        let index = parseInt(e.currentTarget.id.substring(6)) // remove "cable_"
        let element_id = e.currentTarget.value;
        let wire_id = element_id.startsWith('w_') ? parseInt(element_id.substr(2)) : null;
        let section_id = element_id.startsWith('s_') ? parseInt(element_id.substr(2)) : null;
        this.props.onConfiguredCableSelectionChange(index, wire_id, section_id)
    }

    handleDeleteClick(e: React.MouseEvent<HTMLButtonElement>) {
        let tr = e.currentTarget.parentElement.parentElement
        let index = parseInt(tr.id.substring(4)) // remove "row_"
        this.props.onDeleteClick(index)
    }

    getSelect(id: number, title: string, disabled: boolean = false) {
        if (this.props.is_editable) {
            id = id ? id : -1;
            return <select disabled={disabled} defaultValue={id} className="input input-small">
                <option key={id} id={id.toString()}>{title}</option>
            </select>
        }
        return <span>{title}</span>
    }

    render () {
        // objects
        const element = this.props.element
        let index = parseInt(element.index.toString())
        let scope_out = element.pin_out_scope && element.pin_out_scope != 'null' ? element.pin_out_scope : '';
        let scope_in = element.pin_in_scope && element.pin_in_scope != 'null' ? element.pin_in_scope : '';
        let title_out = element.obj_out_title + ' - ' + scope_out + ' - ' + element.pin_out_title;
        let title_in = element.obj_in_title + ' - ' + scope_in + ' - ' + element.pin_in_title;

        // cables
        let cableSelect
        if (! this.props.is_editable) {
            if (this.props.is_route && element.section_id) {
                cableSelect = this.getSelect(element.section_id, element.section_title);
            } else {
                let title = element.cable_title;
                if (element.wire_scope) {
                    title += ' - ' + element.wire_scope;
                }
                title += ' - ' + element.wire_title;
                cableSelect = this.getSelect(element.cable_id, title);
            }
        } else {
            let options: JSX.Element[] = []
            let defaultValue
            element.connections.forEach(obj => {
                // filter by selected objects
                if ((obj.obj_in_id == element.obj_in_id && obj.obj_out_id == element.obj_out_id) ||
                    (obj.obj_out_id == element.obj_in_id && obj.obj_in_id == element.obj_out_id)) {
                } else {
                    return;
                }

                let title: string
                let value: string
                if (this.props.is_route && obj.section_id) {
                    value = 's_' + obj.section_id
                    title = obj.section_title;
                } else {
                    value = 'w_' + obj.wire_id
                    let scope = obj.wire_scope ? obj.wire_scope : '';
                    title = obj.cable_title + ' - ' + scope + ' - ' + obj.wire_title;
                }

                if ((obj.wire_id && obj.wire_id == element.wire_id) || (obj.section_id && obj.section_id == element.section_id)) {
                    defaultValue = value
                }
                options.push(<option key={value} value={value}>{title}</option>)
            });
            cableSelect = <select id={'cable_' + index} defaultValue={defaultValue} onChange={this.handleConfiguredCableSelectionChange} className="input input-small">{options}</select>
        }

        // capacity
        let capacity = '';
        let capacityClass = ''
        if (! this.props.is_route || ! element.section_id) {
            let free = element.cable_free_wires;
            let patched = element.cable_patched_wires;
            let all = free + patched;
            let percent = 0;
            if (all > 0) {
                percent = (patched / all) * 100;
                percent = Math.round(percent);
            }

            capacity = percent + '% (max. ';
            capacity += (element.cable_max_capacity ? element.cable_max_capacity : 0) + '%)';

            if (element.cable_max_capacity > 0 && percent >= element.cable_max_capacity) {
                capacityClass = 'attention'
            }
        }

        const length = element.length ? element.length : 0;
        const deleteButton = this.props.is_editable && this.props.isLastRow ? <button type="button" className="btn" onClick={this.handleDeleteClick}>Delete</button> : null
        const rowTitle = this.props.is_associated_element ? 'associated_row_' : 'row_'
        return <tr key={rowTitle + index + '_'+ element.obj_out_id} id={rowTitle + index} className={this.props.is_associated_element ? 'associated-row' : ''}>
            <td>{this.props.is_associated_element ? '' : index}</td>
            <td>
                {this.getSelect(element.obj_out_id, title_out, true)}
            </td>
            <td>
                {this.getSelect(element.obj_in_id, title_in, true)}
            </td>
            <td>
                {cableSelect}
            </td>
            <td className={capacityClass}>
                {capacity}
            </td>
            <td>
                {length + 'm (' + this.props.completeLength + 'm)'}
            </td>
            <td>
                {deleteButton}
            </td>
        </tr>
    }
}
