import * as React from 'react'
import {BaseElement, SimpleElement} from "../SectionRouteEditor";

interface LastRowProps {
    is_loading: boolean,
    is_route: boolean,
    spinner_url: string,
    last_index: number,
    selectable_objects: BaseElement[],
    last_element: SimpleElement,
    onCableSelectionChange: Function,
}

interface LastRowState {
    selected_object_id: number,
}

export class LastRow extends React.Component<LastRowProps, LastRowState> {
    constructor (props: LastRowProps) {
        super(props)
        this.state = {
            selected_object_id: null
        }
        this.handleObjectSelectionChange = this.handleObjectSelectionChange.bind(this)
        this.handleCableSelectionChange = this.handleCableSelectionChange.bind(this)
    }

    handleObjectSelectionChange(e: React.ChangeEvent<HTMLSelectElement>) {
        let obj_id = parseInt(e.currentTarget.value)
        this.setState((prevState) => ({
            ...prevState,
            selected_object_id: obj_id > 0 ? obj_id : null
        }))
    }

    handleCableSelectionChange(e: React.ChangeEvent<HTMLSelectElement>) {
        let element_id = e.currentTarget.value
        let index = parseInt(e.currentTarget.id.substring(6)) // remove "cable_"
        let wire_id = element_id.startsWith('w_') ? parseInt(element_id.substr(2)) : null;
        let section_id = element_id.startsWith('s_') ? parseInt(element_id.substr(2)) : null;
        this.props.onCableSelectionChange(index, wire_id, section_id);
    }

    render () {
        const index: number = this.props.last_index + 1
        if (this.props.is_loading) {
            return <tr key={'row_' + index} id={'row_' + index}>
                <td>{index}</td>
                <td colSpan={6}>
                    <img key={'station_loading'} src={this.props.spinner_url} alt="Loading..." />
                </td>
            </tr>
        }

        let objectOptions: JSX.Element[] = []
        let cableOptions: JSX.Element[] = []
        objectOptions.push(<option key={-1} value={-1}>-</option>)
        cableOptions.push(<option key={-1} value={-1}>-</option>)
        let objs: string[] = [];
        this.props.selectable_objects.forEach(obj => {
            // handle cables
            let cableValue =  this.props.is_route && obj.section_id ? 's_' + obj.section_id : 'w_' + obj.wire_id
            let title: string;
            if (this.props.is_route && obj.section_id) {
                title = obj.section_title;
            } else {
                let scope = obj.wire_scope ? obj.wire_scope : '';
                title = obj.cable_title + ' - ' + scope + ' - ' + obj.wire_title;
            }
            if (! this.state.selected_object_id || obj.obj_in_id == this.state.selected_object_id) {
                // only add cables of selected object (if an object is selected)
                cableOptions.push(<option key={cableValue} value={cableValue}>{title}</option>)
            }

            // handle objects
            let key = 'id' + obj.obj_in_id;
            if (objs.indexOf(key) >= 0) {
                return;
            }
            objectOptions.push(
                <option key={obj.obj_in_id} value={obj.obj_in_id.toString()}>
                    {obj.obj_in_title}
                </option>)
            objs.push(key);
        })

        return <tr key={'row_' + index} id={'row_' + index}>
            <td>{index}</td>
            <td>
                <select disabled={true} defaultValue={-1} className="input input-small">
                    <option key={-1} id="-1">{this.props.last_element.title}</option>
                </select>
            </td>
            <td>
                <select id={'incoming_' + index} onChange={this.handleObjectSelectionChange} disabled={this.props.is_loading}  className="input input-small">
                    {objectOptions}
                </select>
            </td>
            <td>
                <select id={'cable_' + index} onChange={this.handleCableSelectionChange} disabled={this.props.is_loading}  className="input input-small">
                    {cableOptions}
                </select>
            </td>
            <td colSpan={3} />
        </tr>
    }
}
