import * as React from 'react'
import {SimpleElement} from "../SectionRouteEditor";

interface TypeSelectionRowProps {
    is_loading: boolean,
    spinner_url: string,
    elements: SimpleElement[],
    onObjectSelectionChange: Function,
}

export class TypeSelectionRow extends React.Component<TypeSelectionRowProps, {}> {
    constructor (props: TypeSelectionRowProps) {
        super(props)
        this.handleObjectSelectionChange = this.handleObjectSelectionChange.bind(this)
    }

    handleObjectSelectionChange(e: React.ChangeEvent<HTMLSelectElement>) {
        let obj_id = parseInt(e.currentTarget.value)
        if (obj_id > 0) {
            this.props.onObjectSelectionChange(obj_id)
        }
    }

    render () {
        let objectOptions: JSX.Element[] = []
        objectOptions.push(<option key={-1} value={-1}>-</option>)
        this.props.elements.forEach(obj => {
            objectOptions.push(
                <option key={obj.id} value={obj.id.toString()}>
                    {obj.title}
                </option>)
        })

        const spinner = this.props.is_loading ? <img key={'station_loading'} src={this.props.spinner_url} alt="Loading..." /> : null
        return <tr>
            <td>1</td>
            <td>
                <select className="select input input-small"
                        onChange={this.handleObjectSelectionChange}
                        disabled={this.props.is_loading}>
                    {objectOptions}
                </select>
                {spinner}
            </td>
            <td colSpan={5} />
        </tr>
    }
}
