import * as React from 'react'
import {IndexedRow} from "../../Row/IndexedRow";
import {IndexedElement} from "../../SectionRouteEditor";
const ReactTestRenderer = require('react-test-renderer')

const element: IndexedElement = {
    id: 1,
    index: 1,
    cable_id: 100,
    cable_title: 'Cable01',
    length: 1000,
    wire_id: 1001,
    wire_title: 'Wire01',
    wire_scope: 'B1',
    wire_pin_in_title: 'PinIn01',
    wire_pin_out_title: 'PinOut01',
    section_id: 0,
    section_title: '',
    cable_free_wires: 2,
    cable_patched_wires: 6,
    cable_max_capacity: 80,
    obj_in_id: 200,
    obj_in_title: 'ObjIn01',
    pin_in_id: 300,
    pin_in_title: 'PinIn01',
    pin_in_scope: 'P1',
    obj_out_id: 201,
    obj_out_title: 'ObjOut01',
    pin_out_id: 301,
    pin_out_title: 'PinOut01',
    pin_out_scope: 'P1',
    connections: [
        {
            cable_id: 101,
            cable_title: 'Cable11',
            length: 100,
            wire_id: 1010,
            wire_title: 'Wire10',
            wire_scope: 'B1',
            wire_pin_in_title: 'PinIn01',
            wire_pin_out_title: 'PinOut01',
            section_id: 0,
            section_title: '',
            cable_free_wires: 2,
            cable_patched_wires: 6,
            cable_max_capacity: 80,
            obj_in_id: 200,
            obj_in_title: 'ObjIn01',
            pin_in_id: 300,
            pin_in_title: 'PinIn01',
            pin_in_scope: 'P1',
            obj_out_id: 201,
            obj_out_title: 'ObjOut01',
            pin_out_id: 301,
            pin_out_title: 'PinOut01',
            pin_out_scope: 'P1',
        },
        {
            cable_id: 102,
            cable_title: 'Cable22',
            length: 200,
            wire_id: 0,
            wire_title: '',
            wire_scope: 'B1',
            wire_pin_in_title: 'PinIn01',
            wire_pin_out_title: 'PinOut01',
            section_id: 900,
            section_title: 'Section01',
            cable_free_wires: 0,
            cable_patched_wires: 1,
            cable_max_capacity: 100,
            obj_in_id: 200,
            obj_in_title: 'ObjIn01',
            pin_in_id: 300,
            pin_in_title: 'PinIn01',
            pin_in_scope: 'P1',
            obj_out_id: 201,
            obj_out_title: 'ObjOut01',
            pin_out_id: 301,
            pin_out_title: 'PinOut01',
            pin_out_scope: 'P1',
        }
    ]
}
const cableSelectionChangeMockFunction = jest.fn()
const deleteClickMockFunction = jest.fn()

it('renders readonly indexed row correctly', () => {
    const component = ReactTestRenderer.create(
        <IndexedRow is_editable={false}
                    is_route={true}
                    element={element}
                    completeLength={100}
                    isLastRow={false}
                    is_associated_element={false}
                    onConfiguredCableSelectionChange={cableSelectionChangeMockFunction}
                    onDeleteClick={deleteClickMockFunction} />
    )
    expect(component).toMatchSnapshot()
})

it('renders indexed row correctly', () => {
    const component = ReactTestRenderer.create(
        <IndexedRow is_editable={true}
                    is_route={true}
                    element={element}
                    completeLength={100}
                    isLastRow={false}
                    is_associated_element={false}
                    onConfiguredCableSelectionChange={cableSelectionChangeMockFunction}
                    onDeleteClick={deleteClickMockFunction} />
    )
    expect(component).toMatchSnapshot()

    const instance = component.root
    const select = instance.findByProps({ id: 'cable_1' })
    select.props.onChange({
        currentTarget: {
            id: 'cable_1',
            value: 'w_1010'
        }
    })
    expect(cableSelectionChangeMockFunction).toHaveBeenCalledWith(1, 1010, null)

    select.props.onChange({
        currentTarget: {
            id: 'cable_1',
            value: 's_900'
        }
    })
    expect(cableSelectionChangeMockFunction).toHaveBeenCalledWith(1, null, 900)
})

it('renders last indexed row correctly', () => {
    const component = ReactTestRenderer.create(
        <IndexedRow is_editable={true}
                    is_route={true}
                    element={element}
                    completeLength={100}
                    isLastRow={true}
                    is_associated_element={false}
                    onConfiguredCableSelectionChange={cableSelectionChangeMockFunction}
                    onDeleteClick={deleteClickMockFunction} />
    )
    expect(component).toMatchSnapshot()

    const instance = component.root
    const button = instance.findByProps({ type: 'button' })
    button.props.onClick({
        currentTarget: {
            parentElement: {
                parentElement: {
                    id: 'row_1'
                }
            }
        }
    })
    expect(deleteClickMockFunction).toHaveBeenCalledWith(1)
})

it('renders associated element row correctly', () => {
    const component = ReactTestRenderer.create(
        <IndexedRow is_editable={true}
                    is_route={true}
                    element={element}
                    completeLength={100}
                    isLastRow={false}
                    is_associated_element={true}
                    onConfiguredCableSelectionChange={null}
                    onDeleteClick={null} />
    )
    expect(component).toMatchSnapshot()
})
