import * as React from 'react'
import {SectionRouteTracing} from "../SectionRouteTracing";
const ReactTestRenderer = require('react-test-renderer')

const lang = {}
const object_types = {
    1: 'objType01',
    2: 'objType02'
}
const mockFunction = jest.fn()

it('renders tracing correctly', () => {
    const component = ReactTestRenderer.create(
        <SectionRouteTracing is_loading={false}
                             ajax_url={''}
                             session={''}
                             spinner_url={''}
                             language={lang}
                             object_types={object_types}
                             onTraceStart={mockFunction} />
    )
    expect(component).toMatchSnapshot()

    const instance = component.root
    let select = instance.findByProps({ name: 'start_object' })
    select.props.onChange({
        currentTarget: {
            value: '100',
            name: 'start_object'
        }
    })

    select = instance.findByProps({ name: 'end_object' })
    select.props.onChange({
        currentTarget: {
            value: '200',
            name: 'end_object'
        }
    })

    const button = instance.findByProps({ type: 'button' })
    button.props.onClick()
    expect(mockFunction).toHaveBeenCalledWith(100, 200, 'shortest_path')

    const input = instance.findByProps({ value: 'least_hops' })
    input.props.onClick({
        currentTarget: {
            value: 'least_hops',
        }
    })
    button.props.onClick()
    expect(mockFunction).toHaveBeenCalledWith(100, 200, 'least_hops')
})

it('orders object types correctly', () => {
    const unsorted_options = {
        1: 'MType',
        2: 'AType',
        3: 'zType',
        4: 'fType'
    }
    const sorted_options = [
        <option key={-1} value={-1}>-</option>,
        <option key={2} value="2">AType</option>,
        <option key={4} value="4">fType</option>,
        <option key={1} value="1">MType</option>,
        <option key={3} value="3">zType</option>
    ]
    const res = SectionRouteTracing.getSortedOptions(unsorted_options)
    expect(res).toStrictEqual(sorted_options)
})
