const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');

module.exports = {
  entry: {
    algorithm_editor:[
      './src/algorithm_editor/index.tsx'
    ],
    etsi_color_editor:[
      './src/etsi_color_editor/index.tsx',
      './../../sass/etsi_color_style.scss'
    ],
    etsi_bay_editor:[
      './src/etsi_bay_editor/index.tsx',
      './../../sass/etsi_bay_style.scss'
    ],
    etsi_export_editor:[
      './src/etsi_export_editor/index.tsx',
      './../../sass/etsi_export_style.scss'
    ],
    etsi_ordering_editor:[
      './src/etsi_ordering_editor/index.tsx',
      './../../sass/etsi_export_style.scss'
    ],
    section_route_editor:[
      './src/section_route_editor/index.tsx',
      './../../sass/style.scss'
    ]
  },
  output: {
    filename: '[name]/[name].js',
    path: resolve(__dirname, './../../dist/'),
  },
  resolve: {
    cacheWithContext: false,
    extensions: ['.ts', '.tsx', '.js'],
    symlinks: false,
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        include: resolve(__dirname, 'src'),
        use: 'ts-loader',
        exclude: /node_modules/,
      },
      // sass to css file
      {
        test: /\.(scss)$/,
        include: resolve(__dirname, '../../sass'),
        use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
        sideEffects: true
      },
      {
        test: /\.(png|svg|jpg|gif)$/,
        include: resolve(__dirname, 'icons'),
        loader: 'file-loader',
        options: {
          outputPath: 'images',
          publicPath: '../images'
        }
      }
    ]
  },
  optimization: {
     minimize: false,
    // separate 3rd-party libs because they won't change so often
    moduleIds: 'hashed',
    chunkIds: 'named',
    splitChunks: {
      chunks: 'async',
      maxAsyncRequests: 6,
      maxInitialRequests: 4,
      cacheGroups: {
        vendors: {
          test: /[\\/]node_modules[\\/]/,
          name: 'vendors',
          chunks: 'all',
          filename: 'vendors.js'
        }
      }
    }
  },
  plugins: [
    new CleanWebpackPlugin(),
    new MiniCssExtractPlugin({
      filename: '[name]/style.css'
    }),
  ]
};
