'use strict';

/**
 * i-doit Cable algorithm editor model class.
 *
 * @author Steve Groeber <steve.groeber@shd-online.de>
 */

class AlgorithmEditorModel {

    constructor(elements, inputId) {
        this.elements = elements;
        this.inputId = inputId;
        
        this.saveModelState();
    }
    
    getElements() {
        return this.elements;
    }
    
    setElementsByMask(mask) {
        //let separators = mask.match(/[-,\/]+/g);
        //let count = separators ? separators.length : 0;
        
        // add new elements
        let count = (mask.match(/#/g) || []).length;
        for (i = count; i >= 1; i--) {
            let pos = count - i + 1;
            if (this.elements[i]) {
                // update element position
                this.elements[i]['pos'] = pos;
                continue;
            }
            
            this.elements[i] = {
                'index': i,
                'pos': pos,
                'type': 'n',
                'length': 2
            };
        }
        
        // remove deleted elements
        let elementCount = Object.keys(this.elements).length;
        if (elementCount > count) {
            for (i = count + 1; i <= elementCount; i++) {
                delete this.elements[i];
            }
        }
        this.saveModelState();
    }
    
    setValue(index, key, value) {
        index = parseInt(index);
        if (! this.elements[index]) {
            return;
        }
        
        if (key == 'pos') {
            this.handlePosUpdate(index, value);
        } else {
            this.elements[index][key] = value;
        }
        this.saveModelState();
    }
    
    handlePosUpdate(index, newPos) {
        newPos = parseInt(newPos);
        if (isNaN(newPos)) {
            // value is not a valid integer
            return;
        }
        
        let elementCount = Object.keys(this.elements).length;
        if (newPos < 1 || newPos > elementCount) {
            // value does not fit into range of positions
            return;
        }
        
        // get current position of this element
        let element = this.elements[index];
        let oldPos = element['pos'];
        
        // search current element on new position
        let indexOfNewPos = null;
        for (let key in this.elements) {
            if (! this.elements.hasOwnProperty(key) ) {
                continue;
            }
            
            let e = this.elements[key];
            if (e['pos'] == newPos) {
                indexOfNewPos = key;
            }
        }
        
        // update this element to new position
        this.elements[index]['pos'] = newPos;
        
        // update the other element to old postion of this element
        if (indexOfNewPos) {
            this.elements[indexOfNewPos]['pos'] = oldPos;
        }
    }
    
    saveModelState() {
        if ($(this.inputId)) {
            $(this.inputId).value = JSON.stringify(this.getElements());
        }
    }
}

module.exports = AlgorithmEditorModel;
