'use strict';

/**
 * i-doit Cable algorithm editor renderer class.
 *
 * @author Steve Groeber <steve.groeber@shd-online.de>
 */

class AlgorithmEditorRenderer {
    
    constructor(table, model) {
        let tbodys = $$('#' + table + ' tbody');
        this.tbody = tbodys[0];
        this.model = model;
    }
    
    setTranslation(translation) {
        this.translation = translation;
    }
    
    render(editMode) {
        this.edit_mode = editMode;
        
        let that = this;
        this.tbody.innerHTML = '';
        let elements = this.model.getElements();
        for (let key in elements) {
            if (! elements.hasOwnProperty(key) ) {
                continue;
            }
            
            let element = elements[key];
            let tr = that.get_saved_row(element, Object.keys(elements).length);
            this.tbody.appendChild(tr);
        }
        
        if (! this.edit_mode) {
            return;
        }
    }
    
    get_saved_row(element, elementCount) {
        let that = this;
        let index = element.index;
        let tr = document.createElement('tr');
        tr.writeAttribute('id', 'row_' + index);
        
        // index
        let td = document.createElement('td');
        td.innerHTML = index;
        tr.appendChild(td);
        
        // pos
        td = document.createElement('td');
        let posInput = this._create_input('pos_' + index, element.pos);
        if (this.edit_mode) {
            posInput.writeAttribute('type', 'number');
            posInput.writeAttribute('min', 1);
            posInput.writeAttribute('max', elementCount);
            posInput.on('change', function(event) {
                // re-render table, because two positions may be changed
                that.render(that.edit_mode);
            });
        }
        td.appendChild(posInput);
        tr.appendChild(td);
        
        // type
        td = document.createElement('td');
        if (this.edit_mode) {
            let select = document.createElement('select');
            select.writeAttribute('id', 'type_' + index);
            let option = this._create_option('n', element.type == 'n');
            select.appendChild(option);
            option = this._create_option('e', element.type == 'e');
            select.appendChild(option);
            
            select.on('change', function(event) {
                let options = $(event.target).select('option:selected');
                let option = options[0];
                let parts = event.target.id.split('_');
                that.model.setValue(parts[1], parts[0], option.value);
            });
            
            td.appendChild(select);
        } else {
            td.innerHTML = element.type;
        }
        tr.appendChild(td);
        
        // description
        td = document.createElement('td');
        let descriptionInput = this._create_input('description_' + index, element.description);
        td.appendChild(descriptionInput);
        tr.appendChild(td);
        
        // length
        td = document.createElement('td');
        let lengthInput = this._create_input('length_' + index, element.length);
        if (this.edit_mode) {
            lengthInput.writeAttribute('type', 'number');
            lengthInput.writeAttribute('min', 1);
            lengthInput.writeAttribute('max', 8);
        }
        td.appendChild(lengthInput);
        tr.appendChild(td);
        
        // elements
        td = document.createElement('td');
        let elementsInput = this._create_input('elements_' + index, element.elements);
        td.appendChild(elementsInput);
        tr.appendChild(td);
        
        return tr;
    }
    
    _create_input(id, value) {
        let element;
        let that = this;
        value = value ? value : '';
        if (this.edit_mode) {
            element = document.createElement('input');
            element.writeAttribute('id', id);
            element.writeAttribute('value', value);
            element.addClassName('input');
            element.on('change', function(event) {
                let id = event.target.id;
                let parts = id.split('_');
                that.model.setValue(parts[1], parts[0], event.target.value);
            });
        } else {
            element = document.createElement('span');
            element.innerHTML = value;
        }
        return element;
    }
    
    _create_option(value, selected) {
        let option = document.createElement('option');
        option.writeAttribute('value', value);
        option.innerHTML = value;
        if (selected) {
            option.writeAttribute('selected', 'selected');
        }
        return option;
    }
}

module.exports = AlgorithmEditorRenderer;
