<?php

use idoit\Component\Property\Type\FloatProperty;
use idoit\Component\Property\Type\IntProperty;
use idoit\Component\Property\Type\TextProperty;
use idoit\Module\Cable\LabelGenerator;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cables.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        return [
            'state_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__STATE',
                        C__PROPERTY__INFO__DESCRIPTION => 'State'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD        => 'isys_catg_smview_cable_list__state_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_user_state',
                        C__PROPERTY__DATA__REFERENCES   => [
                            'isys_catg_smview_cable_user_state',
                            'isys_catg_smview_cable_user_state__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT CONCAT_WS(": ", g.isys_catg_smview_cable_user_state__name, g.isys_catg_smview_cable_user_state__description)
                        FROM isys_catg_smview_cable_user_state g
                        INNER JOIN isys_catg_smview_cable_list l ON g.isys_catg_smview_cable_user_state__id = l.isys_catg_smview_cable_list__state_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_STATE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                    'isys_cmdb_dao_category_g_smview_cable',
                                    'callback_property_user_states'
                                ]
                            )
                        ]
                    ]
                ]
            ),
            'user_state' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_USER_STATE',
                'LC__CMDB__CATG__SMVIEW_CABLE__USER_STATE',
                'isys_catg_smview_cable_list__user_state',
                'isys_catg_smview_cable_list'
            )),
            'group_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__GROUP',
                        C__PROPERTY__INFO__DESCRIPTION => 'Group'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD        => 'isys_catg_smview_cable_list__group_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_group',
                        C__PROPERTY__DATA__REFERENCES   => [
                            'isys_catg_smview_cable_group',
                            'isys_catg_smview_cable_group__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT g.isys_catg_smview_cable_group__description FROM isys_catg_smview_cable_group g
                            INNER JOIN isys_catg_smview_cable_list l ON g.isys_catg_smview_cable_group__id = l.isys_catg_smview_cable_list__group_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_GROUP',
                        C__PROPERTY__UI__PARAMS => [
                                'p_arData' => new isys_callback(
                                    [
                                    'isys_cmdb_dao_category_g_smview_cable',
                                    'callback_property_groups'
                                    ]
                                )
                            ]
                    ]
                ]
            ),
            'length' => (new FloatProperty(
                'C__CATG__SMVIEW_CABLE_LENGTH',
                'LC__CMDB__CATG__SMVIEW_CABLE__LENGTH',
                'isys_catg_smview_cable_list__length',
                'isys_catg_smview_cable_list'
            ))->mergePropertyUiParams(
                [
                    'p_strClass'   => 'input-small',
                ]
            ),
            'capacity' => (new IntProperty(
                'C__CATG__SMVIEW_CABLE_CAPACITY',
                'LC__CMDB__CATG__SMVIEW_CABLE__CAPACITY',
                'isys_catg_smview_cable_list__capacity',
                'isys_catg_smview_cable_list'
            ))->mergePropertyUiParams(
                [
                    'p_strClass'   => 'input-small',
                ]
            ),
            'net_type' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_NET_TYPE',
                'LC__CMDB__CATG__SMVIEW_CABLE__NET_TYPE',
                'isys_catg_smview_cable_list__net_type',
                'isys_catg_smview_cable_list'
            )),
            'admin_unit' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__ADMIN_UNIT',
                        C__PROPERTY__INFO__DESCRIPTION => 'Administration Unit'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD        => 'isys_catg_smview_cable_list__admin_unit_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_admin_unit',
                        C__PROPERTY__DATA__REFERENCES   => [
                            'isys_catg_smview_cable_admin_unit',
                            'isys_catg_smview_cable_admin_unit__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT a.isys_catg_smview_cable_admin_unit__description FROM isys_catg_smview_cable_admin_unit a
                            INNER JOIN isys_catg_smview_cable_list l ON a.isys_catg_smview_cable_admin_unit__id = l.isys_catg_smview_cable_list__admin_unit_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_ADMIN_UNIT',
                        C__PROPERTY__UI__PARAMS => [
                                'p_arData' => new isys_callback(
                                    [
                                    'isys_cmdb_dao_category_g_smview_cable',
                                    'callback_property_admin_units'
                                    ]
                                )
                            ]
                    ]
                ]
            ),
            'type_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__TYPE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Type'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD        => 'isys_catg_smview_cable_list__type_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_type',
                        C__PROPERTY__DATA__REFERENCES   => [
                            'isys_catg_smview_cable_type',
                            'isys_catg_smview_cable_type__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT t.isys_catg_smview_cable_type__description FROM isys_catg_smview_cable_type t
                            INNER JOIN isys_catg_smview_cable_list l ON t.isys_catg_smview_cable_type__id = l.isys_catg_smview_cable_list__type_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_TYPE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable',
                                'callback_property_cable_types'
                                ]
                            )
                        ]
                    ]
                ]
            ),
            // only used for object list
            'location_in_path' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__LOCATION_IN_PATH',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD  => 'isys_catg_smview_cable_list__location_in_id',
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            $this->_build_full_location_path_query(5)
                            . ' LEFT JOIN isys_catg_smview_cable_list AS l ON main.isys_catg_location_list__isys_obj__id = l.isys_catg_smview_cable_list__location_in_id ',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id'
                        )
                    ],
                    C__PROPERTY__PROVIDES => [
                            C__PROPERTY__PROVIDES__SEARCH     => false,
                            C__PROPERTY__PROVIDES__LIST       => true,
                            C__PROPERTY__PROVIDES__REPORT     => false,
                            C__PROPERTY__PROVIDES__EXPORT     => false,
                            C__PROPERTY__PROVIDES__IMPORT     => false,
                            C__PROPERTY__PROVIDES__MULTIEDIT  => false,
                            C__PROPERTY__PROVIDES__VALIDATION => false,
                            C__PROPERTY__PROVIDES__VIRTUAL    => true
                        ]
                ]
            ),
            'location_in' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::object_browser(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__LOCATION_IN',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD      => 'isys_catg_smview_cable_list__location_in_id',
                        C__PROPERTY__DATA__REFERENCES => [
                            'isys_obj',
                            'isys_obj__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT o.isys_obj__title FROM isys_obj o
                            INNER JOIN isys_catg_smview_cable_list l ON o.isys_obj__id = l.isys_catg_smview_cable_list__location_in_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id'
                        ),
                        C__PROPERTY__DATA__JOIN => [
                            idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory(
                                'isys_catg_smview_cable_list',
                                'LEFT',
                                'isys_catg_smview_cable_list__location_in_id',
                                'isys_obj__id'
                            ),
                        ]
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_LOCATION_IN'
                    ]
                ]
            ),
            // only used for object list
            'location_out_path' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__LOCATION_OUT_PATH',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD  => 'isys_catg_smview_cable_list__location_out_id',
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            $this->_build_full_location_path_query(5)
                            . ' LEFT JOIN isys_catg_smview_cable_list AS l ON main.isys_catg_location_list__isys_obj__id = l.isys_catg_smview_cable_list__location_out_id ',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id'
                        )
                    ],
                    C__PROPERTY__PROVIDES => [
                            C__PROPERTY__PROVIDES__SEARCH     => false,
                            C__PROPERTY__PROVIDES__LIST       => true,
                            C__PROPERTY__PROVIDES__REPORT     => false,
                            C__PROPERTY__PROVIDES__EXPORT     => false,
                            C__PROPERTY__PROVIDES__IMPORT     => false,
                            C__PROPERTY__PROVIDES__MULTIEDIT  => false,
                            C__PROPERTY__PROVIDES__VALIDATION => false,
                            C__PROPERTY__PROVIDES__VIRTUAL    => true
                    ]
                ]
            ),
            'location_out' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::object_browser(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__LOCATION_OUT',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD      => 'isys_catg_smview_cable_list__location_out_id',
                        C__PROPERTY__DATA__REFERENCES => [
                            'isys_obj',
                            'isys_obj__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT o.isys_obj__title FROM isys_obj o
                            INNER JOIN isys_catg_smview_cable_list l ON o.isys_obj__id = l.isys_catg_smview_cable_list__location_out_id',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id'
                        ),
                        C__PROPERTY__DATA__JOIN => [
                            idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory(
                                'isys_catg_smview_cable_list',
                                'LEFT',
                                'isys_catg_smview_cable_list__location_out_id',
                                'isys_obj__id'
                            ),
                        ]
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_LOCATION_OUT'
                    ]
                ]
            ),
            // only used to transfer data to UI
            'generator' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR',
                        C__PROPERTY__INFO__DESCRIPTION => 'Generator'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_TYPE_GENERATOR',
                        C__PROPERTY__UI__PARAMS => [
                                'p_arData' => new isys_callback(
                                    [
                                    'isys_cmdb_dao_category_g_smview_cable',
                                    'callback_property_generators'
                                    ]
                                )
                            ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'start_pattern' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__START_PATTERN',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE_START_PATTERN'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'end_pattern' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__END_PATTERN',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE_END_PATTERN'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'scope' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__SCOPE',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE_SCOPE'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            'is_section_cable' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__IS_SECTION_CABLE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Is Section Cable'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_list__is_section_cable',
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT (CASE WHEN isys_catg_smview_cable_list__is_section_cable = \'1\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__YES') . ' WHEN isys_catg_smview_cable_list__is_section_cable = \'0\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__NO') . ' END) FROM isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list',
                            'isys_catg_smview_cable_list__id',
                            'isys_catg_smview_cable_list__isys_obj__id'
                        ),
                        C__PROPERTY__DATA__JOIN   => [
                            idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory('isys_catg_smview_cable_list', 'LEFT', 'isys_catg_smview_cable_list__isys_obj__id', 'isys_obj__id')
                        ]
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_IS_SECTION_CABLE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData'   => get_smarty_arr_YES_NO(),
                            'p_strClass' => 'input-small'
                        ]
                    ]
                ]
            ),
        ];
    }
    
    /**
     * Method which builds the location path as query.
     *
     * @param int $p_count
     *
     * @return string
     */
    private function _build_full_location_path_query($p_count = 3)
    {
        $l_separator = isys_tenantsettings::get('gui.separator.location', ' > ');
        $l_query = 'SELECT ';
        $l_joins = '';
        $l_locationPath = '';

        if ($p_count) {
            $l_locationPath = ' CONCAT_WS(\'' . $l_separator . ' \', ';
            for ($i = $p_count; $i > 0; $i--) {
                $l_locationPath .= '(SELECT CONCAT(isys_obj__title, \' {\', isys_obj__id, \'}\') FROM isys_obj WHERE sub' . $i . '.isys_catg_location_list__isys_obj__id = isys_obj__id),';
            }
            $l_locationPath .= 'o.isys_obj__title) AS title'; // add this object to path
            
            $l_previousAlias = 'main';
            for ($i = 1; $i <= $p_count; $i++) {
                $l_alias = 'sub' . $i;
                $l_joins .= ' LEFT JOIN isys_catg_location_list AS ' . $l_alias . ' ON ' . $l_alias . '.isys_catg_location_list__isys_obj__id = ' . $l_previousAlias . '.isys_catg_location_list__parentid ';
                $l_previousAlias = $l_alias;
            }
            $l_joins .= ' LEFT JOIN isys_obj AS o ON o.isys_obj__id = main.isys_catg_location_list__isys_obj__id ';
        }

        return $l_query . $l_locationPath . ' FROM isys_catg_location_list AS main ' . $l_joins;
    }
    
    /**
     * Callback method for admin units.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_admin_units(isys_request $p_request)
    {
        $dao = new isys_cmdb_dao_smview_cable_admin_unit($this->m_db);
        $res = $dao->get_data();
        $units = [];
        foreach ($res as $unit) {
            $description = $unit['description'] . ' (';
            $description .= $unit['classification'] . ')';
            $units[$unit['id']] = $description;
        }
        return $units;
    }
    
    /**
     * Callback method for groups.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_groups(isys_request $p_request)
    {
        $dao = new isys_cmdb_dao_smview_cable_group($this->m_db);
        $res = $dao->get_objects();
        $groups = [];
        foreach ($res as $group) {
            $description = $group['description'] . ' (';
            $description .= $group['classification'] . ')';
            $groups[$group['id']] = $description;
        }
        return $groups;
    }

    /**
     * Callback method for groups.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_user_states(isys_request $p_request)
    {
        $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
        $res = $dao->get_objects();
        $groups = [];
        foreach ($res as $group) {
            $description = $group['name'] . ': ' . $group['description'];
            $groups[$group['id']] = $description;
        }
        return $groups;
    }
    
    /**
     * Callback method for cable types.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_cable_types(isys_request $p_request)
    {
        $language = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_smview_cable_type($this->m_db);
        $res = $dao->get_data();
        $types = [];
        foreach ($res as $type) {
            $description = $type['description'] . ' (';
            $description .= $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_COUNT') . ': ';
            $description .= $type['wire_count'] . ')';
            $types[$type['id']] = $description;
        }
        return $types;
    }
    
    /**
     * Callback method for generators.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_generators(isys_request $p_request)
    {
        $dao = new isys_cmdb_dao_smview_cable_generator($this->m_db);
        $res = $dao->get_data();
        $generators = [];
        foreach ($res as $generator) {
            $description = $generator['title'] . ' (' . $generator['mask'] . ')';
            $generators[$generator['id']] = $description;
        }
        return $generators;
    }

    /**
     * Find usages of the given object.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function find_usages($obj_id)
    {
        $wire_dao = new isys_cmdb_dao_smview_cable_wire($this->m_db);
        $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire($this->m_db);

        // note: obj_id of wire and cable are the same
        $wires = $wire_dao->filter_objects(array('obj_id' => $obj_id));

        $cable_usages = [];
        foreach ($wires as $e) {
            // get usage of each wire
            $wire_usages = $wire_catg->find_usages($e['id']);
            if (! empty($wire_usages)) {
                array_push($cable_usages, $wire_usages);
            }
        }
        return $cable_usages;
    }
    
    /**
     * Validates property data.
     *
     * @param  array $p_data                Associative array of property tags as keys and their values as values.
     * @param  mixed $p_prepend_table_field This can be used to prepend a table field alias (use boolean "true" for the default category table).
     * @return mixed  Returns true on a successful validation, otherwise an associative array with property tags as keys and error messages as values.
     */
    public function validate(array $p_data = [], $p_prepend_table_field = false)
    {
        $language = isys_application::instance()->container->get('language');
        $l_result = parent::validate($p_data, $p_prepend_table_field);
        $l_return = $l_result === true ? [] : $l_result;
        
        if (empty($p_data['generator']) || $p_data['generator'] == '-1') {
            return empty($l_return) ? true : $l_return;
        }
        
        $required_values = array(
            'start_pattern',
            'end_pattern',
        );
        foreach ($required_values as $required_value) {
            if (! isset($p_data[$required_value]) || $p_data[$required_value] == '') {
                $l_return[$required_value] = $language->get('LC__UNIVERSAL__MANDATORY_FIELD_IS_EMPTY');
            }
        }
        return empty($l_return) ? true : $l_return;
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return int  Returns created entity's identifier (int) or false (bool).
     * @throws Exception
     */
    public function create_data($p_data)
    {
        $dao = new isys_cmdb_dao_smview_cable($this->m_db);
        $originalObjId = isset($_POST['table_ids']) && count($_POST['table_ids']) == 1 ? $_POST['table_ids'][0] : null;
        $template = $originalObjId ? $dao->get_object('obj_id', $originalObjId) : [];
        $location_in = $this->_get_location($p_data, 'location_in');
        $location_out = $this->_get_location($p_data, 'location_out');

        $params = [
            'obj_id' => $p_data['isys_obj__id'],
        ];
        $optionalParams = [
            'state_id'         => isset($p_data['state_id']) && $p_data['state_id'] != '-1' ? $p_data['state_id'] : $template['state_id'],
            'group_id'         => isset($p_data['group_id']) && $p_data['group_id'] != '-1' ? $p_data['group_id'] : $template['group_id'],
            'length'           => $p_data['length'],
            'capacity'         => $p_data['capacity'] ?? null,
            'net_type'         => $p_data['net_type'] ?? null,
            'admin_unit_id'    => isset($p_data['admin_unit']) && $p_data['admin_unit'] != '-1' ? $p_data['admin_unit'] : $template['admin_unit_id'],
            'type_id'          => isset($p_data['type_id']) && $p_data['type_id'] != '-1' ? $p_data['type_id'] : $template['type_id'],
            'location_in_id'   => $location_in,
            'location_out_id'  => $location_out,
            'user_state'       => $p_data['user_state'] ?? null,
            'is_section_cable' => $p_data['is_section_cable'] ?? null,
        ];
        $id = $dao->create_object($params, $optionalParams);

        // generate wires
        if (! empty($p_data['generator']) && $p_data['generator'] != '-1') {
            $generator_id = $p_data['generator'];
            $start_pattern = isset($p_data['start_pattern']) ? $p_data['start_pattern'] : 1;
            $end_pattern = isset($p_data['end_pattern']) ? $p_data['end_pattern'] : null;
            $scope = isset($p_data['scope']) ? $p_data['scope'] : null;
            try {
                $this->generate_wires($p_data['isys_obj__id'], $generator_id, $start_pattern, $end_pattern, $scope);
            } catch (isys_exception_validation $e) {
                // todo: handle error
            }
        }
        return $id;
    }

    /**
     * Updates existing entity.
     *
     * @param  int $p_list_id Entity's identifier
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws Exception
     */
    public function save_data($p_list_id, $p_data)
    {
        $location_in = $this->_get_location($p_data, 'location_in');
        $location_out = $this->_get_location($p_data, 'location_out');
        $dao = new isys_cmdb_dao_smview_cable($this->m_db);

        $optionalParams = [
            'state_id'         => isset($p_data['state_id']) && $p_data['state_id'] != '-1' ? $p_data['state_id'] : null,
            'group_id'         => isset($p_data['group_id']) && $p_data['group_id'] != '-1' ? $p_data['group_id'] : null,
            'length'           => $p_data['length'],
            'capacity'         => $p_data['capacity'] ?? null,
            'net_type'         => $p_data['net_type'] ?? null,
            'admin_unit_id'    => isset($p_data['admin_unit']) && $p_data['admin_unit'] != '-1' ? $p_data['admin_unit'] : null,
            'type_id'          => isset($p_data['type_id']) && $p_data['type_id'] != '-1' ? $p_data['type_id'] : null,
            'location_in_id'   => $location_in,
            'location_out_id'  => $location_out,
            'user_state'       => $p_data['user_state'] ?? null,
            'is_section_cable' => $p_data['is_section_cable'] ?? null,
        ];
        $success = $dao->update_object($p_list_id, [], $optionalParams);

        // generate wires
        if (! empty($p_data['generator']) && $p_data['generator'] != '-1') {
            $generator_id = $p_data['generator'];
            $start_pattern = isset($p_data['start_pattern']) ? $p_data['start_pattern'] : 1;
            $end_pattern = isset($p_data['end_pattern']) ? $p_data['end_pattern'] : null;
            $scope = isset($p_data['scope']) ? $p_data['scope'] : null;
            try {
                $this->generate_wires($p_data['isys_obj__id'], $generator_id, $start_pattern, $end_pattern, $scope);
            } catch (InvalidArgumentException $e) {
                // todo: handle error
            }
        }
        
        return $success;
    }
    
    /**
     * Get location selection.
     *
     * @param  array  $p_data
     * @param  string $key
     * @return int|null
     */
    private function _get_location($p_data, $key)
    {
        if (empty($p_data[$key])) {
            return null;
        }
        
        $location = $p_data[$key];
        if (strpos($location, '[') !== false) {
            // handle multiple values as (json) array
            $l = json_decode($location);
            return $l[0];
        }
        return $location;
    }

    /**
     * Generate wires by cable type.
     *
     * @param  int $obj_id
     * @param  int $generator_id
     * @param  string $start_pattern
     * @param  string $end_pattern
     * @param  string $scope
     * @throws Exception
     */
    public function generate_wires($obj_id, $generator_id, $start_pattern = null, $end_pattern = null, $scope = null)
    {
        if (! isset($start_pattern) || ! isset($end_pattern)) {
            throw new InvalidArgumentException('start-pattern or end-pattern is empty');
        }
        
        $name_generator = new LabelGenerator($generator_id);
        $names = $name_generator->generate_names_by_generator($start_pattern, $end_pattern);
        
        $wire_dao = new isys_cmdb_dao_smview_cable_wire($this->m_db);
        foreach ($names as $name) {
            $wire_dao->create_wire($obj_id, $name, '', $scope);
        }
    }
}
