<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable ETSI Bay.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_etsi_bay extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_etsi_bay';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * @var string
     */
    protected $m_object_id_field = 'isys_catg_smview_cable_etsi_bay_list__isys_obj__id';

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        return [
            'description' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Description'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_list__description'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_DESCRIPTION'
                    ]
                ]
            ),
            'name' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__NAME',
                        C__PROPERTY__INFO__DESCRIPTION => 'Name'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_list__name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_NAME'
                    ]
                ]
            ),
            'he' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::int(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__HE',
                        C__PROPERTY__INFO__DESCRIPTION => 'HE'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_list__he',
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT (CASE WHEN isys_catg_smview_cable_etsi_bay_list__he = \'1\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__YES') . ' WHEN isys_catg_smview_cable_etsi_bay_list__he = \'0\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__NO') . ' END) FROM isys_catg_smview_cable_etsi_bay_list',
                            'isys_catg_smview_cable_etsi_bay_list',
                            'isys_catg_smview_cable_etsi_bay_list__id',
                            'isys_catg_smview_cable_etsi_bay_list__isys_obj__id'
                        ),
                        C__PROPERTY__DATA__JOIN   => [
                            idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory('isys_catg_smview_cable_etsi_bay_list', 'LEFT', 'isys_catg_smview_cable_etsi_bay_list__isys_obj__id', 'isys_obj__id')
                        ]
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_HE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_strClass' => 'input-small'
                        ]
                    ]
                ]
            ),
            'use_half_he' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__USE_HALF_HE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Use Half HE'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_list__use_half_he'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_USE_HALF_HE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => get_smarty_arr_YES_NO(),
                            'p_strClass' => 'input-small'
                        ]
                    ]
                ]
            ),
        ];
    }

    /**
     * Get bay by ID.
     *
     * @param  int $id
     * @param  array|null $scopeData
     * @return array|null
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function get_bay_by_id($id, $scopeData = null)
    {
        $l_catdata = $this->get_data($id)->get_row();
        if (! $l_catdata) {
            return null;
        }

        $obj_id = $l_catdata['isys_catg_smview_cable_etsi_bay_list__isys_obj__id'];
        if (empty($scopeData)) {
            $scopeData = $this->get_scope_data($obj_id);
        }

        // get this bay from available bays
        $availableBays = $this->get_bays($obj_id, $id, $scopeData);
        $availableBayItems = [];
        foreach ($availableBays as $b) {
            if ($b['id'] == $l_catdata['isys_catg_smview_cable_etsi_bay_list__name']) {
                $availableBayItems = $b['items'];
                break;
            }
        }

        $dao = new isys_cmdb_dao_smview_cable_etsi_bay_item($this->m_db);
        $items = $dao->filter_objects(['bay_id' => $id]);
        $assigned_bay_items = [];
        foreach ($items as $index => $item) {
            // check if item already exists
            $itemExists = false;
            foreach ($availableBayItems as $c) {
                if ($c['id'] == $item['title']) {
                    $itemExists = true;
                }
            }

            if ($item['status'] != 'planning' && ! $itemExists) {
                // remove obsolete item
                $dao->delete_object($item['id']);
                continue;
            }

            $scope = $l_catdata['isys_catg_smview_cable_etsi_bay_list__name'] . $item['title'];
            $itemId = $item['status'] == 'planning' ? $index . '_' : '';
            $itemId .= $item['title'];
            $assigned_bay_items[] = [
                'id'                 => $itemId,
                'title'              => $item['title'],
                'color'              => $item['color_id'],
                'short_name'         => $item['short_name'],
                'position'           => $item['position'],
                'he'                 => $item['he'],
                'project_identifier' => $item['project_identifier'],
                'comment'            => $item['comment'],
                'status'             => $item['status'],
                'wire_bundles'       => $scopeData[$scope] ?: [],
            ];
        }

        return [
            'id'          => $l_catdata['isys_catg_smview_cable_etsi_bay_list__name'],
            'title'       => $l_catdata['isys_catg_smview_cable_etsi_bay_list__name'],
            'he'          => $l_catdata['isys_catg_smview_cable_etsi_bay_list__he'],
            'use_half_he' => $l_catdata['isys_catg_smview_cable_etsi_bay_list__use_half_he'],
            'items'       => $assigned_bay_items,
        ];
    }

    /**
     * Get scope data of current object.
     *
     * @param  int $object_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_scope_data($object_id)
    {
        $connectionView = new isys_cmdb_dao_smview_cable_connection_view($this->m_db);
        $connectionsIn = $connectionView->filter_objects(['obj_id_in' => $object_id]);
        $connectionsOut = $connectionView->filter_objects(['obj_id_out' => $object_id]);
        $connections = array_merge($connectionsIn, $connectionsOut);

        $userStateDao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
        $stateObjects = $userStateDao->get_objects();
        $states = [];
        foreach ($stateObjects as $s) {
            $states[$s['id']] = $s['name'];
        }

        $wireStateDao = new isys_cmdb_dao_smview_cable_wire_pin_status($this->m_db);
        $wireStateObjects = $wireStateDao->get_objects();
        $wireStates = [];
        foreach ($wireStateObjects as $s) {
            $wireStates[$s['id']] = $s['const'];
        }

        $scopeData = [];
        foreach ($connections as $c) {
            $side = $c['obj_id_in'] == $object_id ? '_in' : '_out';
            $sibSide = $c['obj_id_in'] == $object_id ? '_out' : '_in';
            $wireStateId = $c['wire_pin_state' . $side];
            $data = [
                'pin_id'               => $c['pin_id' . $side],
                'pin_description'      => $c['pin_title' . $side],
                'cable_id'             => $c['cable_id'],
                'cable_title'          => $c['cable_title'],
                'wire_scope'           => $c['wire_scope'],
                'wire_title'           => $c['wire_title'],
                'wire_state_id'        => $wireStateId,
                'wire_state_name'      => $wireStates[$wireStateId],
                'user_state_name'      => $states[$c['user_state_id']],
                'sibling_object_id'    => $c['obj_id' . $sibSide],
                'sibling_object_title' => $c['obj_title' . $sibSide],
                'sibling_pin_scope'    => $c['pin_scope' . $sibSide],
            ];

            $pinScope = $c['pin_scope' . $side];
            $pinDescription = $c['pin_title' . $side];
            $scopeData[$pinScope][$pinDescription] = $data;
        }
        return $this->_group_scope_data($scopeData);
    }

    /**
     * Group scope data.
     *
     * @param  array $scopeData
     * @return array
     * @throws isys_exception_database
     */
    private function _group_scope_data($scopeData)
    {
        $dao = new isys_cmdb_dao_smview_cable_etsi_device_color($this->m_db);
        $device_colors = $dao->get_objects();
        $groupedData = [];
        foreach ($scopeData as $scope => $pins) {
            $wireBundles = [];
            $startWire = null;
            $prevWire = null;
            $prevCable = null;
            $prevWireState = null;
            $prevSibObj = null;
            $prevConnection = null;
            $prevWireStates = [];

            // sort PIN titles
            ksort($pins);

            foreach ($pins as $connection) {
                $wireNr = (int) $connection['wire_title'];
                $startWire = ! $startWire ? $wireNr : $startWire;

                $cableName = $connection['cable_title'];
                $prevCable = ! $prevCable ? $cableName : $prevCable;

                $wireState = $connection['wire_state_name'];
                $prevWireState = ! $prevWireState ? $wireState : $prevWireState;

                $sibObj = $connection['sibling_object_id'];
                $prevSibObj = ! $prevSibObj ? $sibObj : $prevSibObj;

                if ($wireNr < $startWire || $prevCable != $cableName || $prevWireState != $wireState || $prevSibObj != $sibObj) {
                    // save wire bundle
                    $wireBundles[] = $this->_add_wire_bundle($startWire, $prevWire, $prevConnection, $device_colors, $prevWireStates);

                    // update previous values
                    $startWire = $wireNr;
                    $prevCable = $cableName;
                    $prevWireState = $wireState;
                    $prevSibObj = $sibObj;
                }
                $prevWire = $wireNr;
                $prevConnection = $connection;
                if (! empty($connection['wire_state_id'])) {
                    $prevWireStates[$connection['wire_state_id']] = $connection['wire_state_id'];
                }
            }

            $wireBundles[] = $this->_add_wire_bundle($startWire, $prevWire, $prevConnection, $device_colors, $prevWireStates);
            $groupedData[$scope] = $wireBundles;
        }
        return $groupedData;
    }

    /**
     * Add wire bundle.
     *
     * @param  int $wireStart
     * @param  int $wireEnd
     * @param  array $connection
     * @param  array $device_colors
     * @return array
     */
    private function _add_wire_bundle($wireStart, $wireEnd, $connection, $device_colors = [], $usedWireStates = [])
    {
        $cableUrl = isys_helper_link::get_base() . isys_helper_link::create_url([
                C__CMDB__GET__OBJECT => $connection['cable_id'],
                C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_CABLE_WIRE'),
            ]);
        $objectUrl = isys_helper_link::get_base() . isys_helper_link::create_url([
                C__CMDB__GET__OBJECT => $connection['sibling_object_id'],
                C__CMDB__GET__CATG   => defined_or_default('C__CATG__SMVIEW_CABLE_PIN'),
            ]);

        $wireCount = ((int) $wireEnd - (int) $wireStart);
        $wireCount = $wireCount > 1 ? ++$wireCount : $wireCount; // handle off by one
        $colors = $this->_get_wire_bundle_colors($wireCount, $device_colors, $usedWireStates);

        return [
            'cable_name'           => $connection['cable_title'],
            'cable_url'            => $cableUrl,
            'wire_start'           => $wireStart,
            'wire_end'             => $wireEnd,
            'wire_state'           => $connection['wire_state_name'],
            'device_colors'        => $colors,
            'sibling_object_title' => $connection['sibling_object_title'],
            'sibling_pin_scope'    => $connection['sibling_pin_scope'],
            'sibling_object_url'   => $objectUrl,
        ];
    }

    /**
     * Get possible device colors by wire bundle.
     *
     * @param  int $wireCount
     * @param  array $device_colors
     * @param  array $usedWireStates
     * @return array
     * @throws isys_exception_database
     */
    private function _get_wire_bundle_colors($wireCount, $device_colors, $usedWireStates)
    {
        $dao = new isys_cmdb_dao_smview_cable_etsi_device_color_wire_status(isys_application::instance()->database);
        $colors = [];
        foreach ($device_colors as $c) {
            if ($c['wire_count'] != $wireCount) {
                continue;
            }

            $wire_status = $dao->filter_objects(['device_id' => $c['id']]);
            $matches = false;
            foreach ($wire_status as $ws) {
                $operator = $ws['operator'];
                $wirePinStatusId = $ws['status_id'];
                $wireStateIsUsed = isset($usedWireStates[$wirePinStatusId]);
                if ($operator == 'AND') {
                    if (! $wireStateIsUsed) {
                        // todo: break if other states are in array
                        $matches = false;
                        break;
                    } else {
                        $matches = true;
                    }
                }

                if ($operator == 'OR' && $wireStateIsUsed) {
                    $matches = true;
                }

                if ($ws['not'] && $wireStateIsUsed) {
                    $matches = false;
                    break;
                }
            }

            if ($matches) {
                // check if there are unhandled status within used status
                $matches = ! (count($usedWireStates) > $wire_status);
            }

            if ($matches) {
                $colors[$c['id']] = $c['description'];
            }

//            if (! empty($connection['wire_state_id'])) {
//                if ($c['wire_state_id'] != $connection['wire_state_id']) {
//                    continue;
//                }
//            }
//            $colors[$c['id']] = $c['description'];
        }
        return $colors;
    }

    /**
     * Get all bays of this object.
     *
     * @param  int $obj_id
     * @param  int $list_id
     * @param  array $scopeData
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function get_bays($obj_id, $list_id, $scopeData)
    {
        $pinDao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        $pinScopes = $pinDao->get_scopes_by_obj_id($obj_id);
        $scopes = [];
        foreach ($pinScopes as $scope) {
            $matches = preg_split('/(?<=\D)(?=\d)|\d+\K/', $scope);
            if (count($matches) >= 4) {
                $bay = $matches[0] . $matches[1];
                $bayItem = '';
                $items = array_slice($matches, 2);
                foreach ($items as $i) {
                    if (! empty($i)) {
                        $bayItem .= $i;
                    }
                }
                $scopes[$bay][$bayItem] = $scopeData[$scope];
            }
        }

        $bays = [];
        $is_new = ! $list_id || $list_id <= 0;
        $assigned_bays = [];
        $res = $this->get_data(null, $obj_id);
        while ($row = $res->get_row()) {
            $name = $row['isys_catg_smview_cable_etsi_bay_list__name'];
            $assigned_bays[$name] = $name;
        }

        foreach ($scopes as $name => $items) {
            $bayItems = [];
            foreach ($items as $i => $wire_bundles) {
                $bayItems[] = [
                    'id'           => $i,
                    'title'        => $i,
                    'short_name'   => $i,
                    'he'           => 1,
                    'wire_bundles' => $wire_bundles,
                ];
            }
            if ($is_new && $assigned_bays[$name]) {
                // do not add assigned bays
                continue;
            }
            $bays[] = [
                'id'    => $name,
                'title' => $name,
                'items' => $bayItems,
            ];
        }
        return $bays;
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    public function create_data($p_data)
    {
        $originalObjId = isset($_POST['table_ids']) && count($_POST['table_ids']) == 1 ? $_POST['table_ids'][0] : null;
        if (! $originalObjId) {
            // handle save from UI
            return $this->_handleSave($p_data);
        }

        // handle duplication
        $scopeData = $this->get_scope_data($originalObjId);
        $res = $this->get_data(null, $originalObjId);
        $p_list_id = null;
        while ($row = $res->get_row()) {
            if ($row['isys_catg_smview_cable_etsi_bay_list__name'] != $p_data['name']) {
                continue;
            }

            $cat_data_id = $row['isys_catg_smview_cable_etsi_bay_list__id'];
            $bay = $this->get_bay_by_id($cat_data_id, $scopeData);

            $p_data['description'] = $bay['title'];
            $p_data['use_half_he'] = $bay['use_half_he'];
            $p_data['name'] = $bay['title'];
            $p_data['he'] = $bay['he'];
            $p_data['status'] = 2;

            $p_list_id = parent::create_data($p_data);
            $this->_assignBayItems($p_list_id, $bay['items']);
        }
        return $p_list_id;
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws Exception
     */
    public function save_data($p_list_id, $p_data)
    {
        return $this->_handleSave($p_data, $p_list_id);
    }

    /**
     * Handle save.
     *
     * @param  array $p_data
     * @param  int|null $p_list_id
     * @return boolean|mixed
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _handleSave($p_data, $p_list_id = null)
    {
        $dao = new isys_cmdb_dao_smview_cable_etsi_bay_item($this->m_db);
        $items = json_decode($_POST['C__CATG__SMVIEW_CABLE_ETSI_BAY_ITEMS'], true);

        // add name and HE
        $p_data['name'] = $items['bay_id'];
        $p_data['he'] = $items['he'];
        if ($p_list_id) {
            $ret = parent::save_data($p_list_id, $p_data);

            // remove current items
            $bay_items = $dao->filter_objects(['bay_id' => $p_list_id]);
            foreach ($bay_items as $item) {
                $dao->delete_object($item['id']);
            }
        } else {
            $ret = parent::create_data($p_data);
            $p_list_id = $ret;
        }

        $this->_assignBayItems($p_list_id, $items['assigned_bay_items']);
        return $ret;
    }

    /**
     * Assign bay items.
     *
     * @param  int $p_list_id
     * @param  array $assigned_bay_items
     * @throws isys_exception_dao
     */
    private function _assignBayItems($p_list_id, $assigned_bay_items)
    {
        // add new items
        $dao = new isys_cmdb_dao_smview_cable_etsi_bay_item($this->m_db);
        foreach ($assigned_bay_items as $item) {
            $params = [
                'bay_id'   => $p_list_id,
                'title'    => ! empty($item['title']) ? $item['title'] : $item['id'],
                'position' => $item['position'],
                'he'       => $item['he'],
            ];
            $optParams = [
                'color_id'           => $item['color'] != '-1' ? $item['color'] : null,
                'short_name'         => $item['short_name'] != '' ? $item['short_name'] : null,
                'project_identifier' => $item['project_identifier'] != '' ? $item['project_identifier'] : null,
                'comment'            => $item['comment'] != '' ? $item['comment'] : null,
                'status'             => $item['status'] != '' ? $item['status'] : null,
            ];
            $dao->create_object($params, $optParams);
        }
    }

    /**
     * Update method.
     *
     * Set involved export configurations to outdated, if a bay will be deleted.
     *
     * @param  string $p_query
     * @return boolean
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function update($p_query)
    {
        if (strpos($p_query, '__status') === false) {
            // no status will be changed
            return parent::update($p_query);
        }

        if (strpos($p_query, "__status = '2'") !== false) {
            // status can be changed to 2 ("Normal")
            return parent::update($p_query);
        }

        // check if bay is part of an export configuration
        $list_pattern = "/list__id\s?=\s?[']?([\d]*)[']?\s?/";
        $matches = [];
        $res = preg_match($list_pattern, $p_query, $matches);
        if ($res !== false) {
            // found list ID
            $list_id = (int) $matches[1];
            $bay = $this->get_data($list_id)->get_row();

            // get export configurations with this bay
            $export_dao = new isys_cmdb_dao_category_g_smview_cable_etsi_bay_export($this->m_db);
            $exports = $export_dao->get_data(null, $bay['isys_obj__id']);
            while ($export = $exports->get_row()) {
                // flag export configuration as outdated
                $export_dao->set_outdated($export['isys_catg_smview_cable_etsi_bay_export_list__id']);
            }
        }
        return parent::update($p_query);
    }
}
