<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable ETSI Bay Export.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_etsi_bay_export extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_etsi_bay_export';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * @var string
     */
    protected $m_object_id_field = 'isys_catg_smview_cable_etsi_bay_export_list__isys_obj__id';

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        return [
            'description' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Description'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_export_list__description'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_DESCRIPTION'
                    ]
                ]
            ),
            'is_outdated' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__IS_OUTDATED',
                        C__PROPERTY__INFO__DESCRIPTION => 'Is Outdated'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_etsi_bay_export_list__is_outdated'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_IS_OUTDATED'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
        ];
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    public function create_data($p_data)
    {
        return $this->_handleSave($p_data);
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws Exception
     */
    public function save_data($p_list_id, $p_data)
    {
        return $this->_handleSave($p_data, $p_list_id);
    }

    /**
     * Handle save.
     *
     * @param  array $p_data
     * @param  int|null $p_list_id
     * @return boolean|mixed
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _handleSave($p_data, $p_list_id = null)
    {
        $dao = new isys_cmdb_dao_smview_cable_etsi_bay_item_export($this->m_db);
        $items = json_decode($_POST['C__CATG__SMVIEW_CABLE_ETSI_EXPORT'], true);
        if ($p_list_id) {
            $p_data['is_outdated'] = 0;
            $ret = parent::save_data($p_list_id, $p_data);

            // remove current items
            $bay_items = $dao->filter_objects(['list_id' => $p_list_id]);
            foreach ($bay_items as $item) {
                $dao->delete_object($item['id']);
            }
        } else {
            $ret = parent::create_data($p_data);
            $p_list_id = $ret;
        }

        // add new items
        foreach ($items['racks'] as $item) {
            $params = [
                'list_id'     => $p_list_id,
                'bay_item_id' => $item['id'],
                'position'    => $item['position'],
            ];
            $dao->create_object($params);
        }
        return $ret;
    }

    /**
     * Mark this export as outdated.
     *
     * @param  int $p_list_id
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    public function set_outdated($p_list_id)
    {
        $p_data = [
            'is_outdated' => 1
        ];
        return parent::save_data($p_list_id, $p_data);
    }
}
