<?php

use idoit\Module\Cable\LabelGenerator;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable Pins.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_pin extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_pin';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * @var string
     */
    protected $m_object_id_field = 'isys_catg_smview_cable_pin_list__isys_obj__id';

    /**
     * Cache to store pins of an object.
     *
     * @var array
     */
    private $_object_pin_cache = [];

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     * @author Leonard Fischer <lfischer@i-doit.org>
     */
    protected function properties()
    {
        return [
            'description' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Description'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__description'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_DESCRIPTION'
                    ]
                ]
            ),
            'number' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::int(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__NUMBER',
                        C__PROPERTY__INFO__DESCRIPTION => 'Number'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__number'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_NUMBER',
                        C__PROPERTY__UI__PARAMS => [
                            'p_strClass' => 'input-small'
                        ]
                    ]
                ]
            ),
            'type' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__TYPE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Type'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__type'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE',
                        C__PROPERTY__UI__PARAMS => [
                                'p_arData' => new isys_callback(
                                    [
                                    'isys_cmdb_dao_category_g_smview_cable_pin',
                                    'callback_property_types'
                                    ]
                                ),
                                'p_strClass' => 'input-small'
                            ]
                    ]
                ]
            ),
            'internal_pin_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__CORRESPONDING_PIN',
                        C__PROPERTY__INFO__DESCRIPTION => 'Corresponding Pin'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__sibling_id',
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CORRESPONDING_PIN',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_pin',
                                'callback_property_corresponding_pins'
                                ]
                            )
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => false,
                        C__PROPERTY__PROVIDES__LIST   => false,
                        C__PROPERTY__PROVIDES__REPORT => false
                    ]
                ]
            ),
            'patch_pin_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__PATCH_PIN',
                        C__PROPERTY__INFO__DESCRIPTION => 'Patch Pin'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__patch_id'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_PATCH_PIN',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_pin',
                                'callback_property_patch_pins'
                                ]
                            )
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => false,
                        C__PROPERTY__PROVIDES__LIST   => false,
                        C__PROPERTY__PROVIDES__REPORT => false
                    ]
                ]
            ),
            'connector_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__CONNECTOR',
                        C__PROPERTY__INFO__DESCRIPTION => 'Connector'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__connector_id',
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CONNECTOR',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                    'isys_cmdb_dao_category_g_smview_cable_pin',
                                    'callback_property_connectors'
                                ]
                            )
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => false,
                        C__PROPERTY__PROVIDES__LIST   => false,
                        C__PROPERTY__PROVIDES__REPORT => false
                    ]
                ]
            ),
            'classification' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_CLASSIFICATION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Classification'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_pin_list__classification'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CLASSIFICATION',
                    ]
                ]
            ),
            'connection' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CONNECTION',
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'generator' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR',
                        C__PROPERTY__INFO__DESCRIPTION => 'Generator'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_GENERATOR',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable',
                                'callback_property_generators'
                                ]
                            )
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'generate_corresponding' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_GENERATE_CORRESPONDING',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => get_smarty_arr_YES_NO(),
                            'p_strClass' => 'input-small'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'start_pattern' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__START_PATTERN',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE_START_PATTERN'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'end_pattern' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__END_PATTERN',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_TYPE_END_PATTERN'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
        ];
    }

    /**
     * Callback method for types.
     *
     * @return array
     * @throws Exception
     */
    public function callback_property_types()
    {
        $language = isys_application::instance()->container->get('language');
        return array(
            isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT  => $language->get('LC__CMDB__CATG__SMVIEW_CABLE__TYPE_IN'),
            isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT => $language->get('LC__CMDB__CATG__SMVIEW_CABLE__TYPE_OUT'),
        );
    }

    /**
     * Callback method for corresponding pins.
     *
     * @param  isys_request $p_request
     * @return array
     * @throws Exception
     */
    public function callback_property_corresponding_pins(isys_request $p_request)
    {
        $obj_id = $p_request->get_object_id();
        $pin_id = $p_request->get_category_data_id();
        if (empty($obj_id)) {
            return [];
        }
        return $this->_get_all_pins($obj_id, $pin_id);
    }

    /**
     * Callback method for patch pins.
     *
     * @param  isys_request $p_request
     * @return array
     * @throws Exception
     */
    public function callback_property_patch_pins(isys_request $p_request)
    {
        $obj_id = $p_request->get_object_id();
        $pin_id = $p_request->get_category_data_id();
        if (empty($obj_id)) {
            return [];
        }
        return $this->_get_all_pins($obj_id, $pin_id, isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT);
    }

    /**
     * Callback method for connectors.
     *
     * @param  isys_request $p_request
     * @return array
     */
    public function callback_property_connectors(isys_request $p_request)
    {
        if (empty($p_request->get_object_id())) {
            return [];
        }

        // get all connectors of this object
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $l_data = $connector_dao->get_data(
            null,
            $p_request->get_object_id(),
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT
        );

        $connectors = [];
        while ($row = $l_data->get_row()) {
            $id = $row['isys_catg_connector_list__id'];
            $obj_title = $row['isys_obj__title'];
            $title = $row['isys_catg_connector_list__title'];
            $type = $row['isys_connection_type__title'];
            $connectors[$id] = "$obj_title - $title  ($type)";
        }
        return $connectors;
    }

    /**
     * Get all pins of this device.
     *
     * @param  int $obj_id
     * @param  int|null $pin_id
     * @param  string|null $search_type
     * @return array
     * @throws Exception
     */
    private function _get_all_pins($obj_id, $pin_id = null, $search_type = null)
    {
        $dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        if (! isset($this->_object_pin_cache[$obj_id])) {
            $res = $dao->filter_objects(array('obj_id' => $obj_id));
            $this->_object_pin_cache[$obj_id] = $res;
        } else {
            $res = $this->_object_pin_cache[$obj_id];
        }

        $lang = isys_application::instance()->container->get('language');
        $type_label_in = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IN_SHORT');
        $type_label_out = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_OUT_SHORT');

        $pins = [];
        foreach ($res as $pin) {
            if ($pin['id'] == $pin_id) {
                continue;
            }
            if ($search_type && $search_type != $pin['type']) {
                continue;
            }
            
            $type_label = $type_label_in;
            if ($pin['type'] == isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT) {
                $type_label = $type_label_out;
            }
            $pins[$pin['id']] = $pin['description'] . " ($type_label)";
        }
        return $pins;
    }

    /**
     * Find usages of the given object.
     *
     * @param  int $pin_id
     * @return array
     * @throws isys_exception_database
     */
    public function find_usages($pin_id)
    {
        $wire_pin_dao = new isys_cmdb_dao_smview_cable_wire_pin($this->m_db);
        $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire($this->m_db);
        $wire_pin = $wire_pin_dao->get_object('connected_pin_id', $pin_id);
        if (empty($wire_pin)) {
            return [];
        }
        return $wire_catg->find_usages($wire_pin['wire_id']);
    }

    /**
     * Validates property data.
     *
     * @param  array $p_data Associative array of property tags as keys and their values as values.
     * @param  mixed $p_prepend_table_field This can be used to prepend a table field alias (use boolean "true" for the default category table).
     * @return mixed  Returns true on a successful validation, otherwise an associative array with property tags as keys and error messages as values.
     * @throws Exception
     */
    public function validate(array $p_data = [], $p_prepend_table_field = false)
    {
        $l_result = parent::validate($p_data, $p_prepend_table_field);
        $l_return = $l_result === true ? [] : $l_result;
        
        if (empty($p_data['generator']) || $p_data['generator'] == '-1') {
            return empty($l_return) ? true : $l_return;
        }
        
        $required_values = array(
            'start_pattern',
            'end_pattern',
        );

        $lang = isys_application::instance()->container->get('language');
        foreach ($required_values as $required_value) {
            if (! isset($p_data[$required_value]) || $p_data[$required_value] == '') {
                $l_return[$required_value] = $lang->get('LC__UNIVERSAL__MANDATORY_FIELD_IS_EMPTY');
            }
        }
        return empty($l_return) ? true : $l_return;
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_database
     */
    public function create_data($p_data)
    {
        $classification = isset($p_data['classification']) ? $p_data['classification'] : null;
        if (! empty($p_data['generator']) && $p_data['generator'] != '-1') {
            // switch to mass creation
            $generator_id = isset($p_data['generator']) ? $p_data['generator'] : null;
            $start_pattern = isset($p_data['start_pattern']) ? $p_data['start_pattern'] : 1;
            $end_pattern = isset($p_data['end_pattern']) ? $p_data['end_pattern'] : null;
            $generate_corresponding = isset($p_data['generate_corresponding']) && $p_data['generate_corresponding'] == '1';
            $this->_generate_pins($p_data['isys_obj__id'], $generator_id, $start_pattern, $end_pattern, $generate_corresponding, $classification);
            return 0;
        }

        $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT;
        if ($p_data['type'] == isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT) {
            $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT;
        }
        $internal_pin_id = isset($p_data['internal_pin_id']) && $p_data['internal_pin_id'] != '-1' ? $p_data['internal_pin_id'] : null;
        $patch_pin_id = isset($p_data['patch_pin_id']) && $p_data['patch_pin_id'] != '-1' ? $p_data['patch_pin_id'] : null;
        $connector_id = isset($p_data['connector_id']) && $p_data['connector_id'] != '-1' ? $p_data['connector_id'] : null;

        /*
         * ACHTUNG: Ab hier wird es ganz wild!
         *
         * Hier wird nachgeschaut, ob ueber die GUI ein Objekt dupliziert werden soll.
         * Falls ja, wird versucht dessen PINs zu kopieren.
         */
        $templateObjId = null;
        if (isset($_POST['duplicate']) && $_POST['duplicate'] == '1') {
            // special handling on duplication via UI
            if (isset($_POST['id']) && count($_POST['id']) == 1) {
                $templateObjId = $_POST['id'][0];
            }
        }

        /*
         * Hier wird es noch verrueckter.
         *
         * Ueber den Patch-Importer wird intern eine Variable gesetzt, welche die Objekt-ID des zu duplizierenden
         * Objektes enthaelt.
         * Falls diese Variable existiert wird ebenfalls versucht die PINs des zu duplizierenden Objektes zu kopieren.
         *
         * Die Variable wird im Patch-Modul in /src/Importer/Duplicator.php -> duplicate() Zeile 76 gesetzt.
         */
        try {
            $request = isys_application::instance()->container->get('duplicator_request');
            $templateObjId = ! empty($request) ? $request->object_id : $templateObjId;
        } catch (Exception $e) {
        }

        try {
            if (! empty($templateObjId)) {
                $pins = $this->_handle_duplication($p_data, $templateObjId);
                $internal_pin_id = $internal_pin_id ?: $pins['internal_pin_id'];
                $patch_pin_id = $patch_pin_id ?: $pins['patch_pin_id'];
            }
        } catch (Exception $e) {
        }
        /*
         * Ok, dass war der Grusel-Code!
         */

        $dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        $pin_id = $dao->create_pin($p_data['isys_obj__id'], $p_data['description'], $p_data['number'], $type, $internal_pin_id, $patch_pin_id, $connector_id, $classification);

        if (! empty($p_data['connection'])) {
            $wire_pin_id = $p_data['connection'];
            $this->_save_connection($wire_pin_id, $pin_id);
        }

        return $pin_id;
    }

    /**
     * Special handling to support duplication of sibling-PINs.
     *
     * @param  array $p_data
     * @param  int $templateObjId
     * @return array
     */
    private function _handle_duplication($p_data, $templateObjId)
    {
        $return = [
            'internal_pin_id' => null,
            'patch_pin_id'    => null,
        ];

        $pinKey = $p_data['number'] . '|' . $p_data['description'] . '|' . $p_data['classification'] . '|' . $p_data['type'];
        $res = $this->get_data(null, $templateObjId);
        $templatePinById = [];
        $templatePin = null;
        while ($row = $res->get_row()) {
            $key = $row['isys_catg_smview_cable_pin_list__number'] . '|'
                . $row['isys_catg_smview_cable_pin_list__description'] . '|'
                . $row['isys_catg_smview_cable_pin_list__classification'] . '|'
                . $row['isys_catg_smview_cable_pin_list__type'];
            if ($key == $pinKey) {
                $templatePin = $row;
            }
            $templatePinById[$row['isys_catg_smview_cable_pin_list__id']] = $key;
        }

        if (! $templatePin) {
            return $return;
        }

        $res = $this->get_data(null, $p_data['isys_obj__id']);
        $currentPins = [];
        while ($row = $res->get_row()) {
            $key = $row['isys_catg_smview_cable_pin_list__number'] . '|'
                . $row['isys_catg_smview_cable_pin_list__description'] . '|'
                . $row['isys_catg_smview_cable_pin_list__classification'] . '|'
                . $row['isys_catg_smview_cable_pin_list__type'];
            $currentPins[$key] = $row;
        }

        // check if internal- or patch-pin is already available
        $templateInternalPinKey = $templatePinById[$templatePin['isys_catg_smview_cable_pin_list__sibling_id']];
        if ($currentPins[$templateInternalPinKey]) {
            $row = $currentPins[$templateInternalPinKey];
            $return['internal_pin_id'] = $row['isys_catg_smview_cable_pin_list__id'];
        }

        // note: connector_id won't be touched because no one knows when this category will be duplicated

        return $return;
    }

    /**
     * Updates existing entity.
     *
     * @param  integer $p_bsm_list_id Entity's identifier
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws Exception
     */
    public function save_data($p_bsm_list_id, $p_data)
    {
        $is_updateable = $this->is_wire_pin_updateable($p_bsm_list_id);
        if (! $is_updateable) {
            $lang = isys_application::instance()->container->get('language');
            isys_notify::error($lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ERROR_PIN_IN_USE'));
            return false;
        }
        
        $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT;
        if ($p_data['type'] == isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT) {
            $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT;
        }
        $internal_pin_id = isset($p_data['internal_pin_id']) && $p_data['internal_pin_id'] != '-1' ? $p_data['internal_pin_id'] : null;
        $patch_pin_id = isset($p_data['patch_pin_id']) && $p_data['patch_pin_id'] != '-1' ? $p_data['patch_pin_id'] : null;
        $connector_id = isset($p_data['connector_id']) && $p_data['connector_id'] != '-1' ? $p_data['connector_id'] : null;
        $classification = $p_data['classification'] ?? null;
        $dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        $success = $dao->update_pin($p_bsm_list_id, $p_data['description'], $p_data['number'], $type, $internal_pin_id, $patch_pin_id, $connector_id, $classification);
        
        if (! empty($p_data['connection'])) {
            $wire_pin_id = $p_data['connection'];
            $this->_save_connection($wire_pin_id, $p_bsm_list_id);
        }
        
        return $success;
    }

    /**
     * Save PIN connection.
     *
     * @param  int $wire_pin_id
     * @param  int $pin_id
     * @throws isys_exception_database
     */
    private function _save_connection($wire_pin_id, $pin_id)
    {
        $dao = isys_cmdb_dao_smview_cable_wire_pin::instance($this->m_db);
        $connections = $dao->filter_objects(array('connected_pin_id' => $pin_id));
        foreach ($connections as $c) {
            // remove current connection of this PIN
            $dao->update_wire_pin($c['id'], $c['description'], $c['status_id']);
        }
        
        // add connection to wire pin
        $wire_pin = $dao->get_object('id', $wire_pin_id);
        $dao->update_wire_pin($wire_pin_id, $wire_pin['description'], $wire_pin['status_id'], null, $pin_id, true);
    }

    /**
     * Generate a specified amount of pins.
     *
     * @param  int         $obj_id
     * @param  int         $generator_id
     * @param  string|null $start_pattern
     * @param  string|null $end_pattern
     * @param  boolean     $generate_corresponding
     * @param  string|null $classification
     * @throws Exception
     */
    private function _generate_pins($obj_id, $generator_id, $start_pattern = null, $end_pattern = null, $generate_corresponding = false, $classification = null)
    {
        $dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        $name_generator = new LabelGenerator($generator_id);
        $names = $name_generator->generate_names_by_generator($start_pattern, $end_pattern);
        
        $number = 1;
        foreach ($names as $name) {
            $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_INPUT;
            $pin_in_id = $dao->create_pin($obj_id, $name, $number, $type, null, null, null, $classification);
            $number++;
            if ($generate_corresponding) {
                $type = isys_cmdb_dao_smview_cable_pin::PIN_TYPE_OUTPUT;
                $dao->create_pin($obj_id, $name, $number, $type, $pin_in_id, null, null, $classification);
                $number++;
            }
        }
    }

    /**
     * Update method.
     *
     * Handles check of objects, used in sections or routes.
     *
     * @param  string $p_query
     * @return boolean
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function update($p_query)
    {
        if (strpos($p_query, '__status') === false) {
            // no status will be changed
            return parent::update($p_query);
        }
        
        if (strpos($p_query, "__status = '2'") !== false) {
            // status can be changed to 2 ("Normal")
            return parent::update($p_query);
        }
        
        // check if object is in use
        $list_pattern = "/list__id='([\d]*)'/";
        $matches = [];
        $res = preg_match($list_pattern, $p_query, $matches);
        if ($res !== false) {
            // found list ID
            $list_id = (int) $matches[1];
            $is_updateable = $this->is_wire_pin_updateable($list_id);
            if ($is_updateable) {
                // item is updateable
                return parent::update($p_query);
            }
        }
        
        // abort update if object is in use
        $lang = isys_application::instance()->container->get('language');
        isys_notify::error($lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ERROR_PIN_IN_USE'));
        return true;
    }

    /**
     * Checks if wire pin is in use or not.
     *
     * @param  int $pin_id
     * @return boolean
     * @throws Exception
     */
    public function is_wire_pin_updateable($pin_id)
    {
        $dao = isys_cmdb_dao_smview_cable_pin::instance($this->m_db);
        $pin = $dao->get_object('id', $pin_id);
        if (empty($pin['connected_wire_pin_id'])) {
            return true;
        }
        
        $wire_pin_dao = isys_cmdb_dao_smview_cable_wire_pin::instance($this->m_db);
        $current_wire_pin = $wire_pin_dao->get_object('id', $pin['connected_wire_pin_id']);
        $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire($this->m_db);
        $language = isys_application::instance()->container->get('language');
        $in_use = $wire_catg->get_in_use($current_wire_pin['wire_id']);
        if (! empty($in_use) && $in_use == $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED')) {
            return false;
        }
        return true;
    }
}
