<?php

use idoit\Module\Cable\CableTracing;
use idoit\Module\Cable\SectionTracing;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable route elements.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_route_element extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_route_element';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        $hiddenProperties = [
            'id',
            'route_id',
            'cable_id',
            'wire_id',
            'obj_in_id',
            'pin_in_id',
            'obj_out_id',
            'pin_out_id',
        ];
        
        $properties = [
            'index'              => 'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_INDEX',
            'cable_title'        => 'LC__CMDB__CATG__SMVIEW_CABLE__TITLE',
            'wire_title'         => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE',
            'wire_scope'         => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_SCOPE',
            'wire_pin_in_title'  => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_LEFT_PIN',
            'wire_pin_out_title' => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_RIGHT_PIN',
            'obj_in_title'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_OBJECT',
            'pin_in_title'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_DESCRIPTION',
            'pin_in_scope'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_SCOPE',
            'obj_out_title'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_OBJECT',
            'pin_out_title'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_DESCRIPTION',
            'pin_out_scope'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_SCOPE',
            'length'             => 'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_ELEMENT_LENGTH',
        ];
        
        $ret = [];
        foreach ($hiddenProperties as $key) {
            $uiId = strtoupper($key);
            $ret[$key] = array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => $key,
                        C__PROPERTY__INFO__DESCRIPTION => $key
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_route_element__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => "C__CATG__SMVIEW_CABLE_$uiId"
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ],
                ]
            );
        }
        foreach ($properties as $key => $lang) {
            $uiId = strtoupper($key);
            $ret[$key] = array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => $lang,
                        C__PROPERTY__INFO__DESCRIPTION => $key
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_route_element__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => "C__CATG__SMVIEW_CABLE_$uiId"
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            );
        }
        return $ret;
    }

    /**
     * Get all route elements.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_all_elements()
    {
        $dao = new isys_cmdb_dao_smview_cable_route(isys_application::instance()->database);
        $routes = $dao->get_objects();

        $return = [];
        foreach ($routes as $route) {
            $return[$route["obj_id"]] = $this->get_elements_by_route_id($route["id"]);
        }
        return $return;
    }

    /**
     * Get route elements by isys_obj_id.
     *
     * @param  int     $p_obj_id
     * @param  boolean $ignore_start_object
     * @param  boolean $add_connection_information
     * @return array
     * @throws isys_exception_database
     */
    public function get_elements_by_obj_id($p_obj_id, $ignore_start_object = false, $add_connection_information = true)
    {
        $dao = new isys_cmdb_dao_smview_cable_route(isys_application::instance()->database);
        $route = $dao->get_object("obj_id", $p_obj_id);
        return $this->get_elements_by_route_id($route["id"], $ignore_start_object, $add_connection_information);
    }

    /**
     * Get route elements by route_id.
     *
     * @param  int     $route_id
     * @param  boolean $ignore_start_object
     * @param  boolean $add_connection_information
     * @param  boolean $consider_associated_route
     * @return array
     * @throws isys_exception_database
     */
    public function get_elements_by_route_id($route_id, $ignore_start_object = false, $add_connection_information = true, $consider_associated_route = false)
    {
        $dao = new isys_cmdb_dao_smview_cable_route(isys_application::instance()->database);
        $element_dao = new isys_cmdb_dao_smview_cable_route_element(isys_application::instance()->database);
        $route = $dao->get_object('id', $route_id);
        $routeElements = $element_dao->filter_objects(['route_id' => $route_id]);
        $associatedRouteElements = [];
        if ($consider_associated_route && ! empty($route['associated_route'])) {
            $associated_route = $dao->get_object('obj_id', $route['associated_route']);
            $associatedRouteElements = $element_dao->filter_objects(['route_id' => $associated_route['id']]);
        }
        
        // get all connections
        $elements = [];
        foreach ($routeElements as $element) {
            $associatedElement = array_shift($associatedRouteElements);
            if (! empty($element['section_id'])) {
                $section_id = $element['section_id'];
                $section = SectionTracing::get_section_endpoints($section_id);
                $element = array_merge($element, $section);
                if ($associatedElement) {
                    $section = SectionTracing::get_section_endpoints($associatedElement['section_id']);
                    $associatedElement = array_merge($associatedElement, $section);
                    $element['associated_element'] = $associatedElement;
                }
            } else {
                $element = isys_cmdb_dao_category_g_smview_cable_section_element::get_wire_connection($element);
                if ($associatedElement) {
                    $associatedElement = isys_cmdb_dao_category_g_smview_cable_section_element::get_wire_connection($associatedElement);
                    $element['associated_element'] = $associatedElement;
                }
            }
            $elements[$element['index']] = $element;
        }
        
        if (empty($elements)) {
            return $elements;
        }
        // sort elements by index
        ksort($elements);

        $start_obj_id = !empty($route['start_obj_id']) ? $route['start_obj_id'] : null;
        if ($ignore_start_object) {
            $start_obj_id = null;
        }
        $ordered_elements = isys_cmdb_dao_category_g_smview_cable_section_element::order_objects_within_connections($elements, $start_obj_id);

        // add connection information to each element
        $new_elements = [];
        foreach ($ordered_elements as $index => $element) {
            $new_elements[$index] = $add_connection_information ? self::add_connection_information($element) : $element;
        }

        // finally sort elements by index
        ksort($new_elements);
        return $new_elements;
    }

    /**
     * Re-order indices in case if the first one was deleted.
     *
     * @param int $route_id
     */
    public function re_order_indices($route_id)
    {
        $element_dao = new isys_cmdb_dao_smview_cable_route_element(isys_application::instance()->database);
        $elements = $this->get_elements_by_route_id($route_id, true);
        $currentIndex = 1;
        foreach ($elements as $index => $element) {
            if ($index != $currentIndex) {
                $element['index'] = $currentIndex;
                $element_dao->update_object($element['id'], $element);
            }
            $currentIndex++;
        }
    }

    /**
     * Add connection information to the objects of this element.
     *
     * @param  array $element
     * @return array
     * @throws isys_exception_database
     */
    public static function add_connection_information($element)
    {

        /*
         * todo: this is a performance killer!!!
         */

        // use incoming object, to get its connections to next objects
        $obj_id = $element['obj_in_id'];
        $wire_id = ! empty($element['wire_id']) ? $element['wire_id'] : null;
        $section_id = ! empty($element['section_id']) ? $element['section_id'] : null;

        $cable_tracing = new CableTracing($obj_id);
        $section_tracing = new SectionTracing($obj_id);

        // add cable capacity information
        if ($wire_id) {
            $cables = $cable_tracing->get_connected_cables();
            $cable = $cables[$element['cable_id']];
            $element['cable_free_wires'] = $cable['free_wires'];
            $element['cable_patched_wires'] = $cable['patched_wires'];
            $element['cable_max_capacity'] = $cable['max_capacity'];
        } else {
            $element['cable_free_wires'] = 0;
            $element['cable_patched_wires'] = 1;
            $element['cable_max_capacity'] = 100;
        }

        // get connections
        $connections = $cable_tracing->get_unpatched_connections(null, $wire_id);
        $sections = $section_tracing->get_unpatched_sections($section_id, $wire_id);
        $merged_elements = array_merge($connections['objects'], $sections['objects']);
        $connections = $merged_elements;

        $filtered_connections = [];
        foreach ($connections as $connection) {
            if ($connection['obj_in_id'] != $element['obj_out_id']) {
                // only add connections between these objects
                continue;
            }
            $key = $wire_id ? 'w' . $connection['wire_id'] : 's_' . $connection['section_id'];
            $filtered_connections[$key] = $connection;
        }

        // add configured connection at least
        if ($wire_id && ! isset($filtered_connections['w' . $wire_id])) {
            $filtered_connections['w' . $wire_id] = $cable_tracing->get_connection_by_wire_id($wire_id);
        }
        if ($section_id && ! isset($filtered_connections['s' . $section_id])) {
            $section = $section_tracing::get_section_endpoints($section_id);
            $filtered_connections['s' . $section_id] = $section;
        }
        $element['connections'] = array_values($filtered_connections);
        return $element;
    }
    
    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed         Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        // handled by route DAO
        return false;
    }
    
    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array   $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_list_id, $p_data)
    {
        // handled by route DAO
        return false;
    }

    /**
     * Patch route elements.
     *
     * @param int $route_id
     * @param array $current_elements
     */
    public function patch_route_elements($route_id, $current_elements = [])
    {
        // get ordered elements from database
        $elements = $this->get_elements_by_route_id($route_id, false, false);

        // get pins to be patched
        $current_pin_patches = $this->_get_patch_pins($current_elements);
        $pin_patches = $this->_get_patch_pins($elements);
        foreach (array_keys($pin_patches) as $key) {
            if (isset($current_pin_patches[$key])) {
                // remove this entry because patch is already in use
                unset($current_pin_patches[$key]);
            }
        }

        // first remove obsolete patches
        $pin_dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        foreach ($current_pin_patches as $pins) {
            $pin_dao->patch_pins($pins[0], $pins[1], true);
        }

        // then patch new pins
        foreach ($pin_patches as $pins) {
            $pin_dao->patch_pins($pins[0], $pins[1]);
        }
    }

    /**
     * Get PINs which have to be patched.
     *
     * @param array $elements
     * @return array
     */
    private function _get_patch_pins($elements)
    {
        $prev_element = null;
        $pin_patches = [];
        foreach ($elements as $element) {
            if ($prev_element) {
                $key = $prev_element['pin_in_id'] . '_' . $element['pin_out_id'];
                $pin_patches[$key] = [$prev_element['pin_in_id'], $element['pin_out_id']];
            }
            $prev_element = $element;
        }
        return $pin_patches;
    }
}
