<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable sections.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_section extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_section';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        return [
            'user_state_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__USER_STATE',
                        C__PROPERTY__INFO__DESCRIPTION => 'User State'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_list__user_state_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_user_state',
                        C__PROPERTY__DATA__REFERENCES => [
                            'isys_catg_smview_cable_user_state',
                            'isys_catg_smview_cable_user_state__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            "SELECT CONCAT_WS(': ', g.isys_catg_smview_cable_user_state__name, g.isys_catg_smview_cable_user_state__description)
                        FROM isys_catg_smview_cable_user_state g
                        INNER JOIN isys_catg_smview_cable_section_list l ON g.isys_catg_smview_cable_user_state__id = l.isys_catg_smview_cable_section_list__user_state_id",
                            'isys_catg_smview_cable_section_list',
                            'isys_catg_smview_cable_section_list__id',
                            'isys_catg_smview_cable_section_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_section_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_USER_STATE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                    'isys_cmdb_dao_category_g_smview_cable_section',
                                    'callback_property_user_states'
                                ]
                            )
                        ]
                    ]
                ]
            ),
            'channel_count' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::int(),
                [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__CHANNEL_COUNT',
                    C__PROPERTY__INFO__DESCRIPTION => 'Channel Count'
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_list__channel_count'
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CHANNEL_COUNT',
                    C__PROPERTY__UI__PARAMS => [
                        'p_strClass' => 'input-small'
                    ]
                ]
                    ]
            ),
            'current_channel_count' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::int(),
                [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__CURRENT_CHANNEL_COUNT',
                    C__PROPERTY__INFO__DESCRIPTION => 'Current Channel Count'
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_list__current_channel_count'
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_CURRENT_CHANNEL_COUNT',
                    C__PROPERTY__UI__PARAMS => [
                        'p_strClass' => 'input-small'
                    ]
                ]
                    ]
            ),
            'length' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::int(),
                [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__SECTION_LENGTH',
                    C__PROPERTY__INFO__DESCRIPTION => 'Length'
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_list__length'
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_SECTION_LENGTH',
                    C__PROPERTY__UI__PARAMS => [
                        'p_bReadonly' => 1,
                        'p_strClass'  => 'input-small'
                    ]
                ]
                    ]
            ),
            // only used to transfer data to UI
            'start_object_types' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__START_OBJECT_TYPES',
                        C__PROPERTY__INFO__DESCRIPTION => 'Start Object Types'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_START_OBJECT_TYPES',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_section',
                                'callback_property_start_object_types'
                                ]
                            ),
                            'p_strClass' => 'input-medium'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'section_elements' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__SECTION_ELEMENTS',
                        C__PROPERTY__INFO__DESCRIPTION => '-'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_ELEMENTS'
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'start_object' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::object_browser(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__START_OBJECT',
                        C__PROPERTY__INFO__DESCRIPTION => 'OBJECTS'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_START_OBJECT',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                    'isys_cmdb_dao_category_g_smview_cable_section',
                                    'callback_property_objects'
                                ]
                            ),
                            'multiselection' => false,
                            'catFilter'      => 'C__CATG__SMVIEW_CABLE_PIN',
                            'p_strClass'     => 'input-small',
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'end_object' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::object_browser(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__SMVIEW_CABLE__END_OBJECT',
                        C__PROPERTY__INFO__DESCRIPTION => 'OBJECTS'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_END_OBJECT',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                    'isys_cmdb_dao_category_g_smview_cable_section',
                                    'callback_property_objects'
                                ]
                            ),
                            'multiselection' => false,
                            'catFilter'      => 'C__CATG__SMVIEW_CABLE_PIN',
                            'p_strClass'     => 'input-small',
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'route_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => 'LC__SHD__ROUTE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Route'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => 'C__CATG__SMVIEW_CABLE_ROUTE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_bReadonly' => true,
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
        ];
    }

    /**
     * Callback method for start object types.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function callback_property_start_object_types()
    {
        $query = "SELECT 
                t.isys_obj_type__id,
                t.isys_obj_type__title
            FROM isys_catg_smview_cable_pin_list l
            JOIN isys_obj o ON l.isys_catg_smview_cable_pin_list__isys_obj__id = o.isys_obj__id
            JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
            GROUP BY t.isys_obj_type__id;";
        $res = $this->retrieve($query);
        
        $objects = array();
        $lang = isys_application::instance()->container->get('language');
        while ($row = $res->get_row()) {
            $type_title = $lang->get($row['isys_obj_type__title']);
            $objects[$row['isys_obj_type__id']] = $type_title;
        }
        return $objects;
    }

    /**
     * Callback method for start object types.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function callback_property_objects()
    {
        $query = "SELECT 
                o.isys_obj__id,
                o.isys_obj__title
            FROM isys_catg_smview_cable_pin_list l
            JOIN isys_obj o ON l.isys_catg_smview_cable_pin_list__isys_obj__id = o.isys_obj__id
            GROUP BY o.isys_obj__id;";
        $res = $this->retrieve($query);

        $objects = array();
        while ($row = $res->get_row()) {
            $objects[$row['isys_obj__id']] = $row['isys_obj__title'];
        }
        return $objects;
    }

    /**
     * Callback method for groups.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function callback_property_user_states()
    {
        $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
        $res = $dao->get_objects();
        $groups = [];
        foreach ($res as $group) {
            $description = $group['name'] . ': ' . $group['description'];
            $groups[$group['id']] = $description;
        }
        return $groups;
    }

    /**
     * Find usages of the given object.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function find_usages($obj_id)
    {
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $route_dao = new isys_cmdb_dao_smview_cable_route($this->m_db);

        $sql = "SELECT * FROM isys_catg_smview_cable_route_element e
                JOIN isys_catg_smview_cable_section_list l
                ON e.isys_catg_smview_cable_route_element__section_id=l.isys_catg_smview_cable_section_list__id
                WHERE l.isys_catg_smview_cable_section_list__isys_obj__id=$obj_id";
        $res = $this->retrieve($sql);

        $elements = [];
        while ($row = $res->get_row()) {
            $route = $route_dao->get_object('id', $row['isys_catg_smview_cable_route_element__route_id']);
            $obj = $global_dao->get_object($route['obj_id'])->get_row();
            $elements[] = array(
                'id'         => $route['id'],
                'obj_id'     => $route['obj_id'],
                'title'      => $obj['isys_obj__title'],
                'type_id'    => $obj['isys_obj_type__id'],
                'type_title' => $obj['isys_obj_type__title'],
            );
        }
        return $elements;
    }

    /**
     * Calculate damping of a section or route.
     *
     * @param  array $elements
     * @param  float $damping_1310_factor
     * @param  float $damping_1550_factor
     * @return array
     * @throws isys_exception_database
     */
    public function calculate_damping($elements, $damping_1310_factor, $damping_1550_factor)
    {
        $damping = [
            'length'             => 0,
            'length_1310'        => 0,
            'length_1550'        => 0,
            'sum_1310'           => 0,
            'sum_1550'           => 0,
            'connectors'         => [],
            'not_applied_wires'  => false,
            'empty_wire_damping' => false,
        ];

        // prepare states
        $wire_pin_status_dao = new isys_cmdb_dao_smview_cable_wire_pin_status(isys_application::instance()->database);
        $raw_states = $wire_pin_status_dao->get_objects();
        $states = [];
        foreach ($raw_states as $s) {
            $states[$s['id']] = $s;
            $damping['connectors'][$s['const']] = [
                'count'       => 0,
                'factor'      => $s['upper_transition_damping'],
                'description' => $s['description'],
            ];
        }
        $damping = $this->_calculate_elements_damping($damping, $elements, $states, $damping_1310_factor, $damping_1550_factor);

        // remove unused connectors
        $connectors = [];
        foreach ($damping['connectors'] as $const => $c) {
            if ($c['count'] > 0) {
                $connectors[$const] = $c;
            }
        }
        $damping['connectors'] = $connectors;
        return $damping;
    }

    /**
     * Calculate damping of elements.
     *
     * @param  array $damping
     * @param  array $elements
     * @param  array $states
     * @param  float $damping_1310_factor
     * @param  float $damping_1550_factor
     * @return array
     */
    private function _calculate_elements_damping($damping, $elements, $states, $damping_1310_factor, $damping_1550_factor)
    {
        $has_not_applied_wires = false;
        $empty_wire_damping = false;
        foreach ($elements as $e) {
            if ($e['route_id'] && $e['section_id']) {
                // resolve a section within a route
                $dao = new isys_cmdb_dao_category_g_smview_cable_section_element(isys_application::instance()->database);
                $section_elements = $dao->get_elements_by_section_id($e['section_id'], false, false, false);
                $damping = $this->_calculate_elements_damping($damping, $section_elements, $states, $damping_1310_factor, $damping_1550_factor);
                continue;
            }

            // length
            $l = (int) $e['length'];
            $damping['length'] += $l;
            $l = $l / 1000; // from m to km

            $damping_1310 = $damping_1310_factor * $l;
            $damping_1550 = $damping_1550_factor * $l;
            $damping['length_1310'] += $damping_1310;
            $damping['length_1550'] += $damping_1550;
            $damping['sum_1310'] += $damping_1310;
            $damping['sum_1550'] += $damping_1550;

            // count connectors by type
            $status_in = $states[$e['wire_pin_in_state']];
            $status_out = $states[$e['wire_pin_out_state']];
            if ($status_in['wire_not_applied'] == 1 || $status_out['wire_not_applied'] == 1) {
                $has_not_applied_wires = true;
            }
            if (empty($status_in['upper_transition_damping']) || empty($status_out['upper_transition_damping'])) {
                $empty_wire_damping = true;
            }
            $damping = $this->_sum_wire_pin_status($damping, $status_in);
            $damping = $this->_sum_wire_pin_status($damping, $status_out);
        }

        if ($has_not_applied_wires) {
            $damping['not_applied_wires'] = true;
        }
        if ($empty_wire_damping) {
            // reset calculation if any wire is empty
            $damping['length_1310'] = 0;
            $damping['length_1550'] = 0;
            $damping['sum_1310'] = 0;
            $damping['sum_1550'] = 0;
            $damping['empty_wire_damping'] = true;
        }
        return $damping;
    }

    /**
     * Summarize a wire pin status.
     *
     * @param  array $damping
     * @param  array $status
     * @return array
     */
    private function _sum_wire_pin_status($damping, $status)
    {
        if (empty($status['const'])) {
            return $damping;
        }

        $factor = (float) $status['upper_transition_damping'];
        $damping['connectors'][$status['const']]['count']++;

        $damping['sum_1310'] += $factor;
        $damping['sum_1550'] += $factor;
        return $damping;
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return int  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function create_data($p_data)
    {
        $user_state_id = isset($p_data['user_state_id']) && $p_data['user_state_id'] != '-1' ? $p_data['user_state_id'] : null;
        $channel_count = isset($p_data['channel_count']) ? $p_data['channel_count'] : null;
        $current_channel_count = isset($p_data['current_channel_count']) ? $p_data['current_channel_count'] : null;
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $id = $dao->create_section($p_data['isys_obj__id'], '', $user_state_id, $channel_count, $current_channel_count);
        
        $this->_handle_section_elements($id, $p_data);
        return $id;
    }

    /**
     * Updates existing entity.
     *
     * @param  int   $p_list_id Entity's identifier
     * @param  array $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function save_data($p_list_id, $p_data)
    {
        $user_state_id = isset($p_data['user_state_id']) && $p_data['user_state_id'] != '-1' ? $p_data['user_state_id'] : null;
        $channel_count = isset($p_data['channel_count']) ? $p_data['channel_count'] : null;
        $current_channel_count = isset($p_data['current_channel_count']) ? $p_data['current_channel_count'] : null;
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $success = $dao->update_section($p_list_id, '', $user_state_id, $channel_count, $current_channel_count);
        
        $this->_handle_section_elements($p_list_id, $p_data);
        return $success;
    }

    /**
     * Handle section elements.
     *
     * @param  int   $id
     * @param  array $p_data
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _handle_section_elements($id, $p_data)
    {
        $section_element_dao = new isys_cmdb_dao_category_g_smview_cable_section_element($this->m_db);
        if (! isset($p_data['section_elements'])) {
            return;
        }

        $elements = $p_data['section_elements'];

        // first get current elements (to determine patches to be deleted later on)
        $current_elements = $section_element_dao->get_elements_by_section_id($id, null, false, false);

        // save new elements
        $this->_save_section_elements($id, $elements);

        // create new patches and remove obsolete ones
        $section_element_dao->patch_section_elements($id, $current_elements);
    }

    /**
     * Save section elements.
     *
     * @param  int    $section_id
     * @param  string $json_elements
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _save_section_elements($section_id, $json_elements)
    {
        $section_element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);

        // remove old section elements
        $current_elements = $section_element_dao->filter_objects(array('section_id' => $section_id));
        foreach ($current_elements as $current_element) {
            $section_element_dao->delete_object($current_element['id']);
        }

        if (empty($json_elements)) {
            return;
        }
        
        $elements = json_decode($json_elements);
        if (empty($elements)) {
            // set section length to 0
            $section_dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
            $section_dao->update_object($section_id, [], array('length' => 0));
        }

        $start_obj_id = null;
        $end_obj_id = null;
        foreach ($elements as $element) {
            if (empty($element->wire_id)) {
                continue;
            }
            
            if (! $start_obj_id) {
                $start_obj_id = $element->obj_out_id;
            }
            $end_obj_id = $element->obj_in_id;
            
            $params = array(
                'section_id'  => $section_id,
                'wire_id'     => $element->wire_id,
                'index'       => $element->index,
                'description' => $element->description,
            );
            $section_element_dao->create_object($params);
        }
        
        $this->_update_section_start_and_end_objects($section_id, $start_obj_id, $end_obj_id);
    }

    /**
     * Update sections start- and end-object.
     *
     * @param  int $section_id
     * @param  int $start_obj_id
     * @param  int $end_obj_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _update_section_start_and_end_objects($section_id, $start_obj_id, $end_obj_id)
    {
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $dao->get_object('id', $section_id);
        $params = array(
            'start_obj_id' => $start_obj_id,
            'end_obj_id'   => $end_obj_id,
        );
        $dao->update_object($section_id, $params);
    }

    /**
     * Check on which side the section is connected to previous element and fix it if needed.
     *
     * @param  array $prevElement
     * @param  array $sectionElements
     * @return array
     */
    public static function checkAndFixConnectedSectionSide($prevElement, $sectionElements)
    {
        if ($prevElement) {
            $sectionElementValues = array_values($sectionElements);
            $firstElement = array_shift($sectionElementValues);
            $lastElement = count($sectionElementValues) > 0 ? array_pop($sectionElementValues) : $firstElement;
            if ($prevElement['obj_out_id'] == $firstElement['obj_out_id'] || $prevElement['obj_in_id'] == $firstElement['obj_in_id']) {
                // flip IN/OUT
                $flippedElements = [];
                foreach ($sectionElements as $index => $element) {
                    $flippedElements[$index] = isys_cmdb_dao_category_g_smview_cable_section_element::flip_connection($element);
                }
                $sectionElements = $flippedElements;
            }
            if ($prevElement['obj_out_id'] == $lastElement['obj_in_id'] || $prevElement['obj_in_id'] == $lastElement['obj_out_id']) {
                // reverse order
                $sectionElements = array_reverse($sectionElements);
            }
            if ($prevElement['obj_out_id'] == $lastElement['obj_out_id'] || $prevElement['obj_in_id'] == $lastElement['obj_in_id']) {
                // reverse order and flip IN/OUT
                $sectionElements = array_reverse($sectionElements);
                $flippedElements = [];
                foreach ($sectionElements as $index => $element) {
                    $flippedElements[$index] = isys_cmdb_dao_category_g_smview_cable_section_element::flip_connection($element);
                }
                $sectionElements = $flippedElements;
            }
        }
        return $sectionElements;
    }

    /**
     * Method to remove all section information if an object is purged.
     * This method will be triggered via signal handler in init.php.
     *
     * @param  int           $objectId
     * @param  isys_cmdb_dao $cmdbDao
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function onObjectPurge($objectId, isys_cmdb_dao $cmdbDao)
    {
        if (empty($objectId)) {
            return;
        }

        $sectionDao = isys_cmdb_dao_category_g_smview_cable_section::instance(isys_application::instance()->database);
        $sectionElementDao = isys_cmdb_dao_category_g_smview_cable_section_element::instance(isys_application::instance()->database);

        $section = $sectionDao->get_data(null, $objectId)->get_row();
        $sectionId = $section['isys_catg_smview_cable_section_list__id'];

        if (empty($sectionId)) {
            return;
        }

        // first get current elements (to determine patches to be deleted later on)
        $currentElements = $sectionElementDao->get_elements_by_section_id($sectionId, null, false, false);

        // remove all section elements
        $elementDao = isys_cmdb_dao_smview_cable_section_element::instance(isys_application::instance()->database);
        $sectionElements = $elementDao->filter_objects(array('section_id' => $sectionId));
        foreach ($sectionElements as $current_element) {
            $elementDao->delete_object($current_element['id']);
        }

        // remove obsolete patches
        $sectionElementDao->patch_section_elements($sectionId, $currentElements);

        // delete section
        $sectionCustomDao = new isys_cmdb_dao_smview_cable_section(isys_application::instance()->database);
        $sectionCustomDao->delete_object($sectionId);
    }
}
