<?php

use idoit\Module\Cable\CableTracing;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable sections.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_section_element extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_section_element';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        $hiddenProperties = [
            'id',
            'section_id',
            'cable_id',
            'wire_id',
            'obj_in_id',
            'pin_in_id',
            'obj_out_id',
            'pin_out_id',
        ];
        
        $properties = [
            'index'              => 'LC__CMDB__CATG__SMVIEW_CABLE__SECTION_INDEX',
            'cable_title'        => 'LC__CMDB__CATG__SMVIEW_CABLE__TITLE',
            'wire_title'         => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE',
            'wire_scope'         => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_SCOPE',
            'wire_pin_in_title'  => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_LEFT_PIN',
            'wire_pin_out_title' => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_RIGHT_PIN',
            'obj_in_title'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_OBJECT',
            'pin_in_title'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_DESCRIPTION',
            'pin_in_scope'       => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_IN_SCOPE',
            'obj_out_title'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_OBJECT',
            'pin_out_title'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_DESCRIPTION',
            'pin_out_scope'      => 'LC__CMDB__CATG__SMVIEW_CABLE__PIN_OUT_SCOPE',
            'length'             => 'LC__CMDB__CATG__SMVIEW_CABLE__SECTION_ELEMENT_LENGTH',
        ];
        
        $ret = [];
        foreach ($hiddenProperties as $key) {
            $uiId = strtoupper($key);
            $ret[$key] = array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE => $key,
                        C__PROPERTY__INFO__DESCRIPTION => $key
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_element__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID => "C__CATG__SMVIEW_CABLE_$uiId"
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ],
                ]
            );
        }
        foreach ($properties as $key => $lang) {
            $uiId = strtoupper($key);
            $ret[$key] = array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => $lang,
                    C__PROPERTY__INFO__DESCRIPTION => $key
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_section_element__fake_field_name'
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => "C__CATG__SMVIEW_CABLE_$uiId"
                ],
                C__PROPERTY__PROVIDES => [
                    C__PROPERTY__PROVIDES__SEARCH     => false,
                    C__PROPERTY__PROVIDES__LIST       => false,
                    C__PROPERTY__PROVIDES__REPORT     => false,
                    C__PROPERTY__PROVIDES__VALIDATION => false,
                    C__PROPERTY__PROVIDES__VIRTUAL    => true,
                ]
                ]
            );
        }
        return $ret;
    }

    /**
     * Get section elements by isys_obj_id.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_all_elements()
    {
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $sections = $dao->get_objects();

        $return = [];
        foreach ($sections as $section) {
            $return[$section["obj_id"]] = $this->get_elements_by_section_id($section["id"]);
        }
        return $return;
    }

    /**
     * Get section elements by isys_obj_id.
     *
     * @param  int     $p_obj_id
     * @param  boolean $ignore_start_object
     * @param  boolean $add_connection_information
     * @param  boolean $add_capacity_information
     * @return array
     * @throws isys_exception_database
     */
    public function get_elements_by_obj_id($p_obj_id, $ignore_start_object = false, $add_connection_information = true, $add_capacity_information = true)
    {
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $section = $dao->get_object("obj_id", $p_obj_id);
        return $this->get_elements_by_section_id($section["id"], $ignore_start_object, $add_connection_information, $add_capacity_information);
    }

    /**
     * Get section elements by section_id.
     *
     * @param  int     $section_id
     * @param  boolean $ignore_start_object
     * @param  boolean $add_connection_information
     * @param  boolean $add_capacity_information
     * @return array
     * @throws isys_exception_database
     */
    public function get_elements_by_section_id($section_id, $ignore_start_object = false, $add_connection_information = true, $add_capacity_information = true)
    {
        $dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        $filter = array('section_id' => $section_id);
        $res = $element_dao->filter_objects($filter);
        
        // get all connections
        $elements = [];
        foreach ($res as $element) {
            $element = self::get_wire_connection($element);
            $elements[$element['index']] = $element;
        }
        
        if (empty($elements)) {
            return $elements;
        }
        // sort elements by index
        ksort($elements);

        // order elements
        $section = $dao->get_object('id', $section_id);
        $start_obj_id = !empty($section['start_obj_id']) ? $section['start_obj_id'] : null;
        if ($ignore_start_object) {
            $start_obj_id = null;
        }
        $ordered_elements = self::order_objects_within_connections($elements, $start_obj_id);

        // add connection information to each element
        $new_elements = [];
        foreach ($ordered_elements as $index => $element) {
            $new_elements[$index] = self::add_connection_information($element, $add_connection_information, $add_capacity_information);
        }

        // finally sort elements by index
        ksort($new_elements);
        return $new_elements;
    }

    /**
     * Add connection information to the objects of this element.
     *
     * @param  array   $element
     * @param  boolean $add_connection_information
     * @param  boolean $add_capacity_information
     * @return array
     * @throws isys_exception_database
     */
    public static function add_connection_information($element, $add_connection_information = true, $add_capacity_information = true)
    {

        /*
         * todo: this is a performance killer!!!
         */
        if (! $add_capacity_information && ! $add_connection_information) {
            return $element;
        }

        // use incoming object, to get its connections to next objects
        $obj_id = $element['obj_in_id'];
        $wire_id = $element['wire_id'];
        $tracing = new CableTracing($obj_id);

        // add cable capacity information
        if ($add_capacity_information) {
            $cables = $tracing->get_connected_cables();
            $cable = $cables[$element['cable_id']];
            $element['cable_free_wires'] = $cable['free_wires'];
            $element['cable_patched_wires'] = $cable['patched_wires'];
            $element['cable_max_capacity'] = $cable['max_capacity'];
        }

        if ($add_connection_information) {
            // get connections
            $unpatched_connections = $tracing->get_unpatched_connections(null, $wire_id);
            $connections = $unpatched_connections['objects'];
            $filtered_connections = [];
            foreach ($connections as $k => $connection) {
                if ($connection['obj_in_id'] != $element['obj_out_id']) {
                    // only add connections between these objects
                    continue;
                }
                $key = $connection['wire_id'];
                $filtered_connections[$key] = $connection;
            }

            // add configured connection at least
            if (! isset($filtered_connections[$wire_id])) {
                $filtered_connections[$wire_id] = $tracing->get_connection_by_wire_id($wire_id);
            }
            $element['connections'] = array_values($filtered_connections);
        }
        return $element;
    }
    
    /**
     * Order objects within connections to be object in ID the same as
     * object out ID from previous connection.
     *
     * @param  array $elements
     * @param  int   $start_obj_id
     * @return array
     */
    public static function order_objects_within_connections($elements, $start_obj_id = null)
    {
        // get first indices
        $indices = array_keys($elements);
        $first_index = $indices[0];
        $second_index = $indices[1];
        
        $ordered_elements = [];
        $element1 = $elements[$first_index];
        if (empty($start_obj_id) && $second_index) {
            // pre-order first connection pair
            $element2 = $elements[$second_index];
            if ($element1['obj_in_id'] == $element2['obj_in_id']) {
                $element2 = self::flip_connection($element2);
            }
            if ($element1['obj_out_id'] == $element2['obj_out_id']) {
                $element1 = self::flip_connection($element1);
            }
            if ($element1['obj_out_id'] == $element2['obj_in_id']) {
                $element1 = self::flip_connection($element1);
                $element2 = self::flip_connection($element2);
            }

            if ($element1['obj_out_id'] == $element2['obj_in_id'] && isset($indices[2])) {
                // start- and end-objects are equal -> use third element to determine a valid order
                $third_index = $indices[2];
                $element3 = $elements[$third_index];
                if ($element2['obj_in_id'] != $element3['obj_out_id']) {
                    // invalid order between element 2 and element 3
                    if ($element2['obj_in_id'] == $element3['obj_in_id']) {
                        // flip both connections before
                        $element1 = self::flip_connection($element1);
                        $element2 = self::flip_connection($element2);
                    } elseif ($element2['obj_out_id'] == $element3['obj_in_id']) {
                        // flip both connections before
                        $element1 = self::flip_connection($element1);
                        $element2 = self::flip_connection($element2);
                        $element3 = self::flip_connection($element3);
                    }
                }
            }
            
            // add first elements to ordered array
            $lastObj = $element2['obj_in_id'];
            $ordered_elements[$first_index] = $element1;
            $ordered_elements[$second_index] = $element2;
            if (isset($third_index) && isset($element3)) {
                $ordered_elements[$third_index] = $element3;
                $lastObj = $element3['obj_in_id'];
            }
        } else {
            if ($element1['obj_in_id'] == $start_obj_id) {
                $element1 = self::flip_connection($element1);
            }
            $lastObj = $element1['obj_in_id'];
            $ordered_elements[$first_index] = $element1;
        }
        
        // order remaining connections based on last object ID
        foreach ($elements as $index => $element) {
            if (isset($ordered_elements[$index])) {
                continue;
            }
            
            if ($element['obj_in_id'] == $lastObj) {
                $element = self::flip_connection($element);
            }
            $lastObj = $element['obj_in_id'];
            $ordered_elements[$index] = $element;
        }
        
        return $ordered_elements;
    }
    
    /**
     * Flip incoming and outgoing properties.
     *
     * @param  array $element
     * @return array
     */
    public static function flip_connection($element)
    {
        $newElement = $element;
        foreach ($element as $key => $value) {
            if (strpos($key, '_in_')) {
                $newKey = str_replace('_in_', '_out_', $key);
                $newElement[$newKey] = $value;
            }
            if (strpos($key, '_out_')) {
                $newKey = str_replace('_out_', '_in_', $key);
                $newElement[$newKey] = $value;
            }
        }
        return $newElement;
    }

    /**
     * Re-order indices in case if the first one was deleted.
     *
     * @param  int $section_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function re_order_indices($section_id)
    {
        $element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        // todo: ignore connection information?
        $elements = $this->get_elements_by_section_id($section_id, true);
        $currentIndex = 1;
        foreach ($elements as $index => $element) {
            if ($index != $currentIndex) {
                $element['index'] = $currentIndex;
                $element_dao->update_object($element['id'], $element);
            }
            $currentIndex++;
        }
    }
    
    /**
     * Get wire connection.
     *
     * @param  array $element
     * @return array
     */
    public static function get_wire_connection($element)
    {
        $dao_view = new isys_cmdb_dao_smview_cable_connection_view(isys_application::instance()->database);
        $c = $dao_view->get_object('wire_id', $element['wire_id']);

        return array_merge(
            $element,
            array(
            'obj_in_id'          => $c['obj_id_in'],
            'obj_in_title'       => $c['obj_title_in'],
            'pin_in_id'          => $c['pin_id_in'],
            'pin_in_title'       => $c['pin_title_in'],
            'pin_in_scope'       => $c['pin_scope_in'],
            'obj_out_id'         => $c['obj_id_out'],
            'obj_out_title'      => $c['obj_title_out'],
            'pin_out_id'         => $c['pin_id_out'],
            'pin_out_title'      => $c['pin_title_out'],
            'pin_out_scope'      => $c['pin_scope_out'],
            'wire_title'         => $c['wire_title'],
            'wire_scope'         => $c['wire_scope'],
            'wire_pin_in_title'  => $c['wire_pin_title_in'],
            'wire_pin_out_title' => $c['wire_pin_title_out'],
            'wire_pin_in_state'  => $c['wire_pin_state_in'],
            'wire_pin_out_state' => $c['wire_pin_state_out'],
            'cable_id'           => $c['cable_id'],
            'cable_title'        => $c['cable_title'],
            'technology_id'      => $c['technology_id'],
            'length'             => $c['length'],
            )
        );
    }
    
    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        // handled by section DAO
        return false;
    }
    
    /**
     * Updates existing entity.
     *
     * @param  integer $p_list_id Entity's identifier
     * @param  array   $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_list_id, $p_data)
    {
        // handled by section DAO
        return false;
    }

    /**
     * Patch section elements.
     *
     * @param int $section_id
     * @param array $current_elements
     */
    public function patch_section_elements($section_id, $current_elements = [])
    {
        // get ordered elements from database
        $elements = $this->get_elements_by_section_id($section_id, null, false, false);
        
        // get pins to be patched
        $current_pin_patches = $this->_get_patch_pins($current_elements);
        $pin_patches = $this->_get_patch_pins($elements);
        foreach (array_keys($pin_patches) as $key) {
            if (isset($current_pin_patches[$key])) {
                // remove this entry because patch is already in use
                unset($current_pin_patches[$key]);
            }
        }
        
        // first remove obsolete patches
        $pin_dao = new isys_cmdb_dao_smview_cable_pin($this->m_db);
        foreach ($current_pin_patches as $pins) {
            $pin_dao->patch_pins($pins[0], $pins[1], true);
        }

        // then patch new pins
        foreach ($pin_patches as $pins) {
            $pin_dao->patch_pins($pins[0], $pins[1]);
        }
    }

    /**
     * Get PINs which have to be patched.
     *
     * @param array $elements
     * @return array
     */
    private function _get_patch_pins($elements)
    {
        $prev_element = null;
        $pin_patches = [];
        foreach ($elements as $element) {
            if ($prev_element) {
                $key = $prev_element['pin_in_id'] . '_' . $element['pin_out_id'];
                $pin_patches[$key] = [$prev_element['pin_in_id'], $element['pin_out_id']];
            }
            $prev_element = $element;
        }
        return $pin_patches;
    }
}
