<?php

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable virtual routes.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_virtual_route extends isys_cmdb_dao_category_g_smview_cable_route
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_virtual_route';

    /**
     * Table name.
     *
     * @var string
     */
    protected $m_table = 'isys_catg_smview_cable_vir_route_list';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = false;

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        $props = parent::properties();

        // replace property data fields
        foreach ($props as $key => $prop) {
            if ($prop[C__PROPERTY__DATA] && $prop[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD]) {
                $field = $prop[C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD];
                $field = str_replace('_route_list_', '_vir_route_list_', $field);
                $props[$key][C__PROPERTY__DATA][C__PROPERTY__DATA__FIELD] = $field;
            }
        }

        // overwrite special fields
        $props['route_usage_id'][C__PROPERTY__UI][C__PROPERTY__UI__PARAMS] = [
            'p_arData' => new isys_callback(
                [
                    'isys_cmdb_dao_category_g_smview_cable_virtual_route',
                    'callback_property_route_usages'
                ]
            )
        ];
        return $props;
    }

    /**
     * Callback method for route usages.
     *
     * @return array
     */
    public function callback_property_route_usages()
    {
        // virtual routes are not used anywhere
        return [];
    }

    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function create_data($p_data)
    {
        $route_usage_id = isset($p_data['route_usage_id']) && $p_data['route_usage_id'] != '-1' ? $p_data['route_usage_id'] : null;
        $is_external = isset($p_data['is_external']) ? $p_data['is_external'] == '1' : null;

        $params = array(
            'obj_id'      => $p_data['isys_obj__id'],
            'description' => $p_data['description'],
        );
        $optionalParams = array(
            'connection_count'         => $p_data['connection_count'] ?? null,
            'current_connection_count' => $p_data['current_connection_count'] ?? null,
            'permission'               => $p_data['permission'] ?? null,
            'location'                 => $p_data['location'] ?? null,
            'room'                     => $p_data['room'] ?? null,
            'device_type'              => $p_data['device_type'] ?? null,
            'device_number'            => $p_data['device_number'] ?? null,
            'phone_number'             => $p_data['phone_number'] ?? null,
            'route_usage_id'           => $route_usage_id,
            'phone_subscriber'         => $p_data['phone_subscriber'] ?? null,
            'hint'                     => $p_data['hint'] ?? null,
            'is_external'              => $is_external,
            'associated_route'         => null,
            'measured_length'          => $p_data['measured_length'] ?? null,
            'measure_date'             => $p_data['measure_date'] ?? null,
            'start_obj_id'             => null,
            'end_obj_id'               => null,
            'tka_participant'          => null,
            'damping_left1'            => $p_data['damping_left1'] ?? null,
            'damping_left2'            => $p_data['damping_left2'] ?? null,
            'damping_right1'           => $p_data['damping_right1'] ?? null,
            'damping_right2'           => $p_data['damping_right2'] ?? null,
        );

        $dao = new \isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
        $id = $dao->create_object($params, $optionalParams);

        if (isset($p_data['route_elements'])) {
            // handle route elements only if posted
            $jsonElements = $p_data['route_elements'];
            $this->_persist_route_elements($id, $jsonElements);
        }
        return $id;
    }

    /**
     * Updates existing entity.
     *
     * @param  int   $p_list_id Entity's identifier
     * @param  array $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function save_data($p_list_id, $p_data)
    {
        $route_usage_id = isset($p_data['route_usage_id']) && $p_data['route_usage_id'] != '-1' ? $p_data['route_usage_id'] : null;
        $is_external = isset($p_data['is_external']) ? $p_data['is_external'] == '1' : null;
        $tka_participant = $this->m_data['isys_catg_smview_cable_route_list__tka_participant'] ?: null;

        $params = array(
            'description' => $p_data['description'],
        );
        $optionalParams = array(
            'connection_count'         => $p_data['connection_count'] ?? null,
            'current_connection_count' => $p_data['current_connection_count'] ?? null,
            'permission'               => $p_data['permission'] ?? null,
            'location'                 => $p_data['location'] ?? null,
            'room'                     => $p_data['room'] ?? null,
            'device_type'              => $p_data['device_type'] ?? null,
            'device_number'            => $p_data['device_number'] ?? null,
            'phone_number'             => $p_data['phone_number'] ?? null,
            'phone_subscriber'         => $p_data['phone_subscriber'] ?? null,
            'hint'                     => $p_data['hint'] ?? null,
            'measured_length'          => $p_data['measured_length'] ?? null,
            'measure_date'             => $p_data['measure_date'] ?? null,
            'start_obj_id'             => null,
            'end_obj_id'               => null,
            'tka_participant'          => $tka_participant,
            'route_usage_id'           => $route_usage_id,
            'is_external'              => $is_external,
            'associated_route'         => null,
            'damping_left1'            => $p_data['damping_left1'] ?? null,
            'damping_left2'            => $p_data['damping_left2'] ?? null,
            'damping_right1'           => $p_data['damping_right1'] ?? null,
            'damping_right2'           => $p_data['damping_right2'] ?? null,
        );

        $dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
        $success = $dao->update_object($p_list_id, $params, $optionalParams);

        if (isset($p_data['route_elements'])) {
            // handle route elements only if posted
            $jsonElements = $p_data['route_elements'];
            $this->_persist_route_elements($p_list_id, $jsonElements);
        }
        return $success;
    }

    /**
     * Persist route elements.
     *
     * @param  int $id
     * @param string $elements
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _persist_route_elements($id, $elements)
    {
        // first get current elements (to determine patches to be deleted later on)
        $route_element_dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element($this->m_db);
        $current_elements = $route_element_dao->get_elements_by_route_id($id, false, false);

        // save new elements
        $this->_save_route_elements($id, $elements);

        // create new patches and remove obsolete ones
        $route_element_dao->patch_route_elements($id, $current_elements);
    }

    /**
     * Save route elements.
     *
     * @param  int    $route_id
     * @param  string $json_elements
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _save_route_elements($route_id, $json_elements)
    {
        $route_element_dao = new isys_cmdb_dao_smview_cable_virtual_route_element($this->m_db);

        // remove old route elements
        $current_elements = $route_element_dao->filter_objects(array('route_id' => $route_id));
        foreach ($current_elements as $current_element) {
            $route_element_dao->delete_object($current_element['id']);
        }

        if (empty($json_elements)) {
            return;
        }

        $elements = json_decode($json_elements);
        $start_obj_id = null;
        $end_obj_id = null;
        foreach ($elements as $element) {
            if (empty($element->wire_id) && empty($element->section_id)) {
                continue;
            }

            if (! $start_obj_id) {
                $start_obj_id = $element->obj_out_id;
            }
            $end_obj_id = $element->obj_in_id;

            $params = array(
                'route_id'    => $route_id,
                'index'       => $element->index,
            );
            $optional_params = array(
                'wire_id'    => $element->wire_id,
                'section_id' => $element->section_id,
            );
            $route_element_dao->create_object($params, $optional_params);
        }

        $this->_update_route_start_and_end_objects($route_id, $start_obj_id, $end_obj_id);
    }

    /**
     * Update routes start- and end-object.
     *
     * @param  int $route_id
     * @param  int $start_obj_id
     * @param  int $end_obj_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _update_route_start_and_end_objects($route_id, $start_obj_id, $end_obj_id)
    {
        $dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
        $dao->get_object('id', $route_id);
        $params = array(
            'start_obj_id' => $start_obj_id,
            'end_obj_id'   => $end_obj_id,
        );
        $dao->update_object($route_id, $params);
    }

    /**
     * Updated associated route.
     *
     * @param  int $obj_id
     * @param  int $associated_obj_id
     */
    public function update_associated_route($obj_id, $associated_obj_id)
    {
        // associated routes are not supported on virtual routes
    }

    /**
     * Update redundant routes.
     *
     * @param  int $obj_id
     * @param  array $redundant_routes
     */
    public function update_redundant_routes($obj_id, $redundant_routes)
    {
        // redundant routes are not supported on virtual routes
    }
}
