<?php

use idoit\Component\Property\Type\TextProperty;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Cable Wires.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_cable_wire extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_cable_wire';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * Cached user states.
     *
     * @var array|null
     */
    private $_cache_user_states = [];

    /**
     * Cached connection types.
     *
     * @var array|null
     */
    private $_cache_connection_types = [];

    /**
     * Cached wire states.
     *
     * @var array|null
     */
    private $_cache_wire_states = [];

    /**
     * Method for returning the properties.
     *
     * @return array
     * @throws Exception
     */
    protected function properties()
    {
        return [
            'description' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_DESCRIPTION',
                'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION',
                'isys_catg_smview_cable_wire_list__description',
                'isys_catg_smview_cable_wire_list'
            )),
            'color' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_COLOR',
                'LC__CMDB__CATG__SMVIEW_CABLE__COLOR',
                'isys_catg_smview_cable_wire_list__color',
                'isys_catg_smview_cable_wire_list'
            )),
            // only used to transfer data to UI
            'state_in' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_STATE_IN',
                        C__PROPERTY__INFO__DESCRIPTION => 'State In'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_STATE_IN',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_wire',
                                'callback_property_states'
                                ]
                            ),
                            'p_strClass' => 'input-medium'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'state_out' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_STATE_OUT',
                        C__PROPERTY__INFO__DESCRIPTION => 'State Out'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_STATE_OUT',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_wire',
                                'callback_property_states'
                                ]
                            ),
                            'p_strClass' => 'input-medium'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'description_incoming' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION_INCOMING',
                        C__PROPERTY__INFO__DESCRIPTION => 'Description Incoming'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__description_incoming'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_DESCRIPTION_INCOMING',
                        C__PROPERTY__UI__PARAMS => [
                            'p_strClass' => 'input-medium'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => false,
                        C__PROPERTY__PROVIDES__LIST   => false,
                        C__PROPERTY__PROVIDES__REPORT => false
                    ]
                ]
            ),
            // only used to transfer data to UI
            'description_outgoing' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION_OUTGOING',
                        C__PROPERTY__INFO__DESCRIPTION => 'Description Outgoing'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__description_outgoing'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_DESCRIPTION_OUTGOING',
                        C__PROPERTY__UI__PARAMS => [
                            'p_strClass' => 'input-medium'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH => false,
                        C__PROPERTY__PROVIDES__LIST   => false,
                        C__PROPERTY__PROVIDES__REPORT => false
                    ]
                ]
            ),
            // only used to transfer data to UI
            'in_use' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IN_USE',
                        C__PROPERTY__INFO__DESCRIPTION => 'In Use'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__fake_field_name'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_WIRE_IN_USE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => get_smarty_arr_YES_NO()
                        ]
                    ],
                    C__PROPERTY__FORMAT => [
                        C__PROPERTY__FORMAT__CALLBACK => [
                            'isys_export_helper',
                            'get_yes_or_no'
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'section_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__SHD__SECTION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Section'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD    => 'isys_catg_smview_cable_wire_list__section',
                        C__PROPERTY__DATA__READONLY => true,
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_SECTION',
                        C__PROPERTY__UI__PARAMS => [
                            'p_bReadonly' => true,
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'route_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__SHD__ROUTE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Route'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD    => 'isys_catg_smview_cable_wire_list__route',
                        C__PROPERTY__DATA__READONLY => true,
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_ROUTE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_bReadonly' => true,
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'virtual_route_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__SHD__VIRTUAL_ROUTE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Virtual Route'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD    => 'isys_catg_smview_cable_wire_list__virtual_route',
                        C__PROPERTY__DATA__READONLY => true,
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_bReadonly' => true,
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            // only used to transfer data to UI
            'extended_location' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::text(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__EXTENDED_LOCATION',
                        C__PROPERTY__INFO__DESCRIPTION => 'Extended Location'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD    => 'isys_catg_smview_cable_wire_list__extended_location',
                        C__PROPERTY__DATA__READONLY => true,
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_EXTENDED_LOCATION',
                        C__PROPERTY__UI__PARAMS => [
                            'p_bReadonly' => true,
                        ]
                    ],
                    C__PROPERTY__PROVIDES => [
                        C__PROPERTY__PROVIDES__SEARCH     => false,
                        C__PROPERTY__PROVIDES__LIST       => false,
                        C__PROPERTY__PROVIDES__REPORT     => false,
                        C__PROPERTY__PROVIDES__VALIDATION => false,
                        C__PROPERTY__PROVIDES__VIRTUAL    => true,
                    ]
                ]
            ),
            'scope' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_SCOPE',
                'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_SCOPE',
                'isys_catg_smview_cable_wire_list__scope',
                'isys_catg_smview_cable_wire_list'
            )),
            'scope_index' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_SCOPE_INDEX',
                'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_SCOPE_INDEX',
                'isys_catg_smview_cable_wire_list__scope_index',
                'isys_catg_smview_cable_wire_list'
            ))->mergePropertyUiParams(
                [
                    'p_strClass'   => 'input-small',
                ]
            ),
            'reserved' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__RESERVED',
                        C__PROPERTY__INFO__DESCRIPTION => 'Reserved'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__reserved',
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT (CASE WHEN isys_catg_smview_cable_wire_list__reserved = \'1\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__YES') . ' WHEN isys_catg_smview_cable_wire_list__reserved = \'0\' THEN ' .
                            $this->convert_sql_text('LC__UNIVERSAL__NO') . ' END) FROM isys_catg_smview_cable_wire_list',
                            'isys_catg_smview_cable_wire_list',
                            'isys_catg_smview_cable_wire_list__id',
                            'isys_catg_smview_cable_wire_list__isys_obj__id'
                        ),
                        C__PROPERTY__DATA__JOIN   => [
                            idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory('isys_catg_smview_cable_wire_list', 'LEFT', 'isys_catg_smview_cable_wire_list__isys_obj__id', 'isys_obj__id')
                        ]
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_RESERVED',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData'   => get_smarty_arr_YES_NO(),
                            'p_strClass' => 'input-small'
                        ]
                    ]
                ]
            ),
            'user_state_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__USER_STATE',
                        C__PROPERTY__INFO__DESCRIPTION => 'User Status'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD => 'isys_catg_smview_cable_wire_list__user_state_id'
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_USER_STATE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_wire',
                                'callback_property_user_states'
                                ]
                            )
                        ]
                    ]
                ]
            ),
            'user_state' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_STATE',
                'LC__CMDB__CATG__SMVIEW_CABLE__STATE',
                'isys_catg_smview_cable_wire_list__user_state',
                'isys_catg_smview_cable_wire_list'
            )),
            'connection_type_id' => array_replace_recursive(
                isys_cmdb_dao_category_pattern::dialog(),
                [
                    C__PROPERTY__INFO => [
                        C__PROPERTY__INFO__TITLE       => 'LC__CMDB__CATG__SMVIEW_CABLE__CONNECTION_TYPE',
                        C__PROPERTY__INFO__DESCRIPTION => 'Connection Type'
                    ],
                    C__PROPERTY__DATA => [
                        C__PROPERTY__DATA__FIELD        => 'isys_catg_smview_cable_wire_list__connection_type_id',
                        C__PROPERTY__DATA__SOURCE_TABLE => 'isys_catg_smview_cable_connection_type',
                        C__PROPERTY__DATA__REFERENCES   => [
                            'isys_catg_smview_cable_connection_type',
                            'isys_catg_smview_cable_connection_type__id'
                        ],
                        C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                            'SELECT t.isys_catg_smview_cable_connection_type__name FROM isys_catg_smview_cable_connection_type t
                            INNER JOIN isys_catg_smview_cable_wire_list l ON t.isys_catg_smview_cable_connection_type__id = l.isys_catg_smview_cable_wire_list__connection_type_id',
                            'isys_catg_smview_cable_wire_list',
                            'isys_catg_smview_cable_wire_list__id',
                            'isys_catg_smview_cable_wire_list__isys_obj__id',
                            '',
                            '',
                            null,
                            idoit\Module\Report\SqlQuery\Structure\SelectGroupBy::factory(['isys_catg_smview_cable_wire_list__isys_obj__id'])
                        )
                    ],
                    C__PROPERTY__UI => [
                        C__PROPERTY__UI__ID     => 'C__CATG__SMVIEW_CABLE_CONNECTION_TYPE',
                        C__PROPERTY__UI__PARAMS => [
                            'p_arData' => new isys_callback(
                                [
                                'isys_cmdb_dao_category_g_smview_cable_wire',
                                'callback_property_connection_types'
                                ]
                            )
                        ]
                    ]
                ]
            ),
            'damping' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_DAMPING',
                'LC__CMDB__CATG__SMVIEW_CABLE__DAMPING',
                'isys_catg_smview_cable_wire_list__damping',
                'isys_catg_smview_cable_wire_list'
            ))->mergePropertyUiParams(
                [
                    'p_strClass'   => 'input-small',
                ]
            ),
            'notice' => (new TextProperty(
                'C__CATG__SMVIEW_CABLE_NOTICE',
                'LC__CMDB__CATG__SMVIEW_CABLE__NOTICE',
                'isys_catg_smview_cable_wire_list__notice',
                'isys_catg_smview_cable_wire_list'
            )),
        ];
    }
    
    /**
     * Callback method for user states.
     *
     * @return array
     */
    public function callback_property_user_states()
    {
        $dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
        $res = $dao->get_objects();
        $states = [];
        foreach ($res as $status) {
            $name = $status["name"] . ': ' . $status["description"];
            $states[$status['id']] = $name;
        }
        return $states;
    }

    /**
     * Callback method for connection types.
     *
     * @return array
     * @throws Exception
     */
    public function callback_property_connection_types()
    {
        $language = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_smview_cable_connection_type($this->m_db);
        $res = $dao->get_data();
        $types = [];
        foreach ($res as $type) {
            $description = $type['name'] . ' ('
                . $language->get('LC__CMDB__CATG__SMVIEW_CABLE__COUNT') . ': '
                . $type['count'] . ')';
            $types[$type['id']] = $description;
        }
        return $types;
    }
    
    /**
     * Callback method for states.
     *
     * @return array
     */
    public function callback_property_states()
    {
        return isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
    }

    /**
     * Get additional wire information by ID.
     *
     * @param  int $wire_id
     * @param  boolean $use_web_output
     * @return array
     * @throws Exception
     */
    public function get_additional_wire_information_by_id($wire_id, $use_web_output = false)
    {
        $wire_dao = new isys_cmdb_dao_smview_cable_wire($this->m_db);
        $p_row = $wire_dao->get_object('id', $wire_id);
        $p_row = $this->_add_section_or_route_information($p_row, $wire_id, $use_web_output);

        if ($p_row['user_state_id']) {
            if (empty($this->_cache_user_states)) {
                $status_dao = new isys_cmdb_dao_smview_cable_user_state($this->m_db);
                $status = $status_dao->get_objects();
                foreach ($status as $s) {
                    $this->_cache_user_states[$s['id']] = $s['name'] ?? '';
                }
            }
            $p_row['user_state_name'] = $this->_cache_user_states[$p_row['user_state_id']] ?? '';
        }
        
        if ($p_row['connection_type_id']) {
            if (empty($this->_cache_connection_types)) {
                $conn_dao = new isys_cmdb_dao_smview_cable_connection_type($this->m_db);
                $conn = $conn_dao->get_objects();
                foreach ($conn as $c) {
                    $this->_cache_connection_types[$c['id']] = $c['name'] ?? '';
                }
            }
            $p_row['connection_type'] = $this->_cache_connection_types[$p_row['connection_type_id']] ?? '';
        }
        
        // add informations of wire pins
        $dao = new isys_cmdb_dao_smview_cable_wire_pin($this->m_db);
        $pins = $dao->filter_objects(array('wire_id' => $wire_id));
        $pin_left = $pins[0]['id'] < $pins[1]['id'] ? $pins[0] : $pins[1];
        $pin_right = $pins[0]['id'] > $pins[1]['id'] ? $pins[0] : $pins[1];

        if (empty($this->_cache_wire_states)) {
            $this->_cache_wire_states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
        }
        $leftState = isset($pin_left['status_id']) ? $this->_cache_wire_states[$pin_left['status_id']] : null;
        $rightState = isset($pin_right['status_id']) ? $this->_cache_wire_states[$pin_right['status_id']] : null;
        $p_row['state_in'] = $leftState;
        $p_row['state_out'] = $rightState;
        $p_row['description_incoming'] = $pin_left['description'];
        $p_row['description_outgoing'] = $pin_right['description'];
        
        $pin_in = $this->_get_pin_output($pin_left, $use_web_output);
        $p_row['pin_in_object_id'] = $pin_in['object_id'];
        $p_row['pin_in_object'] = $pin_in['object'];
        $p_row['pin_in_id'] = $pin_in['pin_id'];
        $p_row['pin_in_purpose'] = $pin_in['purpose'];
        $p_row['pin_in_scope'] = $pin_in['scope'];
        $p_row['pin_in_description'] = $pin_in['description'];
        $pin_out = $this->_get_pin_output($pin_right, $use_web_output);
        $p_row['pin_out_object_id'] = $pin_out['object_id'];
        $p_row['pin_out_object'] = $pin_out['object'];
        $p_row['pin_out_id'] = $pin_out['pin_id'];
        $p_row['pin_out_purpose'] = $pin_out['purpose'];
        $p_row['pin_out_scope'] = $pin_out['scope'];
        $p_row['pin_out_description'] = $pin_out['description'];

        $extendedLocationDao = new isys_cmdb_dao_category_g_smview_cable_extended_location($this->m_db);
        $location = $extendedLocationDao->get_data(null, $p_row['obj_id'])->get_row();
        $extendedLocationId = $location['isys_catg_smview_cable_extended_location_list__location_id'];
        if ($extendedLocationId) {
            $sql = "SELECT * FROM isys_shd_extended_location WHERE isys_shd_extended_location__id = $extendedLocationId";
            $l = $extendedLocationDao->retrieve($sql)->get_row();
            $p_row['extended_location'] = $l['isys_shd_extended_location__title'];
        } else {
            $p_row['extended_location'] = '';
        }

        return $p_row;
    }
    
    /**
     * Get a human readable output of a connected pin.
     *
     * @param  int     $wire_pin
     * @param  boolean $use_web_output
     * @return array
     */
    private function _get_pin_output($wire_pin, $use_web_output = false)
    {
        global $g_dirs;
        
        $pin_dao = isys_cmdb_dao_smview_cable_pin::instance($this->get_database_component());
        $obj_dao = isys_cmdb_dao_category_g_global::instance($this->get_database_component());
        $output = [];
        if (! empty($wire_pin) && ! empty($wire_pin['connected_pin_id'])) {
            $pin = $pin_dao->get_pin_by_id($wire_pin['connected_pin_id']);
            $obj = $obj_dao->get_data(null, $pin['obj_id'])->get_row();
            
            $output['pin_id'] = $wire_pin['connected_pin_id'];
            $output['object_id'] = $obj['isys_obj__id'];
            $output['object'] = $obj['isys_obj__title'];
            if ($use_web_output) {
                $title = $obj['isys_obj__title'];
                $text = "<img src='" . $g_dirs['images'] . "icons/silk/link.png' class='vam' /> $title";
                $l_quickinfo = isys_factory::get_instance('isys_ajax_handler_quick_info');
                $output['object'] = $l_quickinfo->get_quick_info(
                    $pin['obj_id'],
                    $text,
                    defined_or_default('C__LINK__OBJECT'));
            }
            
            $description = $pin['description'];
            if (! empty($wire_pin['status_id'])) {
                $states = isys_cmdb_dao_smview_cable_wire_pin::get_wire_states();
                $status = $states[$wire_pin['status_id']];
                $description .= ' (' . $status . ')';
            }
            $output['description'] = $description;
            if ($use_web_output) {
                $description = "<img src='" . $g_dirs['images'] . "icons/silk/link.png' class='vam' /> $description";
                $output['description'] = $l_quickinfo->get_quick_info(
                    $pin['obj_id'],
                    $description,
                    defined_or_default('C__LINK__CATG'),
                    false,
                    array(C__CMDB__GET__CATG => defined_or_default('C__CATG__SMVIEW_CABLE_PIN')));
            }
            $output['scope'] = $pin['classification'];
            $output['purpose'] = $obj['isys_purpose__title'];
        }
        return $output;
    }

    /**
     * Check, if a wire is in use by a section or a route.
     *
     * @param  array $p_row
     * @param  int $wire_id
     * @param  boolean $use_web_output
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _add_section_or_route_information($p_row, $wire_id, $use_web_output)
    {
        $section_element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        $section_element = $section_element_dao->get_object('wire_id', $wire_id);
        $route_element_dao = new isys_cmdb_dao_smview_cable_route_element($this->m_db);
        $route_element = $route_element_dao->get_object('wire_id', $wire_id);
        $virtual_route_element_dao = new isys_cmdb_dao_smview_cable_virtual_route_element($this->m_db);
        $virtual_route_element = $virtual_route_element_dao->get_object('wire_id', $wire_id);

        if (! empty($section_element)) {
            $section_dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
            $p_row['section_id'] = $this->_get_object_info($section_dao, $section_element['section_id'], $use_web_output);

            // check if this section is part of a route
            $route_element = $route_element_dao->get_object('section_id', $section_element['section_id']);

            // check if this section is part of a virtual route
            $virtual_route_element = $virtual_route_element_dao->get_object('section_id', $section_element['section_id']);
        }

        if (! empty($route_element)) {
            $route_dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
            $p_row['route_id'] = $this->_get_object_info($route_dao, $route_element['route_id'], $use_web_output);
        }

        if (! empty($virtual_route_element)) {
            $virtual_route_dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
            $p_row['virtual_route_id'] = $this->_get_object_info($virtual_route_dao, $virtual_route_element['route_id'], $use_web_output);
        }
        $p_row['in_use'] = $this->get_in_use($wire_id, $section_element, $route_element);
        return $p_row;
    }

    /**
     * Get object information.
     *
     * @param  isys_cmdb_dao_smview_cable_abstract $dao
     * @param  int $id
     * @param  boolean $use_web_output
     * @return string
     * @throws isys_exception_database
     * @global array $g_dirs
     */
    private function _get_object_info($dao, $id, $use_web_output)
    {
        global $g_dirs;

        $element = $dao->get_object('id', $id);
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $obj = $global_dao->get_object_by_id($element['obj_id'])->get_row();

        $info = $obj['isys_obj__id'];
        if ($use_web_output) {
            $title = $obj['isys_obj__title'];
            $text = "<img src='" . $g_dirs['images'] . "icons/silk/link.png' class='vam' /> $title";
            $l_quickinfo = isys_factory::get_instance('isys_ajax_handler_quick_info');
            $info = $l_quickinfo->get_quick_info($element['obj_id'], $text, defined_or_default('C__LINK__OBJECT'));
        }
        return $info;
    }

    /**
     * Check, if a wire is in use by a section or a route.
     *
     * @param  int $wire_id
     * @param  array|null $section_element
     * @param  array|null $route_element
     * @return string
     * @throws Exception
     * @global array $g_dirs
     */
    public function get_in_use($wire_id, $section_element = null, $route_element = null)
    {
        $language = isys_application::instance()->container->get('language');
        if (! empty($section_element) || ! empty($route_element)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED');
        }

        $in_use = '';
        $section_element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        $section_element = $section_element_dao->get_object('wire_id', $wire_id);
        $route_element_dao = new isys_cmdb_dao_smview_cable_route_element($this->m_db);
        $route_element = $route_element_dao->get_object('wire_id', $wire_id);

        if (! empty($section_element) || ! empty($route_element)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED');
        }

        $conn_dao = isys_cmdb_dao_smview_cable_wire_pin::instance($this->get_database_component());
        $wire_pins = $conn_dao->filter_objects(array('wire_id' => $wire_id));
        $wire_pin_left = $wire_pins[0]['id'] < $wire_pins[1]['id'] ? $wire_pins[0] : $wire_pins[1];
        $wire_pin_right = $wire_pins[0]['id'] > $wire_pins[1]['id'] ? $wire_pins[0] : $wire_pins[1];
        if (! empty($wire_pin_left) && ! empty($wire_pin_left['connected_pin_id']) &&
            ! empty($wire_pin_right) && ! empty($wire_pin_right['connected_pin_id'])) {
            $in_use = $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_DISPOSED');
        }
        return $in_use;
    }

    /**
     * Find usages of the given object.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function find_usages($obj_id)
    {
        $global_dao = new isys_cmdb_dao_category_g_global($this->m_db);
        $route_dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
        $route_element_dao = new isys_cmdb_dao_smview_cable_route_element($this->m_db);
        $route_elements = $route_element_dao->filter_objects(array('wire_id' => $obj_id));

        $r_elements = [];
        foreach ($route_elements as $e) {
            $route = $route_dao->get_object('id', $e['route_id']);
            $obj = $global_dao->get_object($route['obj_id'])->get_row();
            $r_elements[$route['id']] = [
                'id'         => $route['id'],
                'obj_id'     => $route['obj_id'],
                'title'      => $obj['isys_obj__title'],
                'type_id'    => $obj['isys_obj_type__id'],
                'type_title' => $obj['isys_obj_type__title'],
            ];
        }

        $section_dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $section_element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        $section_elements = $section_element_dao->filter_objects(array('wire_id' => $obj_id));
        $s_elements = [];
        foreach ($section_elements as $e) {
            $section = $section_dao->get_object('id', $e['section_id']);
            $obj = $global_dao->get_object($section['obj_id'])->get_row();
            $s_elements[$section['id']] = [
                'id'         => $section['id'],
                'obj_id'     => $section['obj_id'],
                'title'      => $obj['isys_obj__title'],
                'type_id'    => $obj['isys_obj_type__id'],
                'type_title' => $obj['isys_obj_type__title'],
            ];
        }
        return array_merge(array_values($r_elements), array_values($s_elements));
    }
    
    /**
     * Creates new entity.
     *
     * @param  array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     * @return mixed  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        $color = isset($p_data['color']) ? $p_data['color'] : null;
        $scope = isset($p_data['scope']) ? $p_data['scope'] : null;
        $scope_index = isset($p_data['scope_index']) ? $p_data['scope_index'] : null;
        $reserved = isset($p_data['reserved']) ? $p_data['reserved'] : null;
        $user_state_id = isset($p_data['user_state_id']) && $p_data['user_state_id'] != '-1' ? $p_data['user_state_id'] : null;
        $user_state = isset($p_data['user_state']) ? $p_data['user_state'] : null;
        $connection_type_id = isset($p_data['connection_type_id']) && $p_data['connection_type_id'] != '-1' ? $p_data['connection_type_id'] : null;
        $damping = isset($p_data['damping']) ? $p_data['damping'] : null;
        $notice = isset($p_data['notice']) ? $p_data['notice'] : null;

        $dao = new isys_cmdb_dao_smview_cable_wire($this->m_db);
        $wire_id = $dao->create_wire($p_data['isys_obj__id'], $p_data['description'], $color, $scope, $reserved, $scope_index, $user_state_id, $user_state, $connection_type_id, $damping, $notice);
        
        $this->_update_wire_pins($wire_id, $p_data, true);
        return $wire_id;
    }
    
    /**
     * Updates existing entity.
     *
     * @param  integer $p_bsm_list_id Entity's identifier
     * @param  array   $p_data        Properties in a associative array with tags as keys and their corresponding values as values.
     * @return boolean
     */
    public function save_data($p_bsm_list_id, $p_data)
    {
        $color = isset($p_data['color']) ? $p_data['color'] : null;
        $scope = isset($p_data['scope']) ? $p_data['scope'] : null;
        $scope_index = isset($p_data['scope_index']) ? $p_data['scope_index'] : null;
        $reserved = isset($p_data['reserved']) ? $p_data['reserved'] : null;
        $user_state_id = isset($p_data['user_state_id']) && $p_data['user_state_id'] != '-1' ? $p_data['user_state_id'] : null;
        $user_state = isset($p_data['user_state']) ? $p_data['user_state'] : null;
        $connection_type_id = isset($p_data['connection_type_id']) && $p_data['connection_type_id'] != '-1' ? $p_data['connection_type_id'] : null;
        $damping = isset($p_data['damping']) ? $p_data['damping'] : null;
        $notice = isset($p_data['notice']) ? $p_data['notice'] : null;

        $dao = new isys_cmdb_dao_smview_cable_wire($this->m_db);
        $success = $dao->update_wire($p_bsm_list_id, $p_data['description'], $color, $scope, $reserved, $scope_index, $user_state_id, $user_state, $connection_type_id, $damping, $notice);
        
        $this->_update_wire_pins($p_bsm_list_id, $p_data);
        return $success;
    }
    
    /**
     * Update pins of wire.
     *
     * @param int   $wire_id
     * @param array $p_data
     */
    private function _update_wire_pins($wire_id, $p_data, $isNew = false)
    {
        $dao = new isys_cmdb_dao_smview_cable_wire_pin($this->m_db);
        $state_in = isset($p_data['state_in']) ? $p_data['state_in'] : null;
        $state_out = isset($p_data['state_out']) ? $p_data['state_out'] : null;
        $desc_in = isset($p_data['description_incoming']) ? $p_data['description_incoming'] : null;
        $desc_out = isset($p_data['description_outgoing']) ? $p_data['description_outgoing'] : null;
        
        if ($isNew) {
            if (! $desc_in) {
                $desc_in = $p_data['description'];
            }
            if (! $desc_out) {
                $desc_out = $p_data['description'];
            }
        }
        
        $pins = $dao->filter_objects(array('wire_id' => $wire_id));
        $pin_left = $pins[0]['id'] < $pins[1]['id'] ? $pins[0] : $pins[1];
        $pin_right = $pins[0]['id'] > $pins[1]['id'] ? $pins[0] : $pins[1];
        
        $dao->update_wire_pin($pin_left['id'], $desc_in, $state_in, $pin_left['sibling_id'], $pin_left['connected_pin_id'], true);
        $dao->update_wire_pin($pin_right['id'], $desc_out, $state_out, $pin_right['sibling_id'], $pin_right['connected_pin_id'], true);
    }

    /**
     * Update method.
     *
     * Handles check of objects, used in sections or routes.
     *
     * @param  string $p_query
     * @return boolean
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function update($p_query)
    {
        if (strpos($p_query, '__status') === false) {
            // no status will be changed
            return parent::update($p_query);
        }
        
        if (strpos($p_query, "__status = '2'") !== false) {
            // status can be changed to 2 ("Normal")
            return parent::update($p_query);
        }
        
        // check if object is in use
        $list_pattern = "/list__id='([\d]*)'/";
        $matches = [];
        $res = preg_match($list_pattern, $p_query, $matches);
        if ($res !== false) {
            // found list ID
            $list_id = (int) $matches[1];
            $wire_catg = new isys_cmdb_dao_category_g_smview_cable_wire($this->m_db);
            $language = isys_application::instance()->container->get('language');
            $in_use = $wire_catg->get_in_use($list_id);
            if (empty($in_use) || $in_use != $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_WIRED')) {
                return parent::update($p_query);
            }
        }
        
        // abort update if object is in use
        $language = isys_application::instance()->container->get('language');
        isys_notify::error($language->get('LC__CMDB__CATG__SMVIEW_CABLE__ERROR_PIN_IN_USE'));
        return true;
    }
}
