<?php

/**
 * i-doit
 *
 * DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_list__id'               => 'id',
        'isys_catg_smview_cable_list__isys_obj__id'     => 'obj_id',
        'isys_catg_smview_cable_list__state_id'         => 'state_id',
        'isys_catg_smview_cable_list__user_state'       => 'user_state',
        'isys_catg_smview_cable_list__group_id'         => 'group_id',
        'isys_catg_smview_cable_list__length'           => 'length',
        'isys_catg_smview_cable_list__capacity'         => 'capacity',
        'isys_catg_smview_cable_list__net_type'         => 'net_type',
        'isys_catg_smview_cable_list__admin_unit_id'    => 'admin_unit_id',
        'isys_catg_smview_cable_list__type_id'          => 'type_id',
        'isys_catg_smview_cable_list__location_in_id'   => 'location_in_id',
        'isys_catg_smview_cable_list__location_out_id'  => 'location_out_id',
        'isys_catg_smview_cable_list__is_section_cable' => 'is_section_cable',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_list__id';

    /**
     * Get data method.
     *
     * @param null $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_data($obj_id = null)
    {
        $query = "SELECT * FROM isys_catg_smview_cable_list";
        if ($obj_id) {
            $query .= " WHERE isys_catg_smview_cable_list__isys_obj__id=$obj_id";
        }
        $query .= ";";
        $res = $this->retrieve($query);
        return $this->_map_result_to_array($res);
    }

    /**
     * Get cable count between two locations.
     *
     * @param int $sourceLocationId
     * @param int $targetLocationId
     * @return int
     * @throws isys_exception_database
     */
    public function get_cable_count($sourceLocationId, $targetLocationId)
    {
        $tableName = $this->table_name;
        $query = "SELECT count(*) AS count, isys_catg_smview_cable_list__location_in_id, isys_catg_smview_cable_list__location_out_id FROM $tableName"
            . " WHERE isys_catg_smview_cable_list__location_in_id=$sourceLocationId AND isys_catg_smview_cable_list__location_out_id=$targetLocationId"
            . " GROUP BY isys_catg_smview_cable_list__location_in_id, isys_catg_smview_cable_list__location_out_id;";
        $res = $this->retrieve($query);
        if ($res->count() > 0) {
            $row = $res->get_row();
            return $row['count'];
        } else {
            return 0;
        }
    }

    /**
     * Update an existing object.
     *
     * @param int $id
     * @param array $params
     * @param array $optionalParams
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $current_cable = $this->get_object('id', $id);
        $update_length = $current_cable['length'] != $optionalParams['length'];
        $ret = parent::update_object($id, $params, $optionalParams);
        
        if (! $update_length) {
            return $ret;
        }
        
        // update calculated length of sections and routes
        $wire_dao = isys_cmdb_dao_smview_cable_wire::instance(isys_application::instance()->database);
        $wires = $wire_dao->filter_objects(array('obj_id' => $current_cable['obj_id']));

        $section_element_dao = isys_cmdb_dao_smview_cable_section_element::instance(isys_application::instance()->database);
        $route_element_dao = isys_cmdb_dao_smview_cable_route_element::instance(isys_application::instance()->database);
        foreach ($wires as $wire) {
            $section_elements = $section_element_dao->filter_objects(array('wire_id' => $wire['id']));
            foreach ($section_elements as $element) {
                $section_element_dao->update_section_length($element['id']);
                
                // update also sections used within a route
                $route_elements = $route_element_dao->filter_objects(array('section_id' => $element['section_id']));
                foreach ($route_elements as $element) {
                    $route_id = $route_element_dao->get_route_id_by_element($element['id']);
                    $route_element_dao->update_route_length($route_id);
                }
            }
            
            $route_elements = $route_element_dao->filter_objects(array('wire_id' => $wire['id']));
            foreach ($route_elements as $element) {
                $route_id = $route_element_dao->get_route_id_by_element($element['id']);
                $route_element_dao->update_route_length($route_id);
            }
        }
        return $ret;
    }
}
