<?php

/**
 * i-doit
 *
 * Abstract View DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class isys_cmdb_dao_smview_cable_abstract_view extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = [];
    
    /**
     * Constructor. Assigns database component.
     *
     * @param isys_component_database $p_db
     */
    public function __construct(isys_component_database $p_db)
    {
        parent::__construct($p_db);
        
        // map values of select fields to required DB-mapping
        $fields = array_values($this->select_field_mapping);
        $this->db_to_api_map = array_combine($fields, $fields);
    }
    
    /**
     * Get base query.
     *
     * @return string
     */
    abstract protected function _get_base_query();

    /**
     * Get all objects.
     *
     * @param  string $group_by
     * @return array
     */
    public function get_objects($group_by = null)
    {
        $query = $this->_get_base_query();
        if (isset($group_by)) {
            $query .= " GROUP BY $group_by;";
        }
        
        $res = $this->retrieve($query);
        return $this->_map_result_to_array($res);
    }
    
    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @param  string $group_by
     * @return array
     */
    public function get_object($field, $value, $group_by = null)
    {
        $db_field = array_search($field, $this->select_field_mapping);
        
        $query = $this->_get_base_query();
        $query .= " AND $db_field = '$value' ";
        if (isset($group_by)) {
            $query .= " GROUP BY $group_by;";
        }
        
        $res = $this->retrieve($query);
        $objects = $this->_map_result_to_array($res);
        return ! empty($objects) ? $objects[0] : array();
    }
    
    /**
     * Filter objects.
     *
     * @param  array  $filter
     * @param  string $group_by
     * @return array
     */
    public function filter_objects($filter, $group_by = null)
    {
        $query = $this->_get_base_query();
        foreach ($filter as $field => $value) {
            $db_field = array_search($field, $this->select_field_mapping);
            if (! $db_field) {
                continue;
            }
            if (strpos($value, '%') !== false) {
                $query .= " AND $db_field like '$value'";
            } else {
                $query .= " AND $db_field = '$value'";
            }
        }
        if (isset($group_by)) {
            $query .= " GROUP BY $group_by;";
        }
        
        $res = $this->retrieve($query);
        return $this->_map_result_to_array($res);
    }
    
    public function create_object($params, $optionalParams = [])
    {
        // not supported on SQL-Views
    }
    
    public function update_object($id, $params, $optionalParams = [])
    {
        // not supported on SQL-Views
    }
    
    public function delete_object($id)
    {
        // not supported on SQL-Views
    }
}
