<?php

/**
 * i-doit
 *
 * Connection View DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_connection_view extends isys_cmdb_dao_smview_cable_abstract_view
{
    
    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = array(
        'wire.isys_catg_smview_cable_wire_list__id'                 => 'wire_id',
        'wire.isys_catg_smview_cable_wire_list__description'        => 'wire_title',
        'wire.isys_catg_smview_cable_wire_list__scope'              => 'wire_scope',
        'wire.isys_catg_smview_cable_wire_list__user_state_id'      => 'user_state_id',
        'wire.isys_catg_smview_cable_wire_list__user_state'         => 'user_state',
        'ct.isys_catg_smview_cable_connection_type__name'           => 'wire_connection_type',
        'wp_left.isys_catg_smview_cable_wire_pin__description'      => 'wire_pin_title_in',
        'wp_left.isys_catg_smview_cable_wire_pin__status_id'        => 'wire_pin_state_in',
        'wp_right.isys_catg_smview_cable_wire_pin__description'     => 'wire_pin_title_out',
        'wp_right.isys_catg_smview_cable_wire_pin__status_id'       => 'wire_pin_state_out',
        'obj_cable.isys_obj__id'                                    => 'cable_id',
        'obj_cable.isys_obj__title'                                 => 'cable_title',
        'obj_cable.isys_obj__isys_cmdb_status__id'                  => 'cable_cmdb_status_id',
        'cable.isys_catg_smview_cable_list__length'                 => 'length',
        'cable.isys_catg_smview_cable_list__state_id'               => 'cable_user_state_id',
        'ty.isys_catg_smview_cable_type__technology_id'             => 'technology_id',
        'wp_left.isys_catg_smview_cable_wire_pin__id'               => 'id_in',
        'pin_left.isys_catg_smview_cable_pin_list__id'              => 'pin_id_in',
        'pin_left.isys_catg_smview_cable_pin_list__description'     => 'pin_title_in',
        'pin_left.isys_catg_smview_cable_pin_list__classification'  => 'pin_scope_in',
        'pin_left.isys_catg_smview_cable_pin_list__isys_obj__id'    => 'obj_id_in',
        'obj_left.isys_obj__title'                                  => 'obj_title_in',
        'wp_right.isys_catg_smview_cable_wire_pin__id'              => 'id_out',
        'pin_right.isys_catg_smview_cable_pin_list__id'             => 'pin_id_out',
        'pin_right.isys_catg_smview_cable_pin_list__description'    => 'pin_title_out',
        'pin_right.isys_catg_smview_cable_pin_list__classification' => 'pin_scope_out',
        'pin_right.isys_catg_smview_cable_pin_list__isys_obj__id'   => 'obj_id_out',
        'obj_right.isys_obj__title'                                 => 'obj_title_out',
    );
    
    /**
     * Get base query.
     *
     * @return string
     */
    protected function _get_base_query()
    {
        $query = "SELECT ";
        foreach ($this->select_field_mapping as $db => $name) {
            $query .= " $db AS $name,";
        }
        $base_query = trim($query, ',');
        $base_query .= " FROM isys_catg_smview_cable_wire_list wire
            JOIN isys_catg_smview_cable_list cable ON cable.isys_catg_smview_cable_list__isys_obj__id = wire.isys_catg_smview_cable_wire_list__isys_obj__id
            JOIN isys_obj obj_cable ON wire.isys_catg_smview_cable_wire_list__isys_obj__id = obj_cable.isys_obj__id
            JOIN isys_catg_smview_cable_wire_pin wp_left ON wp_left.isys_catg_smview_cable_wire_pin__wire_id = wire.isys_catg_smview_cable_wire_list__id
            JOIN isys_catg_smview_cable_wire_pin wp_right ON wp_right.isys_catg_smview_cable_wire_pin__id = wp_left.isys_catg_smview_cable_wire_pin__sibling_id
            
            LEFT JOIN isys_catg_smview_cable_pin_list pin_left ON wp_left.isys_catg_smview_cable_wire_pin__connected_pin_id = pin_left.isys_catg_smview_cable_pin_list__id
            LEFT JOIN isys_obj obj_left ON pin_left.isys_catg_smview_cable_pin_list__isys_obj__id = obj_left.isys_obj__id
            LEFT JOIN isys_catg_smview_cable_pin_list pin_right ON wp_right.isys_catg_smview_cable_wire_pin__connected_pin_id = pin_right.isys_catg_smview_cable_pin_list__id
            LEFT JOIN isys_obj obj_right ON pin_right.isys_catg_smview_cable_pin_list__isys_obj__id = obj_right.isys_obj__id
            
            LEFT JOIN isys_catg_smview_cable_type ty ON cable.isys_catg_smview_cable_list__type_id = ty.isys_catg_smview_cable_type__id
            LEFT JOIN isys_catg_smview_cable_connection_type ct ON ct.isys_catg_smview_cable_connection_type__id = wire.isys_catg_smview_cable_wire_list__connection_type_id
            WHERE 1 = 1
            AND wp_left.isys_catg_smview_cable_wire_pin__id < wp_left.isys_catg_smview_cable_wire_pin__sibling_id
            AND (wp_left.isys_catg_smview_cable_wire_pin__connected_pin_id IS NOT NULL OR wp_right.isys_catg_smview_cable_wire_pin__connected_pin_id IS NOT NULL) ";
        return $base_query;
    }

    /**
     * Get all objects.
     *
     * @param  string $group_by
     * @return array
     */
    public function get_objects($group_by = null)
    {
        return parent::get_objects(null);
    }
    
    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @param  string $group_by
     * @return array
     */
    public function get_object($field, $value, $group_by = null)
    {
        return parent::get_object($field, $value, 'wire_id');
    }
    
    /**
     * Filter objects.
     *
     * @param  array $filter
     * @param  string $group_by
     * @return array
     */
    public function filter_objects($filter, $group_by = null)
    {
        return parent::filter_objects($filter, 'wire_id');
    }
}
