<?php

/**
 * i-doit
 *
 * ETSI Device Color DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_etsi_device_color extends isys_cmdb_dao_smview_cable_abstract
{

    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_etsi_device_color__id'                 => 'id',
        'isys_catg_smview_cable_etsi_device_color__description'        => 'description',
        'isys_catg_smview_cable_etsi_device_color__layout'             => 'layout',
        'isys_catg_smview_cable_etsi_device_color__title_left_top'     => 'title_left_top',
        'isys_catg_smview_cable_etsi_device_color__color_left_top'     => 'color_left_top',
        'isys_catg_smview_cable_etsi_device_color__title_left_bottom'  => 'title_left_bottom',
        'isys_catg_smview_cable_etsi_device_color__color_left_bottom'  => 'color_left_bottom',
        'isys_catg_smview_cable_etsi_device_color__title_right_top'    => 'title_right_top',
        'isys_catg_smview_cable_etsi_device_color__color_right_top'    => 'color_right_top',
        'isys_catg_smview_cable_etsi_device_color__title_right_bottom' => 'title_right_bottom',
        'isys_catg_smview_cable_etsi_device_color__color_right_bottom' => 'color_right_bottom',
        'isys_catg_smview_cable_etsi_device_color__wire_count'         => 'wire_count',
        'wire_states'                                                  => 'wire_states',
    );

    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_etsi_device_color';

    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_etsi_device_color__id';

    /**
     * Get all objects.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_objects()
    {
        $objects = parent::get_objects();
        $mod_objects = [];
        foreach ($objects as $o) {
            $sql = 'SELECT
                ws.isys_catg_smview_cable_etsi_device_color_wire_status__operator as operator,
                ws.isys_catg_smview_cable_etsi_device_color_wire_status__not as neg,
                ps.isys_catg_smview_cable_wire_pin_status__const as const
                FROM isys_catg_smview_cable_etsi_device_color_wire_status as ws
                JOIN isys_catg_smview_cable_wire_pin_status ps
                    ON ps.isys_catg_smview_cable_wire_pin_status__id = ws.isys_catg_smview_cable_etsi_device_color_wire_status__status_id
                WHERE ws.isys_catg_smview_cable_etsi_device_color_wire_status__device_id = ' . $o['id'] . '
                ORDER BY ws.isys_catg_smview_cable_etsi_device_color_wire_status__id;';
            $res = $this->retrieve($sql);
            $status_string = '';
            $lastOp = null;
            while ($row = $res->get_row()) {
                $status_string .= $lastOp ? " $lastOp " : '';
                $status_string .= $row['neg'] !== null ? '! ' : '';
                $status_string .= $row['const'];
                $lastOp = $row['operator'];
                $o['wire_states'] = $status_string;
            }
            $mod_objects[] = $o;
        }
        return $mod_objects;
    }

    /**
     * Delete an existing object.
     *
     * @param  int $id
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function delete_object($id)
    {
        // remove current wire status
        $dao = new isys_cmdb_dao_smview_cable_etsi_device_color_wire_status(isys_application::instance()->database);
        $ws = $dao->filter_objects(['device_id' => $id]);
        foreach ($ws as $s) {
            $dao->delete_object($s['id']);
        }

        return parent::delete_object($id);
    }
}
