<?php

/**
 * i-doit
 *
 * Free Section View DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_free_section_view extends isys_cmdb_dao_smview_cable_abstract_view
{

    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = array(
        's.isys_catg_smview_cable_section_list__id'             => 'id',
        's.isys_catg_smview_cable_section_list__isys_obj__id'   => 'obj_id',
        'obj.isys_obj__title'                                   => 'title',
        's.isys_catg_smview_cable_section_list__length'         => 'length',
        's.isys_catg_smview_cable_section_list__start_obj_id'   => 'start_obj_id',
        'wp.isys_catg_smview_cable_wire_pin__connected_pin_id'  => 'start_pin_id',
        'wp.isys_catg_smview_cable_wire_pin__description'       => 'start_pin_title',
        'w.isys_catg_smview_cable_wire_list__id'                => 'start_wire_id',
        'w.isys_catg_smview_cable_wire_list__description'       => 'start_wire_title',
        'sib_left.isys_catg_smview_cable_pin_list__patch_id'    => 'start_patch_id',
        's.isys_catg_smview_cable_section_list__end_obj_id'     => 'end_obj_id',
        'wp2.isys_catg_smview_cable_wire_pin__connected_pin_id' => 'end_pin_id',
        'wp2.isys_catg_smview_cable_wire_pin__description'      => 'end_pin_title',
        'w2.isys_catg_smview_cable_wire_list__id'               => 'end_wire_id',
        'w2.isys_catg_smview_cable_wire_list__description'      => 'end_wire_title',
        'sib_right.isys_catg_smview_cable_pin_list__patch_id'   => 'end_patch_id',
    );

    /**
     * Get base query.
     *
     * @return string
     */
    protected function _get_base_query()
    {
        $query = "SELECT ";
        foreach ($this->select_field_mapping as $db => $name) {
            $query .= " $db AS $name,";
        }
        $base_query = trim($query, ',');
        $base_query .= " FROM isys_catg_smview_cable_section_list s
            JOIN isys_obj obj ON obj.isys_obj__id = s.isys_catg_smview_cable_section_list__isys_obj__id
            JOIN isys_catg_smview_cable_pin_list p ON s.isys_catg_smview_cable_section_list__start_obj_id = p.isys_catg_smview_cable_pin_list__isys_obj__id
            JOIN isys_catg_smview_cable_wire_pin wp ON p.isys_catg_smview_cable_pin_list__id = wp.isys_catg_smview_cable_wire_pin__connected_pin_id
            JOIN isys_catg_smview_cable_wire_list w ON wp.isys_catg_smview_cable_wire_pin__wire_id = w.isys_catg_smview_cable_wire_list__id
            JOIN isys_catg_smview_cable_section_element e ON s.isys_catg_smview_cable_section_list__id = e.isys_catg_smview_cable_section_element__section_id
            JOIN isys_catg_smview_cable_pin_list sib_left ON p.isys_catg_smview_cable_pin_list__sibling_id = sib_left.isys_catg_smview_cable_pin_list__id
            
            JOIN isys_catg_smview_cable_pin_list p2 ON s.isys_catg_smview_cable_section_list__end_obj_id = p2.isys_catg_smview_cable_pin_list__isys_obj__id
            JOIN isys_catg_smview_cable_wire_pin wp2 ON p2.isys_catg_smview_cable_pin_list__id = wp2.isys_catg_smview_cable_wire_pin__connected_pin_id
            JOIN isys_catg_smview_cable_wire_list w2 ON wp2.isys_catg_smview_cable_wire_pin__wire_id = w2.isys_catg_smview_cable_wire_list__id
            JOIN isys_catg_smview_cable_section_element e2 ON s.isys_catg_smview_cable_section_list__id = e2.isys_catg_smview_cable_section_element__section_id
            JOIN isys_catg_smview_cable_pin_list sib_right ON p2.isys_catg_smview_cable_pin_list__sibling_id = sib_right.isys_catg_smview_cable_pin_list__id

            LEFT JOIN isys_catg_smview_cable_route_element re ON s.isys_catg_smview_cable_section_list__id = re.isys_catg_smview_cable_route_element__section_id
            
            WHERE e.isys_catg_smview_cable_section_element__wire_id = w.isys_catg_smview_cable_wire_list__id
            AND s.isys_catg_smview_cable_section_list__user_state_id IS NULL
            AND e2.isys_catg_smview_cable_section_element__wire_id = w2.isys_catg_smview_cable_wire_list__id
            AND re.isys_catg_smview_cable_route_element__route_id IS NULL
            AND sib_left.isys_catg_smview_cable_pin_list__patch_id IS NULL
            AND sib_right.isys_catg_smview_cable_pin_list__patch_id IS NULL ";
        return $base_query;
    }

    /**
     * Get all objects.
     *
     * @param  string $group_by
     * @return array
     */
    public function get_objects($group_by = null)
    {
        return parent::get_objects('id');
    }

    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @param  string $group_by
     * @return array
     */
    public function get_object($field, $value, $group_by = null)
    {
        return parent::get_object($field, $value, 'id');
    }

    /**
     * Filter objects.
     *
     * @param  array $filter
     * @param  string $group_by
     * @return array
     */
    public function filter_objects($filter, $group_by = null)
    {
        return parent::filter_objects($filter, 'id');
    }
}
