<?php

/**
 * i-doit
 *
 * Generator DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_generator extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_generator__id'          => 'id',
        'isys_catg_smview_cable_generator__title'       => 'title',
        'isys_catg_smview_cable_generator__description' => 'description',
        'isys_catg_smview_cable_generator__mask'        => 'mask',
        'isys_catg_smview_cable_generator__counter_min' => 'counter_min',
        'isys_catg_smview_cable_generator__counter_max' => 'counter_max',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_generator';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_generator__id';
    
    /**
     * Get assigned algorithms.
     *
     * @param  int $generator_id
     * @return array
     */
    public function get_assigned_algorithms($generator_id)
    {
        $algorithm_dao = new \isys_cmdb_dao_smview_cable_generator_algorithm($this->m_db);
        $current_algorithms = $algorithm_dao->filter_objects(array('generator_id' => $generator_id));
        
        $algorithms = [];
        foreach ($current_algorithms as $algorithm) {
            $algorithm = array(
                'index'       => $algorithm['index'],
                'pos'         => $algorithm['mask_position'],
                'length'      => $algorithm['length'],
                'type'        => $algorithm['type'],
                'description' => $algorithm['description'],
                'elements'    => $algorithm['elements'],
            );
            $algorithms[$algorithm['index']] = $algorithm;
        }
        ksort($algorithms);
        return $algorithms;
    }
    
    /**
     * Get data method.
     *
     * @param  int $obj_id
     * @return array
     */
    public function get_raw_generators()
    {
        $query = "SELECT isys_catg_smview_cable_generator__id as id,
                isys_catg_smview_cable_generator__title as title,
                isys_catg_smview_cable_generator__description as description,
                isys_catg_smview_cable_generator__mask as mask,
                isys_catg_smview_cable_generator__counter_min as counter_min,
                isys_catg_smview_cable_generator__counter_max as counter_max
                FROM isys_catg_smview_cable_generator;";
        return $this->retrieve($query);
    }
    
    /**
     * Delete an existing object.
     *
     * @param  int $id
     * @return boolean
     */
    public function delete_object($id)
    {
        $this->_remove_assigned_algorithms($id);
        return parent::delete_object($id);
    }
    
    /**
     * Remove assigned algorithms.
     *
     * @param int    $generator_id
     * @param string $json_elements
     */
    private function _remove_assigned_algorithms($generator_id)
    {
        $algorithm_dao = new \isys_cmdb_dao_smview_cable_generator_algorithm($this->m_db);
        $current_algorithms = $algorithm_dao->filter_objects(array('generator_id' => $generator_id));
        
        // remove old algorithms
        foreach ($current_algorithms as $current_algorithm) {
            $algorithm_dao->delete_object($current_algorithm['id']);
        }
    }
}
