<?php

/**
 * i-doit
 *
 * Patch View DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_patch_view extends isys_cmdb_dao_smview_cable_abstract_view
{
    
    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = array(
        'wire.isys_catg_smview_cable_wire_list__isys_obj__id' => 'cable_id',
        'wire.isys_catg_smview_cable_wire_list__id'           => 'wire_id',
        'pin_left.isys_catg_smview_cable_pin_list__id'        => 'pin_id_in',
        'sib_left.isys_catg_smview_cable_pin_list__patch_id'  => 'patch_pin_id_in',
        'obj_left.isys_obj__id'                               => 'obj_id_in',
        'obj_left.isys_obj__title'                            => 'obj_title_in',
        'pin_right.isys_catg_smview_cable_pin_list__id'       => 'pin_id_out',
        'sib_right.isys_catg_smview_cable_pin_list__patch_id' => 'patch_pin_id_out',
        'obj_right.isys_obj__id'                              => 'obj_id_out',
        'obj_right.isys_obj__title'                           => 'obj_title_out',
    );
    
    /**
     * Get base query.
     *
     * @return string
     */
    protected function _get_base_query()
    {
        $query = "SELECT ";
        foreach ($this->select_field_mapping as $db => $name) {
            $query .= " $db AS $name,";
        }
        $base_query = trim($query, ',');
        $base_query .= " FROM isys_catg_smview_cable_wire_list wire
            JOIN isys_catg_smview_cable_wire_pin wp_left ON wp_left.isys_catg_smview_cable_wire_pin__wire_id = wire.isys_catg_smview_cable_wire_list__id
            JOIN isys_catg_smview_cable_pin_list pin_left ON wp_left.isys_catg_smview_cable_wire_pin__connected_pin_id = pin_left.isys_catg_smview_cable_pin_list__id
            JOIN isys_catg_smview_cable_pin_list sib_left ON pin_left.isys_catg_smview_cable_pin_list__sibling_id = sib_left.isys_catg_smview_cable_pin_list__id
            JOIN isys_obj obj_left ON pin_left.isys_catg_smview_cable_pin_list__isys_obj__id = obj_left.isys_obj__id
            
            JOIN isys_catg_smview_cable_wire_pin wp_right ON wp_right.isys_catg_smview_cable_wire_pin__id = wp_left.isys_catg_smview_cable_wire_pin__sibling_id
            JOIN isys_catg_smview_cable_pin_list pin_right ON wp_right.isys_catg_smview_cable_wire_pin__connected_pin_id = pin_right.isys_catg_smview_cable_pin_list__id
            JOIN isys_catg_smview_cable_pin_list sib_right ON pin_right.isys_catg_smview_cable_pin_list__sibling_id = sib_right.isys_catg_smview_cable_pin_list__id
            JOIN isys_obj obj_right ON pin_right.isys_catg_smview_cable_pin_list__isys_obj__id = obj_right.isys_obj__id
            
            WHERE (sib_left.isys_catg_smview_cable_pin_list__patch_id IS NOT NULL OR sib_right.isys_catg_smview_cable_pin_list__patch_id IS NOT NULL) ";
        return $base_query;
    }
    
    /**
     * Get all objects.
     *
     * @param  string $group_by
     * @return array
     */
    public function get_objects($group_by = null)
    {
        return parent::get_objects('wire_id');
    }
    
    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @param  string $group_by
     * @return array
     */
    public function get_object($field, $value, $group_by = null)
    {
        return parent::get_object($field, $value, 'wire_id');
    }
    
    /**
     * Filter objects.
     *
     * @param  array $filter
     * @param  string $group_by
     * @return array
     */
    public function filter_objects($filter, $group_by = null)
    {
        return parent::filter_objects($filter, 'wire_id');
    }
}
