<?php

/**
 * i-doit
 *
 * Pin DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_pin extends isys_cmdb_dao_smview_cable_abstract
{
    
    const PIN_TYPE_INPUT = 'in';
    const PIN_TYPE_OUTPUT = 'out';
    const WIRE_PIN_CONNECTION = 'connected_wire_pin_id';
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_pin_list__id'             => 'id',
        'isys_catg_smview_cable_pin_list__isys_obj__id'   => 'obj_id',
        'isys_catg_smview_cable_pin_list__description'    => 'description',
        'isys_catg_smview_cable_pin_list__number'         => 'number',
        'isys_catg_smview_cable_pin_list__type'           => 'type',
        'isys_catg_smview_cable_pin_list__classification' => 'classification',
        'isys_catg_smview_cable_pin_list__sibling_id'     => 'internal_pin_id',
        'isys_catg_smview_cable_pin_list__patch_id'       => 'patch_pin_id',
        'isys_catg_smview_cable_pin_list__connector_id'   => 'connector_id',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_pin_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_pin_list__id';

    /**
     * Get data method.
     *
     * @param  int $obj_id
     * @return array
     */
    public function get_data($obj_id = null)
    {
        $query = "SELECT * FROM isys_catg_smview_cable_pin_list";
        if ($obj_id) {
            $query .= " WHERE isys_catg_smview_cable_pin_list__isys_obj__id=$obj_id";
        }
        $query .= ";";
        $res = $this->retrieve($query);
        return $this->_map_result_to_array($res);
    }
    
    /**
     * Get pin by ID.
     *
     * @param  int $p_pin_id
     * @return array
     */
    public function get_pin_by_id($p_pin_id)
    {
        return $this->get_object('id', $p_pin_id);
    }

    /**
     * Get listing of PINs by obj_id.
     *
     * @param  int $l_object_id
     * @param  string $filter
     * @return array
     * @throws isys_exception_database
     */
    public function get_listing_by_obj_id($l_object_id, $filter = null)
    {
        if ($filter !== null) {
            $filter = $filter != '__all__' ? $filter : null;
        }
        $pins = $this->_get_pins_by_obj_id($l_object_id, $filter);

        // get listing of sibling PINS
        $handledPins = [];
        $pin_connections = [];
        foreach ($pins as $p) {
            if (isset($handledPins[$p['id']])) {
                continue;
            }

            $sib = isset($pins[$p['internal_pin_id']]) ? $pins[$p['internal_pin_id']] : null;
            $pin_connections[] = array(
                'obj_in'  => $p['type'] == 'in' ? $p : $sib,
                'obj_out' => $p['type'] == 'out' ? $p : $sib,
            );
            $handledPins[$p['id']] = true;
            if ($sib) {
                $handledPins[$sib['id']] = true;
            }
        }

        return $pin_connections;
    }

    /**
     * Get scopes by object ID.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_scopes_by_obj_id($obj_id = null)
    {
        $query = "SELECT isys_catg_smview_cable_pin_list__classification FROM isys_catg_smview_cable_pin_list";
        if ($obj_id) {
            $query .= " WHERE isys_catg_smview_cable_pin_list__isys_obj__id=$obj_id";
        }
        $query .= ";";
        $res = $this->retrieve($query);
        $scopes = [];
        while ($row = $res->get_row()) {
            $scope = $row['isys_catg_smview_cable_pin_list__classification'];
            $scopes[$scope] = $scope;
        }
        return $scopes;
    }

    /**
     * Get PINs by object ID.
     *
     * @param  int $obj_id
     * @param  string|null $filter
     * @return array
     * @throws isys_exception_database
     */
    private function _get_pins_by_obj_id($obj_id, $filter = null)
    {
        $sql = "
SELECT
a.isys_catg_smview_cable_pin_list__id AS id,
a.isys_catg_smview_cable_pin_list__description AS description,
a.isys_catg_smview_cable_pin_list__classification AS classification,
a.isys_catg_smview_cable_pin_list__sibling_id AS internal_pin_id,
a.isys_catg_smview_cable_pin_list__patch_id AS patch_pin_id,
a.isys_catg_smview_cable_pin_list__connector_id AS connector_id,
a.isys_catg_smview_cable_pin_list__type AS type,
cable_obj.isys_obj__id as cable_obj_id,
cable_obj.isys_obj__title as cable_title,
wp.isys_catg_smview_cable_wire_pin__id as wire_pin_id,
wire.isys_catg_smview_cable_wire_list__id as wire_id,
wire.isys_catg_smview_cable_wire_list__description as wire_title,
conn.isys_catg_connector_list__title as connector_title,
IF(patch_obj.isys_obj__id IS NOT NULL, 'PATCH', NULL) as cable,
IFNULL(patch_obj.isys_obj__id , sib_obj.isys_obj__id) as connected_object_obj_id,
IFNULL(patch_obj.isys_obj__title, sib_obj.isys_obj__title) as connected_object_title,
IFNULL(patch_pin.isys_catg_smview_cable_pin_list__description, sib_pin.isys_catg_smview_cable_pin_list__description) as connected_object_pin
FROM 
(
	SELECT
	*
	FROM isys_catg_smview_cable_pin_list pin
	WHERE isys_catg_smview_cable_pin_list__isys_obj__id = $obj_id
) a
LEFT JOIN isys_catg_smview_cable_wire_pin wp ON a.isys_catg_smview_cable_pin_list__id = wp.isys_catg_smview_cable_wire_pin__connected_pin_id
LEFT JOIN isys_catg_smview_cable_wire_list wire ON wire.isys_catg_smview_cable_wire_list__id = wp.isys_catg_smview_cable_wire_pin__wire_id
LEFT JOIN isys_obj cable_obj ON wire.isys_catg_smview_cable_wire_list__isys_obj__id = cable_obj.isys_obj__id

LEFT JOIN isys_catg_smview_cable_pin_list patch_pin ON patch_pin.isys_catg_smview_cable_pin_list__id = a.isys_catg_smview_cable_pin_list__patch_id
LEFT JOIN isys_obj patch_obj ON patch_pin.isys_catg_smview_cable_pin_list__isys_obj__id = patch_obj.isys_obj__id

LEFT JOIN isys_catg_smview_cable_wire_pin sib_wp ON wp.isys_catg_smview_cable_wire_pin__sibling_id = sib_wp.isys_catg_smview_cable_wire_pin__id
LEFT JOIN isys_catg_smview_cable_pin_list sib_pin ON sib_pin.isys_catg_smview_cable_pin_list__id = sib_wp.isys_catg_smview_cable_wire_pin__connected_pin_id
LEFT JOIN isys_obj sib_obj ON sib_pin.isys_catg_smview_cable_pin_list__isys_obj__id = sib_obj.isys_obj__id

LEFT JOIN isys_catg_connector_list conn ON conn.isys_catg_connector_list__id = a.isys_catg_smview_cable_pin_list__connector_id
;
        ";

        // get assigned PINS and filter
        $obj_dao = isys_cmdb_dao_category_g_global::instance($this->get_database_component());
        $res = $obj_dao->retrieve($sql);
        $pins = [];
        while ($row = $res->get_row()) {
            $filterMatches = $this->_filter_matches($row['classification'], $filter);
            if ($filter !== null && ! $filterMatches) {
                continue;
            }
            $pins[$row['id']] = $row;
        }
        return $pins;
    }
    
    /**
     * Return if filter matches or not.
     *
     * @param  string $classification
     * @param  string $filter
     * @return boolean
     */
    private function _filter_matches($classification, $filter = null)
    {
        if (! isset($filter)) {
            return false;
        }
        
        if ($classification == $filter) {
            return true;
        }
        
        if (! $classification && $filter == '') {
            return true;
        }
        return false;
    }
    
    /**
     * Optional method to add additional informations to an object.
     *
     * @param  array $obj
     * @return array|null
     */
    protected function _get_additional_informations($obj)
    {
        $wire_pin_dao = isys_cmdb_dao_smview_cable_wire_pin::instance(isys_application::instance()->database);
        
        // add pin connections
        $obj_id = $obj['id'];
        $wire_pin = $wire_pin_dao->get_object('connected_pin_id', $obj_id);
        $connections[self::WIRE_PIN_CONNECTION] = null;
        if (! empty($wire_pin)) {
            $connections[self::WIRE_PIN_CONNECTION] = $wire_pin['id'];
        }
        return $connections;
    }

    /**
     * Create a new pin.
     *
     * @param  int    $obj_id
     * @param  string $description
     * @param  int    $number
     * @param  string $type
     * @param  int    $internalPinId
     * @param  int    $patchPinId
     * @param  int    $connectorId
     * @param  string $classification
     * @return int|false
     */
    public function create_pin($obj_id, $description, $number, $type, $internalPinId = null, $patchPinId = null, $connectorId = null, $classification = null)
    {
        $l_type = strtolower($type);
        if ($l_type != self::PIN_TYPE_OUTPUT) {
            // ensure that type can only be 'in' or 'out'
            $l_type = self::PIN_TYPE_INPUT;
        }
        
        $params = array(
            'obj_id'      => $obj_id,
            'description' => $description,
            'number'      => $number,
            'type'        => $l_type,
        );
        $optionalParams = array(
            'classification'  => $classification,
            'internal_pin_id' => $internalPinId,
            'patch_pin_id'    => $patchPinId,
            'connector_id'    => $connectorId,
        );
        $id = $this->create_object($params, $optionalParams);
        
        if ($internalPinId) {
            // update connected pin to link to this one
            $this->_sanitize_connected_pin($internalPinId, 'internal_pin_id', $id);
        }
        if ($patchPinId) {
            // update connected pin to link to this one
            $this->_sanitize_connected_pin($patchPinId, 'patch_pin_id', $id);
        }
        return $id;
    }

    /**
     * Update an existing pin.
     *
     * @param  int    $pin_id
     * @param  string $description
     * @param  int    $number
     * @param  string $type
     * @param  int    $internalPinId
     * @param  int    $patchPinId
     * @param  int    $connectorId
     * @param  string $classification
     * @return boolean
     */
    public function update_pin($pin_id, $description, $number, $type, $internalPinId = null, $patchPinId = null, $connectorId = null, $classification = null)
    {
        $l_type = strtolower($type);
        if ($l_type != self::PIN_TYPE_OUTPUT) {
            // ensure that type can only be 'in' or 'out'
            $l_type = self::PIN_TYPE_INPUT;
        }
        
        $current_pin = $this->get_object('id', $pin_id);
        if (! empty($current_pin['internal_pin_id'])) {
            // reset link from current connected pin
            $this->_sanitize_connected_pin($current_pin['internal_pin_id'], 'internal_pin_id', null);
        }
        if (! empty($current_pin['patch_pin_id'])) {
            // reset link from current connected pin
            $this->_sanitize_connected_pin($current_pin['patch_pin_id'], 'patch_pin_id', null);
        }
        
        $params = array(
            'description' => $description,
            'number'      => $number,
            'type'        => $l_type,
        );
        $optionalParams = array(
            'classification'  => $classification,
            'internal_pin_id' => $internalPinId,
            'patch_pin_id'    => $patchPinId,
            'connector_id'    => $connectorId,
        );
        $success = $this->update_object($pin_id, $params, $optionalParams);
        
        if ($internalPinId) {
            // update connected pin to link to this one
            $this->_sanitize_connected_pin($internalPinId, 'internal_pin_id', $pin_id);
        }
        if ($patchPinId) {
            // update connected pin to link to this one
            $this->_sanitize_connected_pin($patchPinId, 'patch_pin_id', $pin_id);
        }
        
        return $success;
    }

    /**
     * Patch siblings of two PINs.
     *
     * @param  int     $pin_id1
     * @param  int     $pin_id2
     * @param  boolean $delete_patch
     * @return void
     */
    public function patch_pins($pin_id1, $pin_id2, $delete_patch = false)
    {
        // get pin objects
        $pin1 = $this->get_object('id', $pin_id1);
        $pin2 = $this->get_object('id', $pin_id2);

        // get sibling
        $sibling_id1 = $pin1['internal_pin_id'];
        $sibling_id2 = $pin2['internal_pin_id'];
        $sibling = $this->get_object('id', $sibling_id1);

        if ($delete_patch) {
            $sibling_id2 = null;
        }

        // add patch
        $this->update_pin(
            $sibling_id1,
            $sibling['description'],
            $sibling['number'],
            $sibling['type'],
            $sibling['internal_pin_id'],
            $sibling_id2,
            $sibling['connector_id'],
            $sibling['classification']
        );
    }
    
    /**
     * Sanitize connected PIN.
     *
     * @param  int    $connected_pin_id
     * @param  string $field_name
     * @param  int    $pin_id
     * @return void
     */
    private function _sanitize_connected_pin($connected_pin_id, $field_name, $pin_id = null)
    {
        $params = array(
            $field_name => $pin_id,
        );
        $this->update_object($connected_pin_id, [], $params);
    }
}
