<?php
/**
 * i-doit - Documentation and CMDB solution for IT environments
 *
 * This file is part of the i-doit framework. Modify at your own risk.
 *
 * Please visit http://www.i-doit.com/license for a full copyright and license information.
 *
 * @version   1.8.3
 * @package   i-doit
 * @author    synetics GmbH
 * @copyright synetics GmbH
 * @url       http://www.i-doit.com
 * @license   http://www.i-doit.com/license
 */

/**
 * i-doit
 *
 * Template Module Dao.
 *
 * @package    i-doit
 * @subpackage Modules
 * @author     Dennis Stücken <dstuecken@i-doit.org>
 * @version    0.9
 * @copyright  synetics GmbH
 * @license    http://www.i-doit.com/license
 */

class isys_cmdb_dao_smview_cable_properties extends isys_module_dao
{
    protected static $m_instance = null;

    protected $m_Condition = [];

    protected $m_Column = [];

    public function __construct(isys_component_database $p_db)
    {
        $properties = smview_cmdb_dao_properties::instance(isys_application::instance()->database)->getProperties(false);
        //change the property-const to id
        $tproperties = [];
        foreach ($properties as $property) {
            $tproperties[$property['const'] . '-' . $property['key']] = $property;
        }

        $this->m_properties = $tproperties;
    }

    /**
     * @param  $p_params
     * @return array|string
     */
    public function query($p_params)
    {
        $lang = isys_application::instance()->container->get('language');
        $blacklist = [];
        $properties = array();
        $l_res = smview_cmdb_dao_category_property::instance(isys_application::instance()->database)
            ->retrieve_properties_shd(null, $p_params["p_catg_id"], null, null, "", $p_dynamic_properties = true, null);

        while ($l_data = $l_res->get_row()) {
            //Check Blacklist
            if (array_key_exists($l_data["const"] . "-" . $l_data["key"], $blacklist) == true) {
                continue;
            }

            //Check Horizontale Linie Key start with hr_c_
            if (strpos($l_data["key"], "hr_c_") !== false) {
                continue;
            }

            //Translate
            $l_data['title'] = $lang->get($l_data['title']);
            if ($l_data["catg_title"] != null) {
                $l_data['cat_title'] = $lang->get($l_data["catg_title"]);
            } elseif ($l_data["cats_title"] != null) {
                $l_data['cat_title'] = $lang->get($l_data["cats_title"]);
            } elseif ($l_data["catg_custom_title"] != null) {
                $l_data['cat_title'] = $lang->get($l_data["catg_custom_title"]);
            } else {
                $l_data['cat_title'] = "";
            }

            unset($l_data["catg_title"]);
            unset($l_data["cats_title"]);
            unset($l_data["catg_custom_title"]);

            //set C__CATG__GLOBAL-type
            if ($l_data["const"] . "-" . $l_data["key"] == "C__CATG__GLOBAL-type") {
                $dialog = [];

                $_dao = new isys_cmdb_dao(isys_application::instance()->database, null);

                // Data retrieval:
                $l_data_object_type = $_dao->get_object_types_by_properties(
                    null,
                    'title',
                    null,
                    null,
                    null
                );

                $l_return = [];


                // Data formatting:
                while ($l_row_object_type = $l_data_object_type->get_row()) {
                    $dialog[] = ["id" => $l_row_object_type["isys_obj_type__id"],"title" => $lang->get($l_row_object_type["isys_obj_type__title"])
                        ,"const" => $l_row_object_type["isys_obj_type__id"]];
                }

                usort(
                    $dialog,
                    function ($a, $b) {
                        return strcasecmp($a['title'], $b['title']);
                    }
                );
                $dialog[] = ["id" => "-1","title" => $lang->get("LC__MODULE__SMVIEW__NO_VALUE") ,"const" => ""];
                $dialog[] = ["id" => "-99","title" => "-" ,"const" => ""];
                $l_data["dialog"] = $dialog;
            }

            //todo for all SHD Properties
            $pos = strpos($l_data["const"], "SMVIEW_CABLE");
            if ($pos !== false) {
                $bool_rights = decbin($l_data['provides']);
                for ($i= strlen($bool_rights) - 1; $i >= 0; $i--) {
                    //shown in list is false
                    if ((2 ** $i) == C__PROPERTY__PROVIDES__LIST) {
                        if ($bool_rights[$i - 1] == false) {
                            continue 2;
                        }
                    }
                }
            }
            $properties[] = $l_data;
        }

        foreach ($properties as $f_key => $f_item) {
            if ($f_item["const"] == "C__CATG__SMVIEW_CABLE") {
                $properties[$f_key]["special_column"] = true;
            }

            if ($f_item["const"] == "C__CATG__SMVIEW_CABLE_WIRE") {
                $properties[$f_key]["special_column"] = true;
                $properties[$f_key]["search_disable"] = true;
                $properties[$f_key]["sort_disable"] = true;
            }
        }
        return $properties;
    }

    /**
     * Get data method. Unused.
     *
     * @return null
     */
    public function get_data()
    {
        return null;
    }
}
