<?php

/**
 * i-doit
 *
 * Route DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_route extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_route_list__id'                       => 'id',
        'isys_catg_smview_cable_route_list__isys_obj__id'             => 'obj_id',
        'isys_catg_smview_cable_route_list__description'              => 'description',
        'isys_catg_smview_cable_route_list__connection_count'         => 'connection_count',
        'isys_catg_smview_cable_route_list__current_connection_count' => 'current_connection_count',
        'isys_catg_smview_cable_route_list__permission'               => 'permission',
        'isys_catg_smview_cable_route_list__location'                 => 'location',
        'isys_catg_smview_cable_route_list__room'                     => 'room',
        'isys_catg_smview_cable_route_list__device_type'              => 'device_type',
        'isys_catg_smview_cable_route_list__device_number'            => 'device_number',
        'isys_catg_smview_cable_route_list__phone_number'             => 'phone_number',
        'isys_catg_smview_cable_route_list__route_usage_id'           => 'route_usage_id',
        'isys_catg_smview_cable_route_list__phone_subscriber'         => 'phone_subscriber',
        'isys_catg_smview_cable_route_list__hint'                     => 'hint',
        'isys_catg_smview_cable_route_list__start_obj_id'             => 'start_obj_id',
        'isys_catg_smview_cable_route_list__end_obj_id'               => 'end_obj_id',
        'isys_catg_smview_cable_route_list__length'                   => 'length',
        'isys_catg_smview_cable_route_list__measured_length'          => 'measured_length',
        'isys_catg_smview_cable_route_list__measure_date'             => 'measure_date',
        'isys_catg_smview_cable_route_list__tka_participant'          => 'tka_participant',
        'isys_catg_smview_cable_route_list__is_external'              => 'is_external',
        'isys_catg_smview_cable_route_list__associated_route'         => 'associated_route',
        'isys_catg_smview_cable_route_list__damping_left1'            => 'damping_left1',
        'isys_catg_smview_cable_route_list__damping_left2'            => 'damping_left2',
        'isys_catg_smview_cable_route_list__damping_right1'           => 'damping_right1',
        'isys_catg_smview_cable_route_list__damping_right2'           => 'damping_right2',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_route_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_route_list__id';
    
    /**
     * Get route length.
     *
     * @param  int $route_id
     * @return int
     */
    public function get_route_length($route_id)
    {
        if (empty($route_id)) {
            return 0;
        }
        
        // get cable lengths
        $query1 = "SELECT
            e.isys_catg_smview_cable_route_element__route_id,
            SUM(c.isys_catg_smview_cable_list__length) AS sum
          FROM isys_catg_smview_cable_route_element e
          JOIN isys_catg_smview_cable_wire_list w ON w.isys_catg_smview_cable_wire_list__id = e.isys_catg_smview_cable_route_element__wire_id
          JOIN isys_catg_smview_cable_list c ON c.isys_catg_smview_cable_list__isys_obj__id = w.isys_catg_smview_cable_wire_list__isys_obj__id
          WHERE e.isys_catg_smview_cable_route_element__route_id = $route_id
          GROUP BY e.isys_catg_smview_cable_route_element__route_id;";
        
        // get section lengths
        $query2 = "SELECT
            e.isys_catg_smview_cable_route_element__route_id,
            SUM(s.isys_catg_smview_cable_section_list__length) AS sum
          FROM isys_catg_smview_cable_route_element e
          JOIN isys_catg_smview_cable_section_list s ON s.isys_catg_smview_cable_section_list__id = e.isys_catg_smview_cable_route_element__section_id
          WHERE e.isys_catg_smview_cable_route_element__route_id = $route_id
          GROUP BY e.isys_catg_smview_cable_route_element__route_id;";
        $res1 = $this->retrieve($query1);
        $res2 = $this->retrieve($query2);
        $sum = 0;
        if ($res1->count() > 0) {
            $row1 = $res1->get_row();
            $sum += $row1['sum'];
        }
        if ($res2->count() > 0) {
            $row2 = $res2->get_row();
            $sum += $row2['sum'];
        }
        return $sum;
    }
}
