<?php

/**
 * i-doit
 *
 * Route Element DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_route_element extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_route_element__id'         => 'id',
        'isys_catg_smview_cable_route_element__route_id'   => 'route_id',
        'isys_catg_smview_cable_route_element__wire_id'    => 'wire_id',
        'isys_catg_smview_cable_route_element__section_id' => 'section_id',
        'isys_catg_smview_cable_route_element__index'      => 'index',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_route_element';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_route_element__id';
    
    /**
     * Create a new object.
     *
     * @param  array $params
     * @param  array $optionalParams
     * @return int|false
     */
    public function create_object($params, $optionalParams = [])
    {
        $element_id = parent::create_object($params, $optionalParams);
        $route_id = $this->get_route_id_by_element($element_id);
        $this->update_route_length($route_id);
        return $element_id;
    }
    
    /**
     * Update an existing object.
     *
     * @param  int   $id
     * @param  array $params
     * @param  array $optionalParams
     * @return boolean
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $return = parent::update_object($id, $params, $optionalParams);
        $route_id = $this->get_route_id_by_element($id);
        $this->update_route_length($route_id);
        return $return;
    }
    
    /**
     * Delete an existing object.
     *
     * @param  int $id
     * @return boolean
     */
    public function delete_object($id)
    {
        $route_id = $this->get_route_id_by_element($id);
        $return = parent::delete_object($id);
        $this->update_route_length($route_id);
        return $return;
    }

    /**
     * Get route ID by route element.
     *
     * @param  int $id
     * @return int
     */
    public function get_route_id_by_element($id)
    {
        // get route ID by element
        $element = $this->get_object('id', $id);
        return $element['route_id'];
    }
    
    /**
     * Update route length if necessary.
     *
     * @param int $route_id
     */
    public function update_route_length($route_id)
    {
        // update route length if necessary
        $route_dao = new isys_cmdb_dao_smview_cable_route($this->m_db);
        $route = $route_dao->get_object('id', $route_id);
        $length = $route_dao->get_route_length($route_id);
        
        if ($route['length'] != $length) {
            $route_dao->update_object($route_id, [], array('length' => $length));
        }
    }
}
