<?php

/**
 * i-doit
 *
 * Route Usage DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_route_usage extends isys_cmdb_dao_smview_cable_abstract
{

    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_route_usage__id'            => 'id',
        'isys_catg_smview_cable_route_usage__name'          => 'name',
        'isys_catg_smview_cable_route_usage__description'   => 'description',
        'isys_catg_smview_cable_route_usage__can_be_rented' => 'can_be_rented',
        'isys_catg_smview_cable_route_usage__technology_id' => 'technology_id',
    );

    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_route_usage';

    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_route_usage__id';

    /**
     * Optional method to add additional information to an object.
     *
     * @param  array $obj
     * @return array|null
     * @throws isys_exception_database
     */
    protected function _get_additional_informations($obj)
    {
        $dao = isys_cmdb_dao_smview_cable_technology::instance(isys_application::instance()->database);
        $tech = $dao->get_object('id', $obj['technology_id']);
        return array(
            'technology' => $tech['description'],
        );
    }

    /**
     * Get data method.
     *
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_raw_route_usages()
    {
        $query = "SELECT isys_catg_smview_cable_route_usage__id as id,
                isys_catg_smview_cable_route_usage__name as name,
                isys_catg_smview_cable_route_usage__description as description
                FROM isys_catg_smview_cable_route_usage;";
        return $this->retrieve($query);
    }
}
