<?php

/**
 * i-doit
 *
 * Section DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_section extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_section_list__id'                    => 'id',
        'isys_catg_smview_cable_section_list__isys_obj__id'          => 'obj_id',
        'isys_catg_smview_cable_section_list__description'           => 'description',
        'isys_catg_smview_cable_section_list__user_state_id'         => 'user_state_id',
        'isys_catg_smview_cable_section_list__channel_count'         => 'channel_count',
        'isys_catg_smview_cable_section_list__current_channel_count' => 'current_channel_count',
        'isys_catg_smview_cable_section_list__start_obj_id'          => 'start_obj_id',
        'isys_catg_smview_cable_section_list__end_obj_id'            => 'end_obj_id',
        'isys_catg_smview_cable_section_list__length'                => 'length',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_section_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_section_list__id';

    /**
     * Get section length.
     *
     * @param  int $section_id
     * @return int
     * @throws isys_exception_database
     */
    public function get_section_length($section_id)
    {
        if (empty($section_id)) {
            return 0;
        }
        
        $query = "SELECT
            e.isys_catg_smview_cable_section_element__section_id,
            SUM(c.isys_catg_smview_cable_list__length) AS sum
          FROM isys_catg_smview_cable_section_element e
          JOIN isys_catg_smview_cable_wire_list w ON w.isys_catg_smview_cable_wire_list__id = e.isys_catg_smview_cable_section_element__wire_id
          JOIN isys_catg_smview_cable_list c ON c.isys_catg_smview_cable_list__isys_obj__id = w.isys_catg_smview_cable_wire_list__isys_obj__id
          WHERE e.isys_catg_smview_cable_section_element__section_id = $section_id
          GROUP BY e.isys_catg_smview_cable_section_element__section_id;";
        $res = $this->retrieve($query);
        if ($res->count() > 0) {
            $row = $res->get_row();
            return $row['sum'];
        } else {
            return 0;
        }
    }

    /**
     * Create a new element.
     *
     * @param  int      $obj_id
     * @param  string   $description
     * @param  int      $user_state_id
     * @param  int      $channel_count
     * @param  int      $current_channel_count
     * @param  int|null $start_obj_id
     * @param  int|null $end_obj_id
     * @return int
     * @throws isys_exception_dao
     */
    public function create_section($obj_id, $description, $user_state_id, $channel_count, $current_channel_count, $start_obj_id = null, $end_obj_id = null)
    {
        $params = array(
            'obj_id'                => $obj_id,
            'description'           => $description,
            'user_state_id'         => $user_state_id,
            'channel_count'         => $channel_count,
            'current_channel_count' => $current_channel_count,
            'start_obj_id'          => $start_obj_id,
            'end_obj_id'            => $end_obj_id,
        );
        return $this->create_object($params);
    }

    /**
     * Update an existing element.
     *
     * @param  int    $element_id
     * @param  string $description
     * @param  int    $user_state_id
     * @param  int    $channel_count
     * @param  int    $current_channel_count
     * @return boolean
     * @throws isys_exception_dao
     */
    public function update_section($element_id, $description, $user_state_id, $channel_count, $current_channel_count)
    {
        $params = array(
            'description'           => $description,
            'user_state_id'         => $user_state_id,
            'channel_count'         => $channel_count,
            'current_channel_count' => $current_channel_count,
        );
        return $this->update_object($element_id, $params);
    }

    /**
     * Delete an existing object.
     *
     * @param  int $id
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function delete_object($id)
    {
        // remove assigned section elements
        $section_element_dao = new isys_cmdb_dao_smview_cable_section_element($this->m_db);
        $current_elements = $section_element_dao->filter_objects(array('section_id' => $id));

        // remove old section elements
        foreach ($current_elements as $current_element) {
            $section_element_dao->delete_object($current_element['id']);
        }
        return parent::delete_object($id);
    }
}
