<?php

/**
 * i-doit
 *
 * Section Element DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_section_element extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_section_element__id'          => 'id',
        'isys_catg_smview_cable_section_element__section_id'  => 'section_id',
        'isys_catg_smview_cable_section_element__wire_id'     => 'wire_id',
        'isys_catg_smview_cable_section_element__index'       => 'index',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_section_element';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_section_element__id';

    /**
     * Create a new object.
     *
     * @param  array $params
     * @param  array $optionalParams
     * @return int|false
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function create_object($params, $optionalParams = [])
    {
        $element_id = parent::create_object($params, $optionalParams);
        $this->update_section_length($element_id);
        return $element_id;
    }

    /**
     * Update an existing object.
     *
     * @param  int   $id
     * @param  array $params
     * @param  array $optionalParams
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        $return = parent::update_object($id, $params, $optionalParams);
        $this->update_section_length($id);
        return $return;
    }

    /**
     * Delete an existing object.
     *
     * @param  int $id
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function delete_object($id)
    {
        // get section ID by element
        $element = $this->get_object('id', $id);
        $section_id = $element['section_id'];

        $return = parent::delete_object($id);
        $this->update_section_length($id, $section_id);
        return $return;
    }

    /**
     * Update section length if necessary.
     *
     * @param  int      $element_id
     * @param  int|null $section_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update_section_length($element_id, $section_id = null)
    {
        if (empty($section_id)) {
            // get section ID by element
            $element = $this->get_object('id', $element_id);
            $section_id = $element['section_id'];
        }
        
        // update section length if necessary
        $section_dao = new isys_cmdb_dao_smview_cable_section($this->m_db);
        $section = $section_dao->get_object('id', $section_id);
        $length = $section_dao->get_section_length($section_id);
        
        if ($section['length'] != $length) {
            $section_dao->update_object($section_id, [], array('length' => $length));
        }
    }
}
