<?php

/**
 * i-doit
 *
 * Virtual Route Element DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_virtual_route_element extends isys_cmdb_dao_smview_cable_route_element
{

    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_vir_route_element__id'           => 'id',
        'isys_catg_smview_cable_vir_route_element__vir_route_id' => 'route_id',
        'isys_catg_smview_cable_vir_route_element__wire_id'      => 'wire_id',
        'isys_catg_smview_cable_vir_route_element__section_id'   => 'section_id',
        'isys_catg_smview_cable_vir_route_element__index'        => 'index',
    );

    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_vir_route_element';

    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_vir_route_element__id';

    /**
     * Update route length if necessary.
     *
     * @param  int $route_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update_route_length($route_id)
    {
        // update route length if necessary
        $route_dao = new isys_cmdb_dao_smview_cable_virtual_route($this->m_db);
        $route = $route_dao->get_object('id', $route_id);
        $length = $route_dao->get_route_length($route_id);

        if ($route['length'] != $length) {
            $route_dao->update_object($route_id, [], array('length' => $length));
        }
    }
}
