<?php

/**
 * i-doit
 *
 * Wire DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_wire extends isys_cmdb_dao_smview_cable_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_wire_list__id'                 => 'id',
        'isys_catg_smview_cable_wire_list__isys_obj__id'       => 'obj_id',
        'isys_catg_smview_cable_wire_list__description'        => 'description',
        'isys_catg_smview_cable_wire_list__color'              => 'color',
        'isys_catg_smview_cable_wire_list__scope'              => 'scope',
        'isys_catg_smview_cable_wire_list__scope_index'        => 'scope_index',
        'isys_catg_smview_cable_wire_list__reserved'           => 'reserved',
        'isys_catg_smview_cable_wire_list__user_state_id'      => 'user_state_id',
        'isys_catg_smview_cable_wire_list__user_state'         => 'user_state',
        'isys_catg_smview_cable_wire_list__connection_type_id' => 'connection_type_id',
        'isys_catg_smview_cable_wire_list__damping'            => 'damping',
        'isys_catg_smview_cable_wire_list__notice'             => 'notice',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_wire_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_wire_list__id';

    /**
     * Get data method.
     *
     * @param  int $obj_id
     * @return array
     */
    public function get_data($obj_id = null)
    {
        $query = "SELECT * FROM isys_catg_smview_cable_wire_list";
        if ($obj_id) {
            $query .= " WHERE isys_catg_smview_cable_wire_list__isys_obj__id=$obj_id";
        }
        $query .= ";";
        $res = $this->retrieve($query);
        return $this->_map_result_to_array($res);
    }
    
    /**
     * Get wire by ID.
     *
     * @param  int $wire_id
     * @return isys_component_dao_result
     */
    public function get_wire_by_id($wire_id)
    {
        return $this->get_object('id', $wire_id);
    }

    /**
     * Create a new wire.
     *
     * @param  int    $obj_id
     * @param  string $description
     * @param  string $color
     * @param  string $scope
     * @param  string $reserved
     * @param  int    $scope_index
     * @param  int    $user_state_id
     * @param  string $user_state
     * @param  int    $connection_type_id
     * @param  string $damping
     * @param  string $notice
     * @return int|false
     */
    public function create_wire($obj_id, $description, $color = null, $scope = null, $reserved = null, $scope_index = null, $user_state_id = null, $user_state = null, $connection_type_id = null, $damping = null, $notice = null)
    {
        $params = array(
            'obj_id'      => $obj_id,
            'description' => $description,
        );
        $optionalParams = array(
            'color'              => $color,
            'scope'              => $scope,
            'reserved'           => $reserved,
            'scope_index'        => $scope_index,
            'user_state_id'      => $user_state_id,
            'user_state'         => $user_state,
            'connection_type_id' => $connection_type_id,
            'damping'            => $damping,
            'notice'             => $notice,
        );
        $wire_id = $this->create_object($params, $optionalParams);
        
        // create wire pins
        $pin_dao = isys_cmdb_dao_smview_cable_wire_pin::instance(isys_application::instance()->database);
        $left_pin_id = $pin_dao->create_wire_pin($wire_id, $description);
        $right_pin_id = $pin_dao->create_wire_pin($wire_id, $description, null, $left_pin_id);
        
        // assign right PIN to left PIN
        $pin_dao->update_wire_pin($left_pin_id, $description, null, $right_pin_id);
        return $wire_id;
    }

    /**
     * Update an existing wire.
     *
     * @param  int    $wire_id
     * @param  string $description
     * @param  string $color
     * @param  string $scope
     * @param  string $reserved
     * @param  int    $scope_index
     * @param  int    $user_state_id
     * @param  string $user_state
     * @param  int    $connection_type_id
     * @param  string $damping
     * @param  string $notice
     * @return boolean
     */
    public function update_wire($wire_id, $description, $color = null, $scope = null, $reserved = null, $scope_index = null, $user_state_id = null, $user_state = null, $connection_type_id = null, $damping = null, $notice = null)
    {
        $params = array(
            'description' => $description,
        );
        $optionalParams = array(
            'color'              => $color,
            'scope'              => $scope,
            'reserved'           => $reserved,
            'scope_index'        => $scope_index,
            'user_state_id'      => $user_state_id,
            'user_state'         => $user_state,
            'connection_type_id' => $connection_type_id,
            'damping'            => $damping,
            'notice'             => $notice,
        );
        return $this->update_object($wire_id, $params, $optionalParams);
    }
}
