<?php

/**
 * i-doit
 *
 * Wire Pin Status DAO for SM-VIEW Cable.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_smview_cable_wire_pin_status extends isys_cmdb_dao_smview_cable_abstract
{

    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_cable_wire_pin_status__id'                       => 'id',
        'isys_catg_smview_cable_wire_pin_status__const'                    => 'const',
        'isys_catg_smview_cable_wire_pin_status__description'              => 'description',
        'isys_catg_smview_cable_wire_pin_status__isys_obj__id'             => 'obj_id', // this is the image object
        'isys_catg_smview_cable_wire_pin_status__color'                    => 'color',
        'isys_catg_smview_cable_wire_pin_status__upper_transition_damping' => 'upper_transition_damping',
        'isys_catg_smview_cable_wire_pin_status__wire_not_applied'         => 'wire_not_applied',
    );

    /**
     * Select field mapping.
     *
     * Used to create select-fields like "$key AS $value,".
     *
     * @var array
     */
    protected $select_field_mapping = array(
        't.isys_catg_smview_cable_wire_pin_status__id'                       => 'id',
        't.isys_catg_smview_cable_wire_pin_status__const'                    => 'const',
        't.isys_catg_smview_cable_wire_pin_status__description'              => 'description',
        't.isys_catg_smview_cable_wire_pin_status__isys_obj__id'             => 'obj_id',
        't.isys_catg_smview_cable_wire_pin_status__color'                    => 'color',
        't.isys_catg_smview_cable_wire_pin_status__upper_transition_damping' => 'upper_transition_damping',
        't.isys_catg_smview_cable_wire_pin_status__wire_not_applied'         => 'wire_not_applied',
        'v.isys_file_version__revision'                                      => 'revision',
        'physical.isys_file_physical__filename'                              => 'filename',
        'physical.isys_file_physical__filename_original'                     => 'filename_original',
    );

    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_cable_wire_pin_status';

    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_cable_wire_pin_status__id';

    /**
     * Get base query.
     *
     * @return string
     */
    protected function _get_base_query()
    {
        $query = "SELECT ";
        foreach ($this->select_field_mapping as $db => $name) {
            $query .= " $db AS $name,";
        }
        $base_query = trim($query, ',');
        $base_query .= " FROM isys_catg_smview_cable_wire_pin_status as t
                LEFT JOIN isys_file_version v ON v.isys_file_version__isys_obj__id = t.isys_catg_smview_cable_wire_pin_status__isys_obj__id
                LEFT JOIN isys_file_physical physical ON physical.isys_file_physical__id = isys_file_version__isys_file_physical__id
                WHERE 1=1 ";
        return $base_query;
    }

    /**
     * Get all objects.
     *
     * @return array
     * @throws isys_exception_database
     */
    public function get_objects()
    {
        $query = $this->_get_base_query();
        $query .= ' ORDER BY isys_file_version__revision DESC';
        $res = $this->retrieve($query);

        $objects = array();
        while ($row = $res->get_row()) {
            $id = $row['id'];
            if (isset($objects[$id])) {
                continue;
            }
            $objects[$id] = $row;
        }
        return array_values($objects);
    }

    /**
     * Get an object.
     *
     * @param  string $field
     * @param  string $value
     * @return array
     * @throws isys_exception_database
     */
    public function get_object($field, $value)
    {
        $db_field = array_search($field, $this->select_field_mapping);
        $query = $this->_get_base_query();
        $query .= " AND $db_field = '$value' ";
        $query .= ' ORDER BY isys_file_version__revision DESC';

        $res = $this->retrieve($query);
        return $res->count() > 0 ? $res->get_row() : array();
    }

    /**
     * Get file URL by obj_id of the file.
     *
     * @param  int $file_obj_id
     * @return string|null
     */
    public static function get_file_url($file_obj_id)
    {
        if (empty($file_obj_id)) {
            return null;
        }

        // get latest version of file
        $file_dao = new isys_cmdb_dao_category_s_file(isys_application::instance()->database);
        $files = $file_dao->get_file_by_obj_id($file_obj_id);
        $file = $files->get_row();

        // use CMDB module to display images directly
        return isys_helper_link::create_url([
            C__GET__MODULE_ID    => defined_or_default('C__MODULE__CMDB'),
            C__GET__FILE_MANAGER => 'get',
            C__GET__FILE__ID     => $file['isys_file_physical__id'],
        ]);
    }
}
