<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable ETSI Bay.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_etsi_bay extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_etsi_bay.tpl');
    }

    /**
     * Processes the UI for the category check_mk
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();

        $dao = new isys_cmdb_dao_smview_cable_etsi_device_color($this->m_database_component);
        $colors = $dao->get_data();
        $colorLayouts = [];
        foreach ($colors as $c) {
            $colorLayouts[] = [
                'id'                 => $c['id'],
                'title'              => $c['description'],
                'layout'             => $c['layout'],
                'color_left_top'     => $c['color_left_top'],
                'color_left_bottom'  => $c['color_left_bottom'],
                'color_right_top'    => $c['color_right_top'],
                'color_right_bottom' => $c['color_right_bottom'],
            ];
        }

        if (! $p_cat instanceof isys_cmdb_dao_category_g_smview_cable_etsi_bay) {
            return;
        }

        $scopeData = $p_cat->get_scope_data($this->m_object_id);
        $bays = $p_cat->get_bays($this->m_object_id, $this->m_category_data_id, $scopeData);

        $bay = null;
        $hint = null;
        if ($this->m_category_data_id && $this->m_category_data_id > 0) {
            $bay = $p_cat->get_bay_by_id($this->m_category_data_id, $scopeData);

            // check if bay already exists
            $bayAlreadyExists = false;
            foreach ($bays as $b) {
                if ($b['id'] == $bay['id']) {
                    $bayAlreadyExists = true;
                }
            }

            if (! $bayAlreadyExists) {
                $bay = null;
                $lang = isys_application::instance()->container->get('language');
                $hint = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY_NOT_AVAILABLE');
            }
        }

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign('hint', $hint)
            ->assign('C__CATG__SMVIEW_CABLE_ETSI_BAY_ITEMS', json_encode([
                'bay_id' => $bay['id'],
                'assigned_bay_items' => $bay['items'],
                'he' => $bay['he'],
            ]))
            ->assign('js_config', json_encode([
                'bay'           => $bay,
                'bays'          => $bays,
                'color_layouts' => $colorLayouts,
                'language'      => $this->_get_js_translations(),
            ]));
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY',
            'LC__CMDB__CATG__SMVIEW_CABLE__HE',
            'LC__CMDB__CATG__SMVIEW_CABLE__POSITION',
            'LC__CMDB__CATG__SMVIEW_CABLE__SHORT_NAME',
            'LC__CMDB__CATG__SMVIEW_CABLE__PROJECT_IDENTIFIER',
            'LC__CMDB__CATG__SMVIEW_CABLE__COMMENT',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DEVICE_COLOR',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_AVAILABLE_BAY_ITEMS',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_AUTO_ASSIGNMENT',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_NOT_EMPTY',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_NOT_BETWEEN',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_ALREADY_ASSIGNED',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ERROR_HE_IN_USE',
            'LC__SHD__TELECOMMUNICATION_CABLE',
            'LC__CMDB__CATG__SMVIEW_CABLE__WIRE_BUNDLES',
            'LC__CMDB__CATG__SMVIEW_CABLE__CONNECTED_WITH',
            'LC__CMDB__CATG__SMVIEW_CABLE__FIBRE_WIRE',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY_PLANNED_ITEM',
            'LC__CMDB__CATG__SMVIEW_CABLE__FIBRE_WIRE',
            'LC__CMDB__CATG__SMVIEW_CABLE__SUGGESTED_COLORS',
            'LC__CMDB__CATG__SMVIEW_CABLE__SUGGESTED_COLOR_NOT_MATCH',
            'LC__CMDB__CATG__SMVIEW_CABLE__NO_COLOR_SUGGESTED',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_catdata = $p_cat->get_general_data();
        $description = $l_catdata['isys_catg_smview_cable_etsi_bay_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAY_TITLE') . ": $description";
    }
}
