<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable ETSI Bay Export.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_etsi_bay_export extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_etsi_bay_export.tpl');
    }

    /**
     * Processes the UI for the category check_mk
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();

        $l_catdata = $p_cat->get_general_data();
        $assigned_bays = [];
        if ($l_catdata) {
            $dao = new isys_cmdb_dao_smview_cable_etsi_bay_item_export(isys_application::instance()->database);
            $bay_dao = new isys_cmdb_dao_category_g_smview_cable_etsi_bay(isys_application::instance()->database);
            $bay_items = $dao->filter_objects(['list_id' => $l_catdata['isys_catg_smview_cable_etsi_bay_export_list__id']]);

            foreach ($bay_items as $r) {
                $bay = $bay_dao->get_data($r['bay_item_id'])->get_row();
                $assigned_bays[] = [
                    'id'          => $r['bay_item_id'],
                    'position'    => $r['position'],
                    'name'        => $bay['isys_catg_smview_cable_etsi_bay_list__name'],
                    'description' => $bay['isys_catg_smview_cable_etsi_bay_list__description'],
                    'item_count'  => $bay['isys_catg_smview_cable_etsi_bay_list__he'],
                    'obj_title'   => $bay['isys_obj__title'],
                ];
            }
            $available_bay_count = $bay_dao->get_data(null, $this->m_object_id)->count();
            if ($available_bay_count > count($assigned_bays)) {
                $this->get_template_component()->assign('new_bays_available', true);
            }

            if ($l_catdata['isys_catg_smview_cable_etsi_bay_export_list__is_outdated'] == 1) {
                $this->get_template_component()->assign('is_outdated', true);
            }

            // add export button
            if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_CABLE_ETSI_BAY")) {
                $exportId = $l_catdata['isys_catg_smview_cable_etsi_bay_export_list__id'];
                $lang = isys_application::instance()->container->get('language');
                isys_component_template_navbar::getInstance()->append_button(
                    $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_EXPORT_TO_EXCEL'),
                    'C__NAVBAR__BUTTON__EXPORT', [
                    'tooltip'       => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_EXPORT_TO_EXCEL'),
                    "icon"          => 'icons/silk/page_white_excel.png',
                    'icon_inactive' => 'icons/silk/page_white_excel.png',
                    "navmode"       => defined_or_default('C__NAVMODE__DUPLICATE'), // important to prevent JS syntax error
                    "js_onclick"    => " onclick=\"get_popup('smview_cable_etsi_export', null, '480', '200', {'func':'single_export', 'export_id':'$exportId'});\""
                ]);
                // export will be handled in isys_module_smview_cable.class.php -> init()
            }
        } else {
            // set object title as default description
            $l_rules = [];
            $global_dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
            $row = $global_dao->get_object_by_id($this->m_object_id)->get_row();
            $l_rules['C__CATG__SMVIEW_CABLE_DESCRIPTION']['p_strValue'] = $row['isys_obj__title'];
            $this->get_template_component()->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);
        }

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign('C__CATG__SMVIEW_CABLE_ETSI_EXPORT', json_encode(['racks' => $assigned_bays]))
            ->assign('js_config', json_encode($this->_get_js_config($p_cat, $assigned_bays)));
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @param  array $bays
     * @return array
     * @throws Exception
     */
    private function _get_js_config(isys_cmdb_dao_category $p_cat, $bays = [])
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_ADD_AS_LAST',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_BAYS_OF',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_AUTO_ASSIGN_RACKS',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_DROP_ELEMENT_HERE',
            'LC__CMDB__CATG__SMVIEW_CABLE__ETSI_REMOVE_RACK',
            'LC__CMDB__CATG__SMVIEW_CABLE__HE',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }

        $obj = $p_cat->get_object_by_id($this->m_object_id)->get_row();
        return [
            'ajax_url'    => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'cable_etsi']),
            'session'     => $_COOKIE['PHPSESSID'],
            'language'    => $language,
            'spinner_url' => isys_helper_link::get_base() . '/images/ajax-loading.gif',
            'racks'       => $bays,
            'mode'        => 'bay_ordering',
            'object'      => [
                'id'   => $obj['isys_obj__id'],
                'name' => $obj['isys_obj__title'],
            ],
        ];
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__ETSI_EXPORT_ORDER');
    }
}
