<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable Pin.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_pin extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        
        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_pin.tpl'); 
    }

    /**
     * Process the list.
     *
     * @param isys_cmdb_dao_category $p_cat
     *
     * @return null|void
     * @throws isys_exception_database
     */
    public function process_list(isys_cmdb_dao_category &$p_cat, $p_get_param_override = null, $p_strVarName = null, $p_strTemplateName = null, $p_bCheckbox = true, $p_bOrderLink = true, $p_db_field_name = null)
    {
        $l_object_id = $p_cat->get_object_id() ?: $_GET[C__CMDB__GET__OBJECT];
        $dao = isys_cmdb_dao_smview_cable_pin::instance($p_cat->get_database_component());
        $pins_in = $dao->filter_objects(array('obj_id' => $l_object_id, 'type' => 'in'));
        $pins_out = $dao->filter_objects(array('obj_id' => $l_object_id, 'type' => 'out'));
        
        $types = $this->_get_types($pins_in, $pins_out);
        if (count($types) <= 1) {
            // no filter needed if only one type exists
            $pins = $dao->get_listing_by_obj_id($l_object_id);
        } elseif (count($pins_in) <= 16 && count($pins_out) <= 16) {
            // no filter needed if less than 16 PINs exists
            $pins = $dao->get_listing_by_obj_id($l_object_id);
        } else {
            $filter = isset($_GET['filter']) ? urldecode($_GET['filter']) : null;
            if ($filter !== null) {
                $pins = $dao->get_listing_by_obj_id($l_object_id, $filter);
            } else {
                $pins = [];
            }
            $this->get_template_component()->assign("types", $types);
        }
        
        $quickinfo_pins = [];
        foreach ($pins as $pin) {
            $quickinfo_pins[] = array(
                'obj_in'  => $this->_add_quickinfo($pin['obj_in']),
                'obj_out' => $this->_add_quickinfo($pin['obj_out']),
            );
        }
        
        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign("pins", $quickinfo_pins)
            ->assign("dao_connector", $p_cat)
            ->assign("list_display", true)
            ->assign("bShowCommentary", "0")
            ->smarty_tom_add_rule("tom.content.top.filter.p_bDisabled=0")
            ->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1")
            ->include_template('contentbottomcontent', isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_pin_list.tpl');
    }
    
    /**
     * Add quick info HTML.
     * 
     * @global array $g_dirs
     * @param  array $obj
     * @return array
     */
    private function _add_quickinfo($obj)
    {
        global $g_dirs;
        $l_quickinfo = isys_factory::get_instance('isys_ajax_handler_quick_info');
        if (! empty($obj['connected_object_obj_id'])) {
            $title = $obj['connected_object_title'] . '/' . $obj['connected_object_pin'];
            $title = "<img src='" . $g_dirs['images'] . "icons/silk/link.png' class='vam' /> $title";
            $obj['connected_object'] = $l_quickinfo->get_quick_info(
                $obj['connected_object_obj_id'],
                $title,
                defined_or_default('C__LINK__CATG'),
                false,
                array(C__CMDB__GET__CATG => defined_or_default('C__CATG__SMVIEW_CABLE_PIN'))
            );
        }

        if (! empty($obj['cable_obj_id'])) {
            $title = $obj['cable_title'] . '/' . $obj['wire_title'];
            $title = "<img src='" . $g_dirs['images'] . "icons/silk/link.png' class='vam' /> $title";
            $obj['cable'] = $l_quickinfo->get_quick_info(
                $obj['cable_obj_id'],
                $title,
                defined_or_default('C__LINK__OBJECT')
            );
        }
        return $obj;
    }
    
    /**
     * Get all types.
     * 
     * @param  array $pins_in
     * @param  array $pins_out
     * @return array
     */
    private function _get_types($pins_in, $pins_out)
    {
        $classifications = [];
        foreach ($pins_in as $obj) {
            $class = $obj['classification'];
            $classifications[$class] = $class;
        }
        foreach ($pins_out as $obj) {
            $class = $obj['classification'];
            $classifications[$class] = $class;
        }
        return $classifications;
    }

    /**
     * Processes the UI for the category check_mk
     *
     * @param isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();
        
        $l_catdata = $p_cat->get_general_data();
        $pin_id = $l_catdata['isys_catg_smview_cable_pin_list__id'];
        
        $l_rules = [];
        $wire_pin_id = null;
        if ($pin_id) {
            $l_rules['C__CATG__SMVIEW_CABLE_TYPE']['p_strSelectedID'] = $l_catdata['isys_catg_smview_cable_pin_list__type'];
            $l_rules['C__CATG__SMVIEW_CABLE_CORRESPONDING_PIN']['p_strSelectedID'] = $l_catdata['isys_catg_smview_cable_pin_list__sibling_id'];
            $l_rules['C__CATG__SMVIEW_CABLE_PATCH_PIN']['p_strSelectedID'] = $l_catdata['isys_catg_smview_cable_pin_list__patch_id'];
            
            // get connected wire pin
            $dao = isys_cmdb_dao_smview_cable_wire_pin::instance(isys_application::instance()->database);
            $wire_pin = $dao->get_object('connected_pin_id', $pin_id);
            
            $wire_pin_id = $wire_pin['id'];
            $l_rules['C__CATG__SMVIEW_CABLE_CONNECTION']['p_strSelectedID'] = $wire_pin_id;
            
            // check if this pin is updateable
            $is_updateable = $p_cat->is_wire_pin_updateable($pin_id);
            if (! $is_updateable) {
                $l_rules = $this->_disableFields($p_cat, $l_rules);
                isys_component_template_navbar::getInstance()
                    ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

                $lang = isys_application::instance()->container->get('language');
                $this->get_template_component()
                    ->assign('message', $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ERROR_PIN_IN_USE'));
            }
        }

        if (! empty($wire_pin_id)) {
            // get wire pin
            $wire_pin_dao = new isys_cmdb_dao_smview_cable_wire_pin($this->m_database_component);
            $row = $wire_pin_dao->get_object('id', $wire_pin_id);

            // get its wire
            $wire_dao = new isys_cmdb_dao_smview_cable_wire($this->m_database_component);
            $row = $wire_dao->get_object('id', $row['wire_id']);

            // use cable object ID
            $obj_id = $row['obj_id'];
            $all_wires = self::get_all_disposed_cable_wires($obj_id, $wire_pin_id);
            $l_rules['C__CATG__SMVIEW_CABLE_CONNECTION']['p_arData'] = serialize($all_wires);
        }

        $all_connectors = $this->_get_connectors($l_catdata['isys_catg_smview_cable_pin_list__connector_id']);
        $l_rules['C__CATG__SMVIEW_CABLE_CONNECTOR']['p_arData'] = serialize($all_connectors);
        
        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign('is_new', ! isset($pin_id))
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);
    }
    
    /**
     * Disable all UI fields.
     * 
     * @param  isys_cmdb_dao_category $p_cat
     * @param  array                  $l_rules
     * @return array
     */
    private function _disableFields($p_cat, $l_rules)
    {
        // get all properties
        $props = $p_cat->get_properties();
        $keys = [];
        foreach ($props as $p) {
            $ui = $p[C__PROPERTY__UI];
            $keys[] = $ui[C__PROPERTY__UI__ID];
        }
        
        // disable all of our fields
        foreach ($keys as $key) {
            if (strpos($key, 'C__CATG__SMVIEW') !== false) {
                $l_rules[$key]['p_bDisabled'] = "1";
            }
        }
        return $l_rules;
    }

    /**
     * Get all disposed cable wires of this object.
     *
     * @param  int $obj_id
     * @param  int|null $wire_pin_id
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public static function get_all_disposed_cable_wires($obj_id, $wire_pin_id = null)
    {
        $sql = "
            SELECT
            wire.isys_catg_smview_cable_wire_list__id as id,
            wp.isys_catg_smview_cable_wire_pin__id as wire_pin_id,
            wp.isys_catg_smview_cable_wire_pin__sibling_id as sibling_id,
            wp.isys_catg_smview_cable_wire_pin__connected_pin_id as connected_pin_id,
            IFNULL(wp.isys_catg_smview_cable_wire_pin__description, wire.isys_catg_smview_cable_wire_list__description) as description,
            obj.isys_obj__title as title,
            obj.isys_obj__status as status
            FROM isys_catg_smview_cable_wire_list wire
            JOIN isys_catg_smview_cable_wire_pin wp ON wire.isys_catg_smview_cable_wire_list__id = wp.isys_catg_smview_cable_wire_pin__wire_id
            JOIN isys_obj obj ON wire.isys_catg_smview_cable_wire_list__isys_obj__id = obj.isys_obj__id
            WHERE wire.isys_catg_smview_cable_wire_list__isys_obj__id = $obj_id;";

        $lang = isys_application::instance()->container->get('language');
        $type_label_in = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IN_SHORT');
        $type_label_out = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_OUT_SHORT');

        $obj_dao = isys_cmdb_dao_category_g_global::instance(isys_application::instance()->database);
        $res = $obj_dao->retrieve($sql);
        $wires = [];

        $wire_dao = new isys_cmdb_dao_category_g_smview_cable_wire(isys_application::instance()->database);
        $disposedLabel = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_IS_DISPOSED');
        while ($row = $res->get_row()) {
            $in_use = $wire_dao->get_in_use($row['id']);
            $isWireDisposed = empty($in_use) || $in_use == $disposedLabel;
            if ($row['wire_pin_id'] == $wire_pin_id || $isWireDisposed) {
                $type = $row['wire_pin_id'] < $row['sibling_id'] ? $type_label_in : $type_label_out;
                $suffix = empty($in_use) ? " ($type)" : " ($type / $in_use)";
                $wires[$row['wire_pin_id']] = $row['title'] . ' | ' . $row['description'] . $suffix;
            }
        }
        return $wires;
    }

    /**
     * Get connectors by connector ID.
     *
     * @param  int $connector_id
     * @return array
     */
    private function _get_connectors($connector_id = null)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $obj_id = $this->m_object_id;
        if (! empty($connector_id)) {
            // get connector by ID
            $l_row = $connector_dao->get_data($connector_id)->get_row();
            $obj_id = $l_row['isys_obj__id'];
        }

        // get all other incoming connectors of this object
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT
        );

        $connectors = [];
        while ($row = $l_data->get_row()) {
            $id = $row['isys_catg_connector_list__id'];
            $obj_title = $row['isys_obj__title'];
            $title = $row['isys_catg_connector_list__title'];
            $type = $row['isys_connection_type__title'];
            $connectors[$id] = "$obj_title - $title  ($type)";
        }
        return $connectors;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_catdata = $p_cat->get_general_data();
        $description = $l_catdata['isys_catg_smview_cable_pin_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__PIN_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__PIN_CAT_TITLE') . ": $description";
    }
}
