<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable Route Element.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_route_element extends isys_cmdb_ui_category_global
{

    /**
     * Process the list.
     *
     * @param isys_cmdb_dao_category $p_cat
     * @param null                   $p_get_param_override
     * @param null                   $p_strVarName
     * @param null                   $p_strTemplateName
     * @param boolean                $p_bCheckbox
     * @param boolean                $p_bOrderLink
     * @param null                   $p_db_field_name
     *
     * @return null|void
     */
    public function process_list(isys_cmdb_dao_category &$p_cat, $p_get_param_override = null, $p_strVarName = null, $p_strTemplateName = null, $p_bCheckbox = true, $p_bOrderLink = true, $p_db_field_name = null)
    {
        $l_object_id = $p_cat->get_object_id() ?: $_GET[C__CMDB__GET__OBJECT];
        $url = isys_helper_link::get_base() . isys_helper_link::create_url(
            [
            C__CMDB__GET__OBJECT => $l_object_id,
            C__CMDB__GET__CATG => C__CATG__SMVIEW_CABLE_ROUTE]
        );
        
        $this->get_template_component()
            ->assign("bShowCommentary", "0")
            ->assign("redirect_url", $url)
            ->smarty_tom_add_rule("tom.content.top.filter.p_bDisabled=0")
            ->smarty_tom_add_rule("tom.content.top.buttons.*.p_bInvisible=1")
            ->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1")
            ->include_template('contentbottomcontent', isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_route_element.tpl');
        
        // set buttons invisible
        isys_component_template_navbar::getInstance()
            ->set_visible(false, C__NAVBAR_BUTTON__NEW)
            ->set_visible(false, C__NAVBAR_BUTTON__EDIT)
            ->set_visible(false, C__NAVBAR_BUTTON__ARCHIVE)
            ->set_visible(false, C__NAVBAR_BUTTON__QUICK_PURGE)
            ->set_visible(false, C__NAVBAR_BUTTON__PRINT);
    }
    
    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_catdata = $p_cat->get_general_data();
        $description = $l_catdata['isys_catg_smview_cable_route_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_CAT_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_CAT_TITLE') . ": $description";
    }
}
