<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable Section.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_section extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);

        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_section.tpl');
    }

    /**
     * Processes the UI for this category.
     *
     * @param isys_cmdb_dao_category $p_cat The corresponding category DAO
     *
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();
        
        $l_rules = [];
        $l_cat_data = $p_cat->get_general_data();
        $section_id = $l_cat_data['isys_catg_smview_cable_section_list__id'];
        $section_elements = [];
        $damping = [];
        $damping_1310_factor = isys_settings::get(isys_module_smview_cable::DAMPING_1310, 0);
        $damping_1550_factor = isys_settings::get(isys_module_smview_cable::DAMPING_1550, 0);
        if ($section_id) {
            $dao = new isys_cmdb_dao_category_g_smview_cable_section_element(isys_application::instance()->database);
            $section_elements = $dao->get_elements_by_section_id($section_id, false, isys_glob_is_edit_mode(), true);
            $l_rules['C__CATG__SMVIEW_CABLE_ELEMENTS']['p_strValue'] = $section_elements;

            // check if this section is used within a route
            $route_element_dao = new isys_cmdb_dao_smview_cable_route_element(isys_application::instance()->database);
            $route_element = $route_element_dao->get_object('section_id', $section_id);
            if (! empty($route_element)) {
                $route_dao = new isys_cmdb_dao_smview_cable_route(isys_application::instance()->database);
                $element = $route_dao->get_object('id', $route_element['route_id']);
                $global_dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
                $obj = $global_dao->get_object_by_id($element['obj_id'])->get_row();
                $l_rules['C__CATG__SMVIEW_CABLE_ROUTE']['p_strValue'] = $obj['isys_obj__title'];
            }

            $section_dao = new isys_cmdb_dao_category_g_smview_cable_section(isys_application::instance()->database);
            $damping = $section_dao->calculate_damping($section_elements, $damping_1310_factor, $damping_1550_factor);
        }

        $section_dao = new isys_cmdb_dao_category_g_smview_cable_section(isys_application::instance()->database);
        $object_types = $section_dao->callback_property_start_object_types();

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign('download_url', isys_helper_link::create_url([C__GET__AJAX => 1,
                                                                         C__GET__AJAX_CALL => 'cable_section',
                                                                         'func' => 'download',
                                                                         'obj_id' => $l_cat_data['isys_obj__id']]))
            ->assign('damping', $damping)
            ->assign('damping_1310_factor', $damping_1310_factor)
            ->assign('damping_1550_factor', $damping_1550_factor)
            ->assign('C__CATG__SMVIEW_CABLE_ELEMENTS', json_encode($section_elements))
            ->assign('js_config', json_encode([
                'ajax_url'     => isys_helper_link::create_url([C__GET__AJAX => 1,
                                                                C__GET__AJAX_CALL => 'cable_section']),
                'session'      => $_COOKIE['PHPSESSID'],
                'language'     => $this->get_js_translations(),
                'spinner_url'  => isys_helper_link::get_base() . '/images/ajax-loading.gif',
                'elements'     => array_values($section_elements),
                'object_types' => $object_types,
                'is_route'     => false,
            ]))
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    public static function get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_INDEX',
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_OBJECT_OUTGOING',
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_OBJECT_INCOMING',
            'LC__SHD__TELECOMMUNICATION_CABLE',
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_ELEMENT_CAPACITY',
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_ELEMENT_LENGTH',
            'LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_OPTIONS',
            'LC__CMDB__CATG__SMVIEW_CABLE__START_OBJECT_TYPES',
            'LC__CMDB__CATG__SMVIEW_CABLE__AUTOMATIC_TRACING',
            'LC__CMDB__CATG__SMVIEW_CABLE__SHORTEST_PATH',
            'LC__CMDB__CATG__SMVIEW_CABLE__LEAST_HOPS',
            'LC__CMDB__CATG__SMVIEW_CABLE__FROM',
            'LC__CMDB__CATG__SMVIEW_CABLE__TO',
            'LC__CMDB__CATG__SMVIEW_CABLE__START_AUTOMATIC_TRACING',
            'LC__CMDB__CATG__SMVIEW_CABLE__CONFIGURE_ASSOCIATED_ROUTE',
            'LC__CMDB__CATG__SMVIEW_CABLE__ASSOCIATED_ROUTE',
            'LC__CMDB__CATG__SMVIEW_CABLE__CONSIDER_PATCHED_CONNECTIONS',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_cat_data = $p_cat->get_general_data();
        $description = $l_cat_data['isys_catg_smview_cable_section_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__SECTION_CAT_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__SECTION_CAT_TITLE') . ": $description";
    }
}
