<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable Virtual Route.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_virtual_route extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_virtual_route.tpl');
    }

    /**
     * Processes the UI for this category.
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb|isys_exception_database
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        $l_rules = [];
        $l_cat_data = $p_cat->get_general_data();
        $route_id = $l_cat_data['isys_catg_smview_cable_vir_route_list__id'];
        $route_elements = [];
        $damping = [];
        $damping_1310_factor = isys_settings::get(isys_module_smview_cable::DAMPING_1310, 0);
        $damping_1550_factor = isys_settings::get(isys_module_smview_cable::DAMPING_1550, 0);
        $section_dao = new isys_cmdb_dao_category_g_smview_cable_section(isys_application::instance()->database);
        if ($route_id) {
            $l_rules['C__CATG__SMVIEW_CABLE_DESCRIPTION']['p_strValue'] = $l_cat_data['isys_catg_smview_cable_vir_route_list__description'];

            $dao = new isys_cmdb_dao_category_g_smview_cable_virtual_route_element(isys_application::instance()->database);
            $route_elements = $dao->get_elements_by_route_id($route_id, false, true, true);
            $l_rules['C__CATG__SMVIEW_CABLE_ELEMENTS']['p_strValue'] = $route_elements;

            $damping = $section_dao->calculate_damping($route_elements, $damping_1310_factor, $damping_1550_factor);
        }
        $object_types = $section_dao->callback_property_start_object_types();

        // only get new routes without elements
        $sql = "SELECT o.isys_obj__id, o.isys_obj__title
                FROM isys_catg_smview_cable_vir_route_list r
                JOIN isys_obj o ON r.isys_catg_smview_cable_vir_route_list__isys_obj__id = o.isys_obj__id
                WHERE r.isys_catg_smview_cable_vir_route_list__id NOT IN
                  (SELECT e.isys_catg_smview_cable_vir_route_element__vir_route_id FROM isys_catg_smview_cable_vir_route_element e)";
        $res = $p_cat->retrieve($sql);
        $routes = [];
        while ($row = $res->get_row()) {
            $routes[$row['isys_obj__id']] = $row['isys_obj__title'];
        }

        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->assign('download_url', isys_helper_link::create_url([C__GET__AJAX => 1,
                                                                   C__GET__AJAX_CALL => 'cable_route',
                                                                   'func' => 'download',
                                                                   'obj_id' => $l_cat_data['isys_obj__id']]))
            ->assign('damping', $damping)
            ->assign('damping_1310_factor', $damping_1310_factor)
            ->assign('damping_1550_factor', $damping_1550_factor)
            ->assign('C__CATG__SMVIEW_CABLE_ELEMENTS', json_encode($route_elements))
            ->assign('js_config', json_encode([
                'ajax_url'         => isys_helper_link::create_url([C__GET__AJAX => 1,
                                                                    C__GET__AJAX_CALL => 'cable_route']),
                'session'          => $_COOKIE['PHPSESSID'],
                'language'         => isys_cmdb_ui_category_g_smview_cable_section::get_js_translations(),
                'spinner_url'      => isys_helper_link::get_base() . '/images/ajax-loading.gif',
                'elements'         => array_values($route_elements),
                'object_types'     => $object_types,
                'routes'           => $routes,
                'is_route'         => true,
                'is_virtual_route' => true,
            ]))
            ->assign('obj_id', $l_cat_data['isys_obj__id'])
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_cat_data = $p_cat->get_general_data();
        $description = $l_cat_data['isys_catg_smview_cable_vir_route_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__VIRTUAL_ROUTE_CAT_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__VIRTUAL_ROUTE_CAT_TITLE') . ": $description";
    }
}
