<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Cable Wire.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_cable_wire extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_cable::get_tpl_dir() . 'module/cmdb/catg__smview_cable_wire.tpl');
    }

    /**
     * Processes the UI for the category check_mk
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->disable_ajax_save();
        $this->deactivate_commentary();
        
        $l_rules = [];
        $l_catdata = $p_cat->get_general_data();
        $wire_id = $l_catdata['isys_obj__id'];
        if ($wire_id) {
            // add informations of wire pins
            $dao = new isys_cmdb_dao_smview_cable_wire_pin(isys_application::instance()->database);
            $pins = $dao->filter_objects(array('wire_id' => $l_catdata['isys_catg_smview_cable_wire_list__id']));
            $pin_left = $pins[0]['id'] < $pins[1]['id'] ? $pins[0] : $pins[1];
            $pin_right = $pins[0]['id'] > $pins[1]['id'] ? $pins[0] : $pins[1];
            
            $l_rules['C__CATG__SMVIEW_CABLE_STATE_IN']['p_strSelectedID'] = $pin_left['status_id'];
            $l_rules['C__CATG__SMVIEW_CABLE_STATE_OUT']['p_strSelectedID'] = $pin_right['status_id'];
            $l_rules['C__CATG__SMVIEW_CABLE_DESCRIPTION_INCOMING']['p_strValue'] = $pin_left['description'];
            $l_rules['C__CATG__SMVIEW_CABLE_DESCRIPTION_OUTGOING']['p_strValue'] = $pin_right['description'];
        }
        
        $this->get_template_component()
            ->assign('assets_dir', isys_module_smview_cable::get_assets_www_dir())
            ->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title, otherwise error message.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $language = isys_application::instance()->container->get('language');
        $l_catdata = $p_cat->get_general_data();
        $description = $l_catdata['isys_catg_smview_cable_wire_list__description'];
        if (empty($description)) {
            return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_CAT_TITLE');
        }
        return $language->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_CAT_TITLE') . ": $description";
    }
}
