<?php

/**
 * Cable generator AJAX handler
 *
 * @package i-doit
 * @subpackage smview_cable
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class isys_ajax_handler_cable_generator extends isys_ajax_handler {
    
    /**
     * Init method, which gets called from the framework.
     */
    public function init() {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            switch ($this->m_get['func']) {
                case 'load':
                    $generator_id = $this->m_post['generator_id'];
                    $dao = new \isys_cmdb_dao_smview_cable_generator(\isys_application::instance()->database);
                    $generator = $dao->get_object('id', $generator_id);
                    $l_return['data'] = array(
                        'counter_min' => $generator['counter_min'],
                        'counter_max' => $generator['counter_max'],
                    );
                    break;
                case 'preview':
                    $generator_id = $this->m_post['generator_id'];
                    $start_pattern = $this->m_post['start_pattern'];
                    $end_pattern = $this->m_post['end_pattern'];
                    
                    $name_generator = new \idoit\Module\Cable\LabelGenerator($generator_id);
                    try {
                        $names = $name_generator->generate_names_by_generator($start_pattern, $end_pattern);
                    } catch (InvalidArgumentException $e) {
                        $l_return['data'] = array(
                            'error' => $e->getMessage(),
                        );
                        break;
                    }
                    
                    $l_return['data'] = array(
                        'elements' => $names,
                    );
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return  boolean
     */
    public static function needs_hypergate() {
        return true;
    }
}
