<?php

/**
 * Cable pin AJAX handler
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_cable_pin extends isys_ajax_handler
{

    /**
     * Init method, which gets called from the framework.
     *
     * @throws Exception
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $canView = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_CABLE_PIN");
            $canEdit = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_CABLE_PIN");
            switch ($this->m_get['func']) {
                case 'detach-connection':
                    if ($canEdit) {
                        $pin_id = $this->m_post['pin_id'];
                        $this->_remove_connection($pin_id);
                    }
                    break;
                case 'get-wires':
                    if ($canView) {
                        $obj_id = $this->m_get['obj_id'];
                        $l_return['data'] = isys_cmdb_ui_category_g_smview_cable_pin::get_all_disposed_cable_wires($obj_id);
                    }
                    break;
                case 'get-connectors':
                    if ($canView) {
                        $obj_id = $this->m_get['obj_id'];
                        $l_return['data'] = $this->_get_connectors($obj_id);
                    }
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }
    
    /**
     * Remove the connection of a pin.
     * 
     * @param int $pin_id
     */
    private function _remove_connection($pin_id)
    {
        $dao = isys_cmdb_dao_smview_cable_wire_pin::instance($this->m_database_component);
        $connections = $dao->filter_objects(array('connected_pin_id' => $pin_id));
        foreach ($connections as $c) {
            // remove current connection of this PIN
            $dao->update_wire_pin($c['id'], $c['description'], $c['status_id']);
        }
    }

    /**
     * Get connectors of this object.
     *
     * @param  int $obj_id
     * @return array
     */
    private function _get_connectors($obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_database_component);
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__INPUT
        );

        $connectors = [];
        while ($row = $l_data->get_row()) {
            $id = $row['isys_catg_connector_list__id'];
            $obj_title = $row['isys_obj__title'];
            $title = $row['isys_catg_connector_list__title'];
            $type = $row['isys_connection_type__title'];
            $connectors[$id] = "$obj_title - $title  ($type)";
        }
        return $connectors;
    }
}
