<?php

use idoit\Module\Cable\CableTracing;
use idoit\Module\Cable\SectionTracing;
use idoit\Module\Cable\Tracing;
use idoit\Module\Cable\VirtualRoute;

/**
 * Cable route AJAX handler
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_cable_route extends isys_ajax_handler
{
    
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            $canView = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_CABLE_ROUTE");
            $canViewVR = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE");
            $canActivateVR = isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EXECUTE, "CATEGORY/C__CATG__SMVIEW_CABLE_VIRTUAL_ROUTE");
            switch ($this->m_get['func']) {
                case 'get-objects-by-type':
                    if ($canView) {
                        $type_id = !empty($this->m_post['type_id']) ? $this->m_post['type_id'] : $this->m_get['type_id'];
                        $l_return['data'] = $this->_get_objects_by_type($type_id);
                    }
                    break;
                case 'get-connected-objects':
                    if ($canView) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $cable_tracing = new CableTracing($obj_id);
                        $section_tracing = new SectionTracing($obj_id);
                        $ignored_wire_id = !empty($this->m_post['wire_id']) ? $this->m_post['wire_id'] : $this->m_get['wire_id'];
                        $ignored_section_id = !empty($this->m_post['section_id']) ? $this->m_post['section_id'] : $this->m_get['section_id'];
                        $associated_route = !empty($this->m_post['associated_route']) ? $this->m_post['associated_route'] : $this->m_get['associated_route'];
                        $consider_patched_connections = !empty($this->m_post['consider_patched_connections']) ? $this->m_post['consider_patched_connections'] : $this->m_get['consider_patched_connections'];
                        $consider_patched_connections = $consider_patched_connections == '1';
                        $ignored_wire_id = $ignored_wire_id != 'null' ? $ignored_wire_id : null;
                        $ignored_section_id = $ignored_section_id != 'null' ? $ignored_section_id : null;

                        $connections = $cable_tracing->get_unpatched_connections($ignored_wire_id, $ignored_wire_id, $associated_route, $consider_patched_connections);
                        $sections = $section_tracing->get_unpatched_sections($ignored_section_id, $ignored_wire_id, $associated_route, $consider_patched_connections);
                        $merged_elements = array_merge($connections['objects'], $sections['objects']);
                        $connections['objects'] = $merged_elements;
                        $l_return['data'] = $connections;
                    }
                    break;
                case 'trace':
                    if ($canView) {
                        $start_obj_id = !empty($this->m_post['start_obj_id']) ? $this->m_post['start_obj_id'] : $this->m_get['start_obj_id'];
                        $end_obj_id = !empty($this->m_post['end_obj_id']) ? $this->m_post['end_obj_id'] : $this->m_get['end_obj_id'];
                        $search_method = isset($this->m_post['search_method']) ? $this->m_post['search_method'] : 'shortest_path';
                        $l_return['data'] = $this->_trace($start_obj_id, $end_obj_id, $search_method);
                    }
                    break;
                case 'download':
                    if ($canView) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        isys_ajax_handler_cable_section::downloadCalculatedDampingAsExcelFile($obj_id);
                    }
                    break;
                case 'check-virtual-route':
                    if ($canViewVR) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $vr = new VirtualRoute();
                        $res = $vr->check($obj_id);
                        $l_return['data'] = $res;
                    }
                    break;
                case 'activate-virtual-route':
                    if ($canActivateVR) {
                        $obj_id = !empty($this->m_post['obj_id']) ? $this->m_post['obj_id'] : $this->m_get['obj_id'];
                        $vr = new VirtualRoute();
                        $route_obj_id = $vr->create_route_from_vr($obj_id, true);
                        $url = isys_helper_link::create_url([C__CMDB__GET__OBJECT => $route_obj_id]);
                        $l_return['data'] = [
                            'obj_id' => $route_obj_id,
                            'url'    => $url,
                        ];
                    }
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * Get objects by type.
     *
     * @param  int $type_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_objects_by_type($type_id)
    {
        $query = "SELECT 
                o.isys_obj__id,
                o.isys_obj__title
            FROM isys_catg_smview_cable_pin_list l
            JOIN isys_obj o ON l.isys_catg_smview_cable_pin_list__isys_obj__id = o.isys_obj__id
            JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
            WHERE t.isys_obj_type__id=$type_id;";
        $dao = new isys_cmdb_dao_smview_cable_section(isys_application::instance()->database);
        $res = $dao->retrieve($query);
        
        $objects = array();
        while ($row = $res->get_row()) {
            $obj_id = $row['isys_obj__id'];
            $objects[$obj_id] = array(
                'id'    => $obj_id,
                'title' => $row['isys_obj__title'],
            );
        }
        
        uasort($objects, array($this, '_sortByTitle'));
        return array_values($objects);
    }

    /**
     * Sort array by title of an object.
     *
     * @param  array $a
     * @param  array $b
     * @return int
     */
    private function _sortByTitle($a, $b)
    {
        $left = $a['title'];
        $right = $b['title'];
        return strnatcmp($left, $right);
    }

    /**
     * Trace between two objects.
     *
     * @param  int $start_obj_id
     * @param  int $end_obj_id
     * @param  string $search_method
     * @return array
     */
    private function _trace($start_obj_id, $end_obj_id, $search_method)
    {
        $tracing = new Tracing($start_obj_id);
        if ($search_method == 'shortest_path') {
            $route = $tracing->get_shortest_path($end_obj_id);
        } else {
            $route = $tracing->get_least_number_of_hops($end_obj_id);
        }

        if (empty($route)) {
            return [];
        }

        $elements = [];
        $index = 1;
        foreach ($route as $part) {
            $wires = $part['wires'];
            $wire = null;
            // get first wire of shortest cable
            foreach ($wires as $w) {
                if (! $wire || $w['length'] < $wire['length']) {
                    $wire = $w;
                }
            }

            $element = array(
                'index'   => $index,
                'wire_id' => isset($wire['wire_id']) ? $wire['wire_id'] : null,
                'section_id' => isset($wire['section_id']) ? $wire['section_id'] : null,
            );

            if (! empty($wire['section_id'])) {
                $section_id = $element['section_id'];
                $section = SectionTracing::get_section_endpoints($section_id);
                $element = array_merge($element, $section);
            } else {
                $element = isys_cmdb_dao_category_g_smview_cable_section_element::get_wire_connection($element);
            }
            $elements[$element['index']] = $element;
            $index++;
        }

        // sort elements by index
        ksort($elements);
        $ordered_elements = isys_cmdb_dao_category_g_smview_cable_section_element::order_objects_within_connections($elements, $start_obj_id);

        // add connection information to each element
        $new_elements = [];
        foreach ($ordered_elements as $index => $element) {
            $new_elements[$index] = isys_cmdb_dao_category_g_smview_cable_route_element::add_connection_information($element);
        }

        // finally sort elements by index
        ksort($new_elements);
        return $new_elements;
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }
}
