<?php

use idoit\Module\Cable\LocationTracing;
use idoit\Module\Cable\OrientDB;

/**
 * Cable tracing AJAX handler
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_cable_tracing extends isys_ajax_handler
{
    
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            switch ($this->m_get['func']) {
                case 'trace-between-locations':
                    if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, "CATEGORY/C__CATG__SMVIEW_CABLE")) {
                        $location_start = $this->m_post['location_start'];
                        $location_end = $this->m_post['location_end'];

                        $tracing = new LocationTracing();
                        $data = $tracing->get_nodes_between_locations($location_start, $location_end);
                        $l_return['data'] = $data;
                    }
                    break;
                case 'sync-cache':
                    if (isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, "CATEGORY/C__CATG__SMVIEW_CABLE")) {
                        $orient_db = new OrientDB();
                        $orient_db->sync_connections();
                        $l_return['data'] = 'success';
                    }
                    break;
            }
        }
        catch (isys_exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate()
    {
        return true;
    }
}
