<?php

/**
 * i-doit SM-VIEW cable module
 *
 * Installation routines
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */

class isys_module_smview_cable_install {

    /**
     * Executes module installation.
     *
     * @param isys_component_database $p_database Tenant database
     *
     * @throws Exception in CLI or HTML format
     */
    public static function init($p_database) {
        if (empty($p_database)) {
            // disabled tenant database
            return;
        }

        // migrate pin status
        $sql = 'SELECT isys_catg_smview_cable_wire_pin__state FROM isys_catg_smview_cable_wire_pin LIMIT 1;';
        $l_dao = new isys_cmdb_dao_category_property($p_database);
        try {
            $l_dao->retrieve($sql)->get_row();
            self::_migrate_wire_pin_status($p_database);
        } catch (Exception $e) {
            // field does not exist, migration already applied
        }

        // migrate cable status
        $sql = 'SELECT isys_catg_smview_cable_list__state FROM isys_catg_smview_cable_list LIMIT 1;';
        try {
            $l_dao->retrieve($sql)->get_row();
            self::_migrate_cable_status($p_database);
        } catch (Exception $e) {
            // field does not exist, migration already applied
        }

        // clear temp-dir / cache after (to load new classes)
        self::_clear_cache();

        /*
            HOTFIX: Check if class exist to avoid the error below.
            @todo: Search for reason, why this class can not be found.

            PHP Fatal error:  Uncaught Error: Class 'idoit\\Module\\Report\\SqlQuery\\Structure\\SelectSubSelect' not found in
            /var/www/html/i-doit/src/classes/modules/cmdb/dao/category/global/isys_cmdb_dao_category_g_custom_fields.class.php:1220\n

            Stack trace:\n
            #0 /var/www/html/i-doit/src/classes/modules/cmdb/dao/category/isys_cmdb_dao_category.class.php(2287): isys_cmdb_dao_category_g_custom_fields->properties()\n
            #1 /var/www/html/i-doit/updates/classes/isys_update_property_migration.class.php(236): isys_cmdb_dao_category->get_properties()\n
            #2 /var/www/html/i-doit/src/classes/modules/cmdb/dao/category/isys_cmdb_dao_category_property.class.php(3649): isys_update_property_migration->prepare_sql_queries('g_custom')\n
            #3 /var/www/html/i-doit/src/classes/modules/smview_cable/install/isys_module_smview_cable_install.class.php(52): isys_cmdb_dao_category_property->rebuild_properties()\n
            #4 /var/www/html/i-doit/admin/src/functions.inc.php(210): isys_module_smview_cable_install::init(Object(isys_component_database_mysqli), Object(isys_component_database_proxy)
            in /var/www/html/i-doit/src/classes/modules/cmdb/dao/category/global/isys_cmdb_dao_category_g_custom_fields.class.php on line 1220
         */

        if (! class_exists('idoit\Module\Report\SqlQuery\Structure\SelectSubSelect')) {
            return;
        }
        
        try {
            // rebuild properties on update
            $l_dao = new isys_cmdb_dao_category_property($p_database);
            $l_dao->rebuild_properties();
        } catch (Exception $e) {
            $msg = array();

            while ($e) {
                $msg[] = $e->getMessage();
                $e = $e->getPrevious();
            }

            $lineBreak = defined('STDIN') ? PHP_EOL : '<br/>';
            throw new Exception(implode($lineBreak, $msg));
        }
    }

    /**
     * Migrate wire pin status from constant to ID.
     *
     * @param  isys_component_database $p_database
     * @throws isys_exception_dao
     */
    private static function _migrate_wire_pin_status($p_database)
    {
        // add default data
        $status = [
            'H'  => 'hängend',
            'VH' => 'vorbereitet, hängend',
            'R'  => 'reserviert',
        ];

        $l_dao = new isys_cmdb_dao_category_property($p_database);
        foreach ($status as $const => $description) {
            $sql = "INSERT INTO isys_catg_smview_cable_wire_pin_status SET isys_catg_smview_cable_wire_pin_status__const = '$const',
                isys_catg_smview_cable_wire_pin_status__description = '$description';";
            $l_dao->update($sql);
            $l_dao->apply_update();
            $id = $l_dao->get_last_insert_id();

            // update wire pin data with new ID
            $sql = "UPDATE isys_catg_smview_cable_wire_pin SET isys_catg_smview_cable_wire_pin__status_id = $id
                WHERE isys_catg_smview_cable_wire_pin__state = '$const';";
            $l_dao->update($sql);
            $l_dao->apply_update();
        }
    }

    /**
     * Migrate cable status from constant to ID.
     *
     * @param  isys_component_database $p_database
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private static function _migrate_cable_status($p_database)
    {
        // get current user states
        $l_dao = new isys_cmdb_dao_category_property($p_database);
        $sql = "SELECT * FROM isys_catg_smview_cable_user_state;";
        $res = $l_dao->retrieve($sql);
        while ($row = $res->get_row()) {
            $ident = $row['isys_catg_smview_cable_user_state__name'] . ': ' . $row['isys_catg_smview_cable_user_state__description'];
            $ident = trim($ident);
            $states[$ident] = $row['isys_catg_smview_cable_user_state__id'];
        }

        $sql = 'SELECT * FROM isys_catg_smview_cable_list;';
        $res = $l_dao->retrieve($sql);
        while ($row = $res->get_row()) {
            // try to get user state by name
            $status = trim($row['isys_catg_smview_cable_list__state']);
            if (empty($status)) {
                continue;
            }

            if (! isset($states[$status])) {
                continue;
            }

            // update cable with user state ID
            $userStateId = $states[$status];
            $cableId = $row['isys_catg_smview_cable_list__id'];
            $sql = "UPDATE isys_catg_smview_cable_list SET isys_catg_smview_cable_list__state_id = $userStateId
                WHERE isys_catg_smview_cable_list__id = '$cableId';";
            $l_dao->update($sql);
            $l_dao->apply_update();
        }
    }

    /**
     * Clear cache.
     *
     * @throws Exception
     */
    private static function _clear_cache()
    {
        global $g_dirs;

        // Removing isys_cache values
        \isys_cache::keyvalue()->flush();

        // Deleting contents of the temp directory.
        $l_deleted = $l_undeleted = 0;
        \isys_glob_delete_recursive($g_dirs["temp"], $l_deleted, $l_undeleted, (ENVIRONMENT === 'development'));

        \isys_component_signalcollection::get_instance()->emit('system.afterFlushSystemCache');
    }
}
