<?php

/**
 * i-doit
 *
 * Class autoloader.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_module_smview_cable_autoload extends isys_module_manager_autoload {

    /**
     * Autoloader
     *
     * @param string $className
     * @return boolean
     */
    public static function init($className)
    {
        $addOnPath = '/src/classes/modules/smview_cable/';
        $class_loader_paths = array(
            '/api/model/',
            '/api/model/cable/',
            '/cmdb/dao/',
            '/cmdb/dao/global/',
            '/cmdb/dao/list/',
            '/cmdb/ui/global/',
            '/handler/controller/',
        );

        //dynamic loading of classes
        foreach ($class_loader_paths as $clp) {
            $l_path = $clp . $className . '.class.php';
            if (! file_exists( __DIR__ . $l_path)) {
                continue;
            }

            if (parent::include_file($addOnPath . $l_path)) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_path);

                return true;
            }
        }
        
        // single classes
        $l_classes = [
            'isys_ajax_handler_cable_etsi'                                => '/handler/ajax/isys_ajax_handler_cable_etsi.class.php',
            'isys_ajax_handler_cable_generator'                           => '/handler/ajax/isys_ajax_handler_cable_generator.class.php',
            'isys_ajax_handler_cable_pin'                                 => '/handler/ajax/isys_ajax_handler_cable_pin.class.php',
            'isys_ajax_handler_cable_route'                               => '/handler/ajax/isys_ajax_handler_cable_route.class.php',
            'isys_ajax_handler_cable_section'                             => '/handler/ajax/isys_ajax_handler_cable_section.class.php',
            'isys_ajax_handler_cable_tracing'                             => '/handler/ajax/isys_ajax_handler_cable_tracing.class.php',
            'idoit\Module\Cable\CableCapacity'                            => '/src/CableCapacity.php',
            'idoit\Module\Cable\CableListImporter'                        => '/src/CableListImporter.php',
            'idoit\Module\Cable\CableTracing'                             => '/src/CableTracing.php',
            'idoit\Module\Cable\ExcelExporter\EtsiBayExporter'            => '/src/ExcelExporter/EtsiBayExporter.php',
            'idoit\Module\Cable\ExcelExporter\AbstractExporter'           => '/src/ExcelExporter/AbstractExporter.php',
            'idoit\Module\Cable\ExcelExporter\DampingExporter'            => '/src/ExcelExporter/DampingExporter.php',
            'idoit\Module\Cable\ExcelExporter\RouteExporter'              => '/src/ExcelExporter/RouteExporter.php',
            'idoit\Module\Cable\LabelGenerator'                           => '/src/LabelGenerator.php',
            'idoit\Module\Cable\LocationTracing'                          => '/src/LocationTracing.php',
            'idoit\Module\Cable\ObjectTitleValidator'                     => '/src/ObjectTitleValidator.php',
            'idoit\Module\Cable\OrientDB'                                 => '/src/OrientDB.php',
            'idoit\Module\Cable\OrientDBSync'                             => '/src/OrientDBSync.php',
            'idoit\Module\Cable\SectionRouteValidator'                    => '/src/SectionRouteValidator.php',
            'idoit\Module\Cable\SectionTracing'                           => '/src/SectionTracing.php',
            'idoit\Module\Cable\Tracing'                                  => '/src/Tracing.php',
            'idoit\Module\Cable\VirtualRoute'                             => '/src/VirtualRoute.php',
            'idoit\Module\Cable\Algorithms\IAlgorithm'                    => '/src/Algorithms/IAlgorithm.php',
            'idoit\Module\Cable\Algorithms\BreadthFirstSearch'            => '/src/Algorithms/BreadthFirstSearch.php',
            'idoit\Module\Cable\Algorithms\Dijkstra'                      => '/src/Algorithms/Dijkstra.php',
            'idoit\Module\Cable\Algorithms\AllPaths'                      => '/src/Algorithms/AllPaths.php',
            'idoit\Module\Smview_Cable\Console\Command\TestDataGeneratorCommand' => __DIR__ . '/src/Console/Command/TestDataGeneratorCommand.php',
            'isys_popup_smview_cable_etsi_export'                         => 'popups/isys_popup_smview_cable_etsi_export.class.php',
            'isys_popup_smview_cable_route_export'                        => 'popups/isys_popup_smview_cable_route_export.class.php',
        ];
        
        // manager classes, available via main menu
        $manager_classes = [
            'AbstractManager',
            'GeneratorManager',
            'ConnectionTypeManager',
            'UserStateManager',
            'GroupManager',
            'CableTypeManager',
            'AdminUnitManager',
            'TechnologyManager',
            'RouteUsageManager',
            'WirePinStatusManager',
            'EtsiDeviceColorManager',
            'EtsiExportManager',
        ];
        
        foreach ($manager_classes as $cls) {
            $class_path = "idoit\Module\Cable\DependentObjectManager\\" . $cls;
            $file_path =  '/src/DependentObjectManager/' . $cls . '.php';
            $l_classes[$class_path] = $file_path;
        }

        // include composer's autoloader
        $vendorDir = dirname(__FILE__) . '/vendor/';
        if (file_exists($vendorDir . 'autoload.php')) {
            include_once($vendorDir . 'autoload.php');
        }
        unset($vendorDir);
        
        if (isset($l_classes[$className])) {
            if (parent::include_file($addOnPath . $l_classes[$className])) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_classes[$className]);

                return true;
            }
        }

        return false;
    }
}
