<?php

namespace idoit\Module\Cable\Algorithms;

use SplDoublyLinkedList;
use SplQueue;

require_once('IAlgorithm.php'); // needed for PHPUnit

/**
 * All Path algorithm class.
 *
 * @package i-doit
 * @subpackage smview_cable
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class AllPaths implements IAlgorithm
{
    
    /**
     * Graph.
     *
     * @var array
     */
    protected $graph;
    
    /**
     * Visited nodes.
     *
     * @var array
     */
    protected $visited = array();

    /**
     * Constructor.
     *
     * @param array $graph

        Structure of graph:

        $graph = array(
          'A' => array('B', 'F'),
          'B' => array('A', 'D', 'E'),
          'C' => array('F'),
          'D' => array('B', 'E'),
          'E' => array('B', 'D', 'F'),
          'F' => array('A', 'E', 'C'),
        );

     */
    public function __construct($graph)
    {
        $this->graph = $graph;
    }
    
    /**
     * Get all path between two nodes.
     *
     * @param string $origin
     * @param string $destination
     * @return array
     */
    public function calculate($origin, $destination)
    {
        // mark all nodes as unvisited
        foreach ($this->graph as $vertex => $adj) {
            $this->visited[$vertex] = false;
        }

        // create an empty queue
        $q = new SplQueue();

        // enqueue the origin vertex and mark as visited
        $q->enqueue($origin);
        $this->visited[$origin] = true;

        // this is used to track the path back from each node
        $path = array();
        $path[$origin] = new SplDoublyLinkedList();
        $path[$origin]->setIteratorMode(
            SplDoublyLinkedList::IT_MODE_FIFO|SplDoublyLinkedList::IT_MODE_KEEP
        );

        $path[$origin]->push($origin);
        $dest_paths = [];

        // while queue is not empty and destination not found
        while (!$q->isEmpty()) {
            $t = $q->dequeue();
            if (empty($this->graph[$t])) {
                continue;
            }
            
            // for each adjacent neighbor
            foreach ($this->graph[$t] as $vertex) {
                if (strval($t) == strval($destination) && $path[$vertex]) {
                    $new_path = clone $path[$vertex];
                    $new_path->push($t);
                    $dest_paths[] = $new_path;
                }
                
                if (!$this->visited[$vertex]) {
                  // if not yet visited, enqueue vertex and mark as visited
                    $q->enqueue($vertex);
                    $this->visited[$vertex] = true;
                  
                  // add vertex to current path
                    $path[$vertex] = clone $path[$t];
                    $path[$vertex]->push($vertex);
                }
            }
        }
        
        $paths = [];
        foreach ($dest_paths as $p) {
            $ps = [];
            foreach ($p as $vertex) {
                $ps[] = $vertex;
            }
            $paths[] = $ps;
        }
        return $paths;
    }
}
