<?php

namespace idoit\Module\Cable\Algorithms;

require_once('IAlgorithm.php'); // needed for PHPUnit

/**
 * Dijkstra algorithm class.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Dijkstra implements IAlgorithm
{

    /**
     * Graph.
     *
     * @var array
     */
    protected $graph;

    /**
     * Constructor.
     *
     * @param array $graph

        Structure of graph:

        $graph = array(
            array('A', 'B', 3),
            array('A', 'C', 6),
            array('B', 'C', 3),
            array('B', 'F', 1),
            array('C', 'D', 3),
            array('D', 'F', 2),
        );

     */
    public function __construct($graph)
    {
        $this->graph = $graph;
    }

    /**
     * Get shortest path.
     *
     * Thanks to: https://rosettacode.org/wiki/Dijkstra%27s_algorithm#PHP
     *
     * @param int $source
     * @param int $target
     * @return array
     */
    public function calculate($source, $target)
    {
        $vertices = array();
        $neighbours = array();
        foreach ($this->graph as $edge) {
            array_push($vertices, $edge[0], $edge[1]);
            $neighbours[$edge[0]][] = array("end" => $edge[1], "cost" => $edge[2]);
            $neighbours[$edge[1]][] = array("end" => $edge[0], "cost" => $edge[2]);
        }
        $vertices = array_unique($vertices);

        foreach ($vertices as $vertex) {
            $dist[$vertex] = INF;
            $previous[$vertex] = null;
        }

        $dist[$source] = 0;
        $Q = $vertices;
        while (count($Q) > 0) {
            $min = INF;
            foreach ($Q as $vertex) {
                if ($dist[$vertex] < $min) {
                    $min = $dist[$vertex];
                    $u = $vertex;
                }
            }

            $Q = array_diff($Q, array($u));
            if ($dist[$u] == INF or $u == $target) {
                break;
            }

            if (isset($neighbours[$u])) {
                foreach ($neighbours[$u] as $arr) {
                    $alt = $dist[$u] + $arr["cost"];
                    if ($alt < $dist[$arr["end"]]) {
                        $dist[$arr["end"]] = $alt;
                        $previous[$arr["end"]] = $u;
                    }
                }
            }
        }
        $path = array();
        $u = $target;
        while (isset($previous[$u])) {
            array_unshift($path, $u);
            $u = $previous[$u];
        }
        array_unshift($path, $u);
        return $path;
    }
}
