<?php

namespace idoit\Module\Cable\DependentObjectManager;

use Exception;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_cmdb_dao_smview_cable_abstract;
use isys_component_list;
use isys_component_template_navbar;
use UnexpectedValueException;

/**
 * Abstract Manager class.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class AbstractManager
{
    
    /**
     * DAO.
     *
     * @var isys_cmdb_dao_smview_cable_abstract
     */
    protected $_dao;
    
    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping;

    /**
     * Process listing.
     */
    abstract public function processListing();

    /**
     * Process listing.
     *
     * @param  string $content_title
     * @param  array $l_list_headers
     * @param  array|null $objects
     * @throws Exception
     * @global array $index_includes
     */
    protected function _processListing($content_title, $l_list_headers, $objects = null)
    {
        global $index_includes;

        $l_list = new isys_component_list();
        if (! $objects) {
            $objects = $this->_dao->get_objects();
        }
        $l_list->set_data($objects);

        $get = $_GET;
        unset($get['instance_id']); // remove parameter instance ID from URL
        $l_list->config(
            $l_list_headers,
            isys_glob_build_url(http_build_query($get) . "&instance_id=[{id}]"),
            "[{id}]"
        );
        $l_list->set_row_modifier($this);

        $lang = isys_application::instance()->container->get('language');
        isys_component_template_navbar::getInstance()
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__NEW'))
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::DELETE, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__DELETE'))
            ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__NEW'))
            ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__DELETE'))
            ->add_onclick_prepend(
                defined_or_default('C__NAVBAR_BUTTON__DELETE'),
                'if (! confirm(\'' .
                $lang->get('LC__CMDB__CATG__CABLE__TREE_DELETE_OBJECT') . '\')) {return false;}'
            );

        if ($l_list->createTempTable()) {
            isys_application::instance()->template->assign(
                "objectTableList",
                $l_list->getTempTableHtml()
            );
        }

        isys_application::instance()->template->assign('content_title', $content_title)
            ->smarty_tom_add_rule("tom.content.bottom.buttons.*.p_bInvisible=1");
        $index_includes['contentbottomcontent'] = "content/bottom/content/object_table_list.tpl";
    }
    
    /**
     * Process new page.
     *
     * @param array $additional_fields
     */
    public function processNewPage($additional_fields = [])
    {
        foreach ($additional_fields as $field => $data) {
            isys_application::instance()->template->assign($field, $data);
        }
        $l_rules = $this->_add_listing_rules([]);
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }
    
    /**
     * Process page.
     *
     * @param int   $element_id
     * @param array $additional_fields
     */
    public function processPage($element_id, $additional_fields = [])
    {
        $object = $this->_dao->get_object('id', $element_id);
        $l_rules = [];
        foreach ($this->_mapping as $db => $field) {
            $l_rules[$field]['p_strValue'] = $object[$db];
        }
        foreach ($additional_fields as $field => $data) {
            isys_application::instance()->template->assign($field, $data);
        }
        
        $l_all_rules = $this->_add_listing_rules($l_rules);
        $l_navbar = isys_component_template_navbar::getInstance();
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $l_navbar->set_selected(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_CABLE"),
                    defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
        } else {
            $lang = isys_application::instance()->container->get('language');
            $l_navbar->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::DELETE, "MODULE/C__MODULE__SMVIEW_CABLE"),
                    defined_or_default('C__NAVBAR_BUTTON__DELETE'))
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__DELETE'))
                ->add_onclick_prepend(
                    defined_or_default('C__NAVBAR_BUTTON__DELETE'),
                    'if (! confirm(\'' .
                    $lang->get('LC__CMDB__CATG__CABLE__TREE_DELETE_OBJECT') . '\')) {return false;}'
                );
        }

        isys_application::instance()->template
            ->assign("instance_id", $element_id)
            ->smarty_tom_add_rules("tom.content.bottom.content", $l_all_rules);
    }
    
    /**
     * Add list data to rules array.
     *
     * @param  array $l_rules
     * @return array
     */
    protected function _add_listing_rules($l_rules)
    {
        return $l_rules;
    }
    
    /**
     * Handle creation of new objects.
     */
    public function handleNewMode()
    {
        isys_component_template_navbar::getInstance()
            ->set_selected(isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__NEW'));
        $_POST[C__GET__NAVMODE] = defined_or_default('C__NAVMODE__EDIT');
    }

    /**
     * Save object.
     *
     * @param int $element_id
     * @param array $optionalParams
     * @param array|null $data
     */
    public function saveObject($element_id = null, $optionalParams = [], $data = null)
    {
        isys_component_template_navbar::getInstance()
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::DELETE, "MODULE/C__MODULE__SMVIEW_CABLE"),
                defined_or_default('C__NAVBAR_BUTTON__DELETE'));
        
        $params = [];
        $optParams = [];
        foreach ($this->_mapping as $db => $field) {
            $value = isset($_POST[$field]) ? $_POST[$field] : null;
            $value = isset($data[$field]) ? $data[$field] : $value;
            if (in_array($db, $optionalParams) && ! empty($value)) {
                $optParams[$db] = $value;
            } else {
                $params[$db] = $value;
            }
        }
        
        try {
            if (! isset($element_id) || $element_id == "") {
                $element_id = $this->_dao->create_object($params, $optParams);
            } else {
                $this->_dao->update_object($element_id, $params, $optParams);
            }
        } catch (UnexpectedValueException $e) {
            $_POST["navMode"] = defined_or_default('C__NAVMODE__EDIT');
        }
        
        $l_rules = [];
        foreach ($this->_mapping as $db => $field) {
            $value = isset($optParams[$db]) ? $optParams[$db] : $params[$db];
            $l_rules[$field]['p_strValue'] = $value;
        }

        isys_application::instance()->template->assign("instance_id", $element_id)
            ->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }
    
    /**
     * Delete object.
     *
     * @param int $element_id
     */
    public function deleteObject($element_id = null)
    {
        if (! isys_auth_auth::instance()->is_allowed_to(isys_auth::DELETE, "MODULE/C__MODULE__SMVIEW_CABLE")) {
            return;
        }

        try {
            if (isset($element_id)) {
                $this->_dao->delete_object($element_id);
            }
        } catch (Exception $e) {
            // handle error?
        }
    }
}
