<?php

namespace idoit\Module\Cable\DependentObjectManager;

use isys_application;

/**
 * Cable Type Manager class.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class CableTypeManager extends AbstractManager
{
    
    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'description'   => 'C__CATG__SMVIEW_CABLE_DESCRIPTION',
        'wire_count'    => 'C__CATG__SMVIEW_CABLE_WIRE_COUNT',
        'technology_id' => 'C__CATG__SMVIEW_CABLE_TECHNOLOGY',
    );
    
    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_dao = new \isys_cmdb_dao_smview_cable_type(\isys_application::instance()->database);
    }
    
    /**
     * Process generator listing.
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $title = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__CABLE_TYPES');
        $headers = [
            "id"          => "ID",
            "description" => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__DESCRIPTION'),
            "wire_count"  => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__WIRE_COUNT'),
            "technology"  => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__TECHNOLOGY'),
        ];
        $this->_processListing($title, $headers);
    }
    
    /**
     * Process page.
     *
     * @param int   $element_id
     * @param array $additional_fields
     */
    public function processPage($element_id, $additional_fields = [])
    {
        parent::processPage($element_id, $additional_fields);
        
        $l_rules = [];
        $type = $this->_dao->get_object('id', $element_id);
        foreach ($this->_mapping as $db => $field) {
            $l_rules[$field]['p_strValue'] = $type[$db];
        }
        $l_rules['C__CATG__SMVIEW_CABLE_TECHNOLOGY']['p_strSelectedID'] = $type['technology_id'];
        \isys_application::instance()->template
            ->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }
    
    /**
     * Add list data to rules array.
     *
     * @param  array $l_rules
     * @return array
     */
    protected function _add_listing_rules($l_rules)
    {
        $dao = new \isys_cmdb_dao_smview_cable_technology(\isys_application::instance()->database);
        $res = $dao->get_data();
        $types = [];
        foreach ($res as $type) {
            $description = $type['description'] . ' (';
            $description .= $type['technology'] . ')';
            $types[$type['id']] = $description;
        }
        
        $l_rules['C__CATG__SMVIEW_CABLE_TECHNOLOGY']['p_arData'] = serialize($types);
        return $l_rules;
    }
}
