<?php

namespace idoit\Module\Cable\DependentObjectManager;

use isys_application;
use isys_cmdb_dao_smview_cable_generator;
use isys_cmdb_dao_smview_cable_generator_algorithm;

/**
 * Generator Manager class.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class GeneratorManager extends AbstractManager {
    
    /**
     * Property mapping.
     * 
     * @var array
     */
    protected $_mapping = array(
        'title'       => 'C__CATG__SMVIEW_CABLE_TITLE',
        'description' => 'C__CATG__SMVIEW_CABLE_DESCRIPTION',
        'mask'        => 'C__CATG__SMVIEW_CABLE_MASK',
        'counter_min' => 'C__CATG__SMVIEW_CABLE_COUNTER_MIN',
        'counter_max' => 'C__CATG__SMVIEW_CABLE_COUNTER_MAX',
    );
    
    /**
     * Constructor.
     */
    public function __construct() {
        $this->_dao = new isys_cmdb_dao_smview_cable_generator(isys_application::instance()->database);
    }
    
    /**
     * Process generator listing.
     */
    public function processListing() {
        $lang = isys_application::instance()->container->get('language');
        $title = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR_GENERATORS');
        $headers = [
            "id"          => "ID",
            "title"       => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR_TITLE'),
            "mask"        => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR_MASK'),
            "counter_min" => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR_COUNTER_MIN'),
            "counter_max" => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__GENERATOR_COUNTER_MAX'),
        ];
        $this->_processListing($title, $headers);
    }
    
    /**
     * Process page.
     * 
     * @param int   $element_id
     * @param array $additional_fields
     */
    public function processPage($element_id, $additional_fields = []) {
        $genDao = new isys_cmdb_dao_smview_cable_generator(isys_application::instance()->database);
        $algos = $genDao->get_assigned_algorithms($element_id);
        $additional_fields['C__CATG__SMVIEW_CABLE_ALGORITHMS'] = json_encode($algos);
        parent::processPage($element_id, $additional_fields);
    }

    /**
     * Save object.
     * 
     * @param int   $type_id
     * @param array $optionalParams
     */
    public function saveObject($type_id = null, $optionalParams = [], $data = null) {
        parent::saveObject($type_id, $optionalParams, $data);
        
        // hack to get object ID
        $id = isys_application::instance()->template->getTemplateVars("instance_id");
        $this->_handleAlgorithms($id);
    }
    
    /**
     * Handle generator algorithms.
     * 
     * @param int $id
     */
    private function _handleAlgorithms($id) {
        $json_algos = $_POST['C__CATG__SMVIEW_CABLE_ALGORITHMS'];
        $algos = json_decode($json_algos);
        $dao = isys_cmdb_dao_smview_cable_generator_algorithm::instance(isys_application::instance()->database);
        
        // validate algorithm positions
        $duplicatePositions = [];
        foreach ($algos as $algo) {
            if (in_array($algo->pos, $duplicatePositions)) {
                // because we have no error handling, we can only
                // do nothing and return
                return;
            }
            $duplicatePositions[] = $algo->pos;
        }
        
        // remove current elements
        $current_elements = $dao->filter_objects(array('generator_id' => $id));
        foreach ($current_elements as $element) {
            $dao->delete_object($element['id']);
        }
        
        // create new elements
        foreach ($algos as $algo) {
            $params = array(
                'generator_id'  => $id,
                'index'         => $algo->index,
                'mask_position' => $algo->pos,
            );
            $optional_params = array(
                'type'        => isset($algo->type) ? $algo->type : null,
                'description' => isset($algo->description) ? $algo->description : null,
                'length'      => isset($algo->length) ? $algo->length : null,
                'increment'   => 1,
                'elements'    => isset($algo->elements) ? $algo->elements : null,
            );
            $dao->create_object($params, $optional_params);
        }
    }
}
