<?php

namespace idoit\Module\Cable\DependentObjectManager;

use Exception;
use isys_application;
use isys_cmdb_dao_smview_cable_route_usage;
use isys_exception_database;

/**
 * Route Usage Manager class.
 *
 * @package    i-doit
 * @subpackage smview_cable
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class RouteUsageManager extends AbstractManager
{

    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'name'          => 'C__CATG__SMVIEW_ROUTE_USAGE_NAME',
        'description'   => 'C__CATG__SMVIEW_ROUTE_USAGE_DESCRIPTION',
        'can_be_rented' => 'C__CATG__SMVIEW_ROUTE_USAGE_CAN_BE_RENTED',
        'technology_id' => 'C__CATG__SMVIEW_CABLE_TECHNOLOGY',
    );

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_dao = new isys_cmdb_dao_smview_cable_route_usage(isys_application::instance()->database);
    }

    /**
     * Process generator listing.
     *
     * @throws Exception
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $title = $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_USAGES');
        $headers = [
            "id"            => "ID",
            "name"          => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_USAGE_NAME'),
            "description"   => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_USAGE_DESCRIPTION'),
            "can_be_rented" => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__ROUTE_USAGE_CAN_BE_RENTED'),
            "technology"    => $lang->get('LC__CMDB__CATG__SMVIEW_CABLE__TECHNOLOGY'),
        ];

        // modify object list and update property text
        $objects = $this->_dao->get_objects();
        $modObjects = [];
        foreach ($objects as $i => $o) {
            switch ($o['can_be_rented']) {
                case '1': $o['can_be_rented'] = $lang->get('LC__UNIVERSAL__YES'); break;
                case '0': $o['can_be_rented'] = $lang->get('LC__UNIVERSAL__NO'); break;
                default: $o['can_be_rented'] = '';
            }
            $modObjects[$i] = $o;
        }

        $this->_processListing($title, $headers, $modObjects);
    }

    /**
     * Process page.
     *
     * @param  int   $element_id
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processPage($element_id, $additional_fields = [])
    {
        parent::processPage($element_id, $additional_fields);

        $l_rules = [];
        $type = $this->_dao->get_object('id', $element_id);
        $l_rules['C__CATG__SMVIEW_CABLE_TECHNOLOGY']['p_strSelectedID'] = $type['technology_id'];
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Add list data to rules array.
     *
     * @param  array $l_rules
     * @return array
     * @throws isys_exception_database
     */
    protected function _add_listing_rules($l_rules)
    {
        $id = (int) $l_rules['C__CATG__SMVIEW_ROUTE_USAGE_CAN_BE_RENTED']['p_strValue'];
        if ($id > -1) {
            $l_rules['C__CATG__SMVIEW_ROUTE_USAGE_CAN_BE_RENTED']['p_strSelectedID'] = $id;
        }
        $l_rules['C__CATG__SMVIEW_ROUTE_USAGE_CAN_BE_RENTED']['p_arData'] = get_smarty_arr_YES_NO();

        $dao = new \isys_cmdb_dao_smview_cable_technology(isys_application::instance()->database);
        $res = $dao->get_data();
        $types = [];
        foreach ($res as $type) {
            $description = $type['description'] . ' (';
            $description .= $type['technology'] . ')';
            $types[$type['id']] = $description;
        }

        $l_rules['C__CATG__SMVIEW_CABLE_TECHNOLOGY']['p_arData'] = serialize($types);
        return $l_rules;
    }
}
